@@ -7,15 +7,16 @@
 
 for f in ${1+"$@"} ; do
     case "x$f" in
-    x*.fsg) ;;
+    x*.fsg) EXT=fsg ;;
+    x*.fsm) EXT=fsm ;;
     *) echo "not an fsg file: $f" >&2 ; exit 1 ;;
     esac
-    n=`basename "$f" .fsg`
+    n=`basename "$f" ."$EXT"`
     mkdir "$n" || exit $?
     cp "$f" "$n"
     (cd "$n" ;\
-        unzip "$n.fsg" 1> /dev/null ;\
-        rm -f "$n.fsg" ;\
+        unzip "$n.$EXT" 1> /dev/null ;\
+        rm -f "$n.$EXT" ;\
         for s in *.xml ; do xml_pp -i "$s" 2> /dev/null ; done) || exit $?
 done
 exit 0
