@@ -27,6 +27,7 @@ import java.awt.Container;
 import java.awt.Dimension;
 import java.awt.Graphics;
 import java.awt.GraphicsDevice;
+import java.awt.Insets;
 import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
@@ -50,10 +51,13 @@ import javax.swing.JInternalFrame;
 import javax.swing.JLabel;
 import javax.swing.JLayeredPane;
 import javax.swing.JMenuBar;
+import javax.swing.JRootPane;
 import javax.swing.SwingUtilities;
 import javax.swing.Timer;
 import javax.swing.UIManager;
 import javax.swing.border.EmptyBorder;
+import javax.swing.event.InternalFrameAdapter;
+import javax.swing.event.InternalFrameEvent;
 import javax.swing.plaf.basic.BasicInternalFrameUI;
 
 import net.sf.freecol.client.ClientOptions;
@@ -68,9 +72,12 @@ import net.sf.freecol.client.gui.menu.MapEditorMenuBar;
 import net.sf.freecol.client.gui.menu.MenuMouseMotionListener;
 // Special case panels, TODO: can we move these to Widgets?
 import net.sf.freecol.client.gui.panel.FreeColPanel;
+import net.sf.freecol.client.gui.panel.InfoPanel;
 import net.sf.freecol.client.gui.panel.MainPanel;
 import net.sf.freecol.client.gui.panel.MapControls;
+import net.sf.freecol.client.gui.panel.MapEditorToolboxPanel;
 import net.sf.freecol.client.gui.panel.MapEditorTransformPanel;
+import net.sf.freecol.client.gui.panel.MiniMapFreeColPanel;
 import net.sf.freecol.client.gui.panel.Utility;
 import net.sf.freecol.client.gui.video.VideoComponent;
 import net.sf.freecol.client.gui.video.VideoListener;
@@ -227,7 +234,7 @@ public final class Canvas extends JDesktopPane {
         setFocusable(true);
         setFocusTraversalKeysEnabled(false);
         // Create key bindings for all actions
-        for (Option option : freeColClient.getActionManager().getOptions()) {
+        for (Option<?> option : freeColClient.getActionManager().getOptions()) {
             FreeColAction action = (FreeColAction)option;
             getInputMap().put(action.getAccelerator(), action.getId());
             getActionMap().put(action.getId(), action);
@@ -239,21 +246,39 @@ public final class Canvas extends JDesktopPane {
         revalidate();
         repaint();
         
-        /*
-         * TODO: Stop using hardcoded 125ms for animation. Instead, we should
-         *       define timing for every terrain animation.
-         */
         animationTimer = new Timer(125, (event) -> {
             paintJustTheMapImmediately();
         });
+        updateRepaintTimer(false);
         this.animationTimer.start();
         
         logger.info("Canvas created with bounds: " + windowBounds);
     }
 
+    
+    /**
+     * Updates the timing for repaints.
+     * 
+     * @param fastRendering If {@code true}, the repaints should happen as often
+     *      as possible. This is used for smooth scrolling.
+     */
+    public void updateRepaintTimer(boolean fastRendering) {
+        /*
+         * TODO: Stop using hardcoded 125ms for animation. Instead, we should
+         *       define timing for every terrain animation.
+         */
+        if (fastRendering) {
+            animationTimer.setDelay(4);
+        } else {
+            animationTimer.setDelay(125);
+        }
+    }
+    
     // Internals
     
     private void updateSize() {
+        freeColClient.getGUI().refreshScaleFactorIfNecessary();
+        
         Dimension size = getSize();
         if (oldSize.width != size.width || oldSize.height != size.height) {
             logger.info("Canvas resize from " + oldSize + " to " + size);
@@ -263,12 +288,34 @@ public final class Canvas extends JDesktopPane {
                 freeColClient.getGUI().updateMapControls();
                 addMapControls();
             }
+            if (this.mapControls != null) {
+                this.mapControls.updateMinimap();
+            }
             updateFrameSizesAndPositions(size);
             
             revalidate();
             repaint();
         }
     }
+    
+    public void refreshAllFrames() {
+        final Component[] components = getComponents();
+        for (Component c : components) {
+            if (!(c instanceof JInternalFrame)) {
+                continue;
+            }
+            
+            /*
+             * Determines a new sensible size.
+             */
+            final JInternalFrame f = (JInternalFrame) c;
+            final FreeColPanel panel = getFreeColPanel(f);
+            if (panel != null) {
+                panel.refreshLayout();
+            }
+        }
+        updateFrameSizesAndPositions(getSize());
+    }
 
     private void updateFrameSizesAndPositions(Dimension canvasSize) {
         for (Component c : getComponents()) {
@@ -280,6 +327,13 @@ public final class Canvas extends JDesktopPane {
              * Determines a new sensible size.
              */
             final JInternalFrame f = (JInternalFrame) c;
+            final FreeColPanel panel = getFreeColPanel(f);
+            if (panel != null && panel.isFullscreen()) {
+                f.setSize(canvasSize);
+                f.setLocation(0, 0);
+                continue;
+            }
+            
             final Boolean resizeable = (Boolean) f.getClientProperty(PROPERTY_RESIZABLE);
             final Dimension newSize;
             if (resizeable != null && resizeable) {
@@ -308,11 +362,24 @@ public final class Canvas extends JDesktopPane {
             f.setLocation(p);
         }
     }
+
+
+    private FreeColPanel getFreeColPanel(final JInternalFrame f) {
+        if (f.getContentPane() == null) {
+            return null;
+        }
+        if (f.getContentPane().getComponentCount() <= 0) {
+            return null;
+        }
+        if (!(f.getContentPane().getComponent(0) instanceof FreeColPanel)) {
+            return null;
+        }
+        return (FreeColPanel) f.getContentPane().getComponent(0);
+    }
     
     private Dimension capSizeToMaximum(JInternalFrame f, Dimension maxSize) {
-        final int FRAME_EMPTY_SPACE = 60;
-        final int width = Math.max(f.getMinimumSize().width, Math.min(f.getWidth(), maxSize.width - FRAME_EMPTY_SPACE));
-        final int height = Math.max(f.getMinimumSize().height, Math.min(f.getHeight(), maxSize.height - FRAME_EMPTY_SPACE));
+        final int width = Math.max(f.getMinimumSize().width, Math.min(f.getWidth(), maxSize.width));
+        final int height = Math.max(f.getMinimumSize().height, Math.min(f.getHeight(), maxSize.height));
         return new Dimension(width, height);
     }
 
@@ -395,8 +462,32 @@ public final class Canvas extends JDesktopPane {
     private JInternalFrame addAsFrame(JComponent comp, boolean toolBox,
                                       PopupPosition popupPosition,
                                       boolean resizable) {
-        final JInternalFrame f = (toolBox) ? new ToolBoxFrame()
-            : new JInternalFrame();
+        @SuppressWarnings("serial")
+        final JInternalFrame f = (toolBox) ? new ToolBoxFrame() : new JInternalFrame() {
+            @Override
+            public Dimension getMinimumSize() {
+                final Dimension innerDimension = comp.getMinimumSize();
+                final Insets insets = getInsets();
+                return new Dimension(innerDimension.width + insets.left + insets.right,
+                        innerDimension.height + insets.top + insets.bottom);
+            }
+        };
+        
+        final boolean fullscreenPanel;
+        if (comp instanceof FreeColPanel) {
+            final FreeColPanel freeColPanel = (FreeColPanel) comp;
+            fullscreenPanel = freeColPanel.isFullscreen();
+            if (popupPosition == null) {
+                popupPosition = freeColPanel.getFramePopupPosition();
+            }
+        } else {
+            fullscreenPanel = false;
+        }
+        
+        if (fullscreenPanel) {
+            resizable = false;
+        }
+        
         Container con = f.getContentPane();
         if (con instanceof JComponent) {
             JComponent c = (JComponent)con;
@@ -404,45 +495,87 @@ public final class Canvas extends JDesktopPane {
             c.setBorder(null);
         }
 
-        if (comp.getBorder() != null) {
-            if (comp.getBorder() instanceof EmptyBorder) {
-                f.setBorder(Utility.blankBorder(10, 10, 10, 10));
+        if (!toolBox) {
+            if (comp instanceof FreeColPanel
+                    && ((FreeColPanel) comp).getFrameBorder() != null) {
+                f.setBorder(((FreeColPanel) comp).getFrameBorder());
+            } else if (comp.getBorder() != null) {
+                if (comp.getBorder() instanceof EmptyBorder) {
+                    f.setBorder(Utility.blankBorder(10, 10, 10, 10));
+                } else {
+                    f.setBorder(comp.getBorder());
+                    comp.setBorder(Utility.blankBorder(5, 5, 5, 5));
+                }
             } else {
-                f.setBorder(comp.getBorder());
-                comp.setBorder(Utility.blankBorder(5, 5, 5, 5));
+                f.setBorder(null);
+            }
+        }
+        if (comp instanceof FreeColPanel) {
+            final FreeColPanel panel = (FreeColPanel) comp;
+            final String title = panel.getFrameTitle();
+            if (title != null) {
+                f.setTitle(panel.getFrameTitle());
             }
-        } else {
-            f.setBorder(null);
         }
 
-        final FrameMotionListener fml = new FrameMotionListener(f);
-        comp.addMouseMotionListener(fml);
-        comp.addMouseListener(fml);
-        if (f.getUI() instanceof BasicInternalFrameUI) {
+        if (!fullscreenPanel) {
+            final FrameMotionListener fml = new FrameMotionListener(f);
+            comp.addMouseMotionListener(fml);
+            comp.addMouseListener(fml);
+        }
+        
+        if (!toolBox && f.getUI() instanceof BasicInternalFrameUI) {
+            f.getRootPane().setWindowDecorationStyle(JRootPane.NONE);
             BasicInternalFrameUI biu = (BasicInternalFrameUI) f.getUI();
             biu.setNorthPane(null);
             biu.setSouthPane(null);
             biu.setWestPane(null);
             biu.setEastPane(null);
+        } else {
+            f.setMaximizable(true);
+            f.setClosable(true);
+            f.setIconifiable(true);
         }
+        
+        f.addInternalFrameListener(new InternalFrameAdapter() {
+            @Override
+            public void internalFrameClosing(InternalFrameEvent e) {
+                savePositionAndSize(comp, f);
+                if (comp instanceof FreeColPanel) {
+                    ((FreeColPanel) comp).onFrameClosing();
+                }
+            }
+        });
 
         f.getContentPane().add(comp);
         f.setOpaque(false);
         f.pack();
-        
-        final Dimension size = capSizeToMaximum(f, getSize());
+
+        final Dimension size;
+        if (fullscreenPanel) {
+            size = getSize();
+        } else {
+            size = capSizeToMaximum(f, getSize());
+        }
         f.setSize(size);
         
-        final Point p = chooseLocation(comp, size.width, size.height, popupPosition);
-        final Point adjustedP = adjustLocationForClearSpace(p, size.width, size.height);
-        if (popupPosition == PopupPosition.CENTERED_FORCED || p.equals(adjustedP)) {
-            f.setLocation(p);
-            f.putClientProperty(PROPERTY_POPUP_POSITION, popupPosition);
-            f.putClientProperty(PROPERTY_RESIZABLE, resizable);
+        if (fullscreenPanel) {
+            f.setLocation(0, 0);
         } else {
-            f.setLocation(adjustedP);
+            final Point p = chooseLocation(comp, size.width, size.height, popupPosition);
+            final Point adjustedP = adjustLocationForClearSpace(p, size.width, size.height);
+            
+            if (popupPosition == PopupPosition.CENTERED_FORCED || p.equals(adjustedP)) {
+                f.setLocation(p);
+                f.putClientProperty(PROPERTY_POPUP_POSITION, popupPosition);
+                f.putClientProperty(PROPERTY_RESIZABLE, resizable);
+            } else {
+                f.setLocation(adjustedP);
+            }
         }
-        this.addToCanvas(f, MODAL_LAYER);
+        
+        final Integer layer = MODAL_LAYER;
+        this.addToCanvas(f, layer);
         f.setName(comp.getClass().getSimpleName());
 
         f.setFrameIcon(null);
@@ -566,6 +699,27 @@ public final class Canvas extends JDesktopPane {
                 x = ((getWidth() - width) * 3) / 4;
                 y = (getHeight() - height) / 2;
                 break;
+            case LEFT:
+                x = 0;
+                y = (getHeight() - height) / 2;
+                break;
+            case LOWER_LEFT:
+                x = 0;
+                y = getHeight() - height;
+                break;
+            case LOWER_RIGHT:
+                x = getWidth() - width;
+                y = getHeight() - height;
+                break;
+            case UPPER_RIGHT:
+                x = getWidth() - width;
+                y = 0;
+                break;
+            case RIGHT:
+                x = getWidth() - width;
+                y = (getHeight() - height) / 2;
+                break;
+            case UPPER_LEFT:
             case ORIGIN:
                 x = y = 0;
                 break;
@@ -778,13 +932,25 @@ public final class Canvas extends JDesktopPane {
      */
     private void notifyClose(Component c, JInternalFrame jif) {
         if (c instanceof FreeColPanel) {
-            FreeColPanel fcp = (FreeColPanel)c;
+            FreeColPanel fcp = (FreeColPanel) c;
             fcp.firePropertyChange("closing", false, true);
-            
+
+            if (nothingShowing()
+                    // These are added/removed as a part of "updateMenuBar"
+                    && !(c instanceof InfoPanel) && !(c instanceof MiniMapFreeColPanel)) {
+                updateMenuBar();
+            }
+        }
+    }
+    
+    private void savePositionAndSize(Component c, JInternalFrame jif) {
+        if (c instanceof FreeColPanel) {
+            FreeColPanel fcp = (FreeColPanel) c;
+            if (fcp.isFullscreen()) {
+                return;
+            }
             savePosition(fcp, jif.getLocation());
             saveSize(fcp, fcp.getSize());
-
-            if (nothingShowing()) updateMenuBar();
         }
     }
 
@@ -798,6 +964,26 @@ public final class Canvas extends JDesktopPane {
      */
     public void removeInGameComponents() {
         // remove listeners, they will be added when launching the new game...
+        removeKeyAndMouseListeners();
+
+        for (JInternalFrame f : getAllFrames()) {
+            final FreeColPanel fcp = getFreeColPanel(f);
+            if (fcp != null) {
+                removeFromCanvas(fcp);
+            } else {
+                removeFromCanvas(f);
+            }
+        }
+        
+        for (Component c : getComponents()) {
+            if (c instanceof CanvasMapViewer) {
+                continue;
+            }
+            removeFromCanvas(c);
+        }
+    }
+
+    private void removeKeyAndMouseListeners() {
         KeyListener[] keyListeners = getKeyListeners();
         for (KeyListener keyListener : keyListeners) {
             removeKeyListener(keyListener);
@@ -812,19 +998,14 @@ public final class Canvas extends JDesktopPane {
         for (MouseMotionListener mouseMotionListener : mouseMotionListeners) {
             removeMouseMotionListener(mouseMotionListener);
         }
-
-        for (Component c : getComponents()) {
-            if (c instanceof CanvasMapViewer) {
-                continue;
-            }
-            removeFromCanvas(c);
-        }
     }
 
     /**
      * Map editor initialization.
      */
     public void startMapEditorGUI() {
+        removeKeyAndMouseListeners();
+        
         freeColClient.updateActions();
         this.parentFrame.setMenuBar(new MapEditorMenuBar(this.freeColClient,
                 new MenuMouseMotionListener(scrolling)));
@@ -838,6 +1019,8 @@ public final class Canvas extends JDesktopPane {
      * In game initializations.
      */
     public void initializeInGame() {
+        removeKeyAndMouseListeners();
+        
         this.parentFrame.setMenuBar(new InGameMenuBar(this.freeColClient,
                 new MenuMouseMotionListener(scrolling)));
         addMouseListener(new CanvasMouseListener(this.freeColClient));
@@ -861,6 +1044,10 @@ public final class Canvas extends JDesktopPane {
      * @param d The {@code Dimension} to use as default.
      */
     public void restoreSavedSize(Component comp, Dimension d) {
+        if (comp instanceof FreeColPanel && ((FreeColPanel) comp).isFullscreen()) {
+            return;
+        }
+        
         final Dimension pref = comp.getPreferredSize();
         final Dimension sugg = (d == null) ? pref : d;
         boolean save = false;
@@ -926,10 +1113,11 @@ public final class Canvas extends JDesktopPane {
         if (panel.endsWith("Panel")) {
             for (Component c1 : getComponents()) {
                 if (c1 instanceof JInternalFrame) {
-                    for (Component c2 : ((JInternalFrame)c1).getContentPane()
-                             .getComponents()) {
+                    final JInternalFrame jif = (JInternalFrame) c1;
+                    for (Component c2 : jif.getContentPane().getComponents()) {
                         if (panel.equals(c2.getClass().getName())) {
-                            notifyClose(c2, (JInternalFrame)c1);
+                            savePositionAndSize(c2, jif);
+                            notifyClose(c2, jif);
                             return;
                         }
                     }
@@ -993,6 +1181,13 @@ public final class Canvas extends JDesktopPane {
      *     requested type, or null if none found.
      */
     public <T extends FreeColPanel> T getExistingFreeColPanel(Class<T> type) {
+        for (JInternalFrame f : getAllFrames()) {
+            for (Component c2 : f.getContentPane().getComponents()) {
+                if (type.isAssignableFrom(c2.getClass())) {
+                    return type.cast(c2);
+                }
+            }
+        }
         for (Component c1 : getComponents()) {
             if (c1 instanceof JInternalFrame) {
                 for (Component c2 : ((JInternalFrame)c1).getContentPane()
@@ -1013,6 +1208,38 @@ public final class Canvas extends JDesktopPane {
         }
         return null;
     }
+    
+    public boolean isAddedAsPanelFrameOrIcon(Class<?> clazz) {
+        for (JInternalFrame f : getAllFrames()) {
+            for (Component c2 : f.getContentPane().getComponents()) {
+                if (clazz.isAssignableFrom(c2.getClass())) {
+                    return true;
+                }
+            }
+        }
+        for (Component c1 : getComponents()) {
+            if (clazz.isAssignableFrom(c1.getClass())) {
+                return true;
+            }
+        }
+        return false;
+    }
+    
+    public boolean isAddedAsPanelFrameOrIcon(Component component) {
+        for (JInternalFrame f : getAllFrames()) {
+            for (Component c2 : f.getContentPane().getComponents()) {
+                if (component == c2) {
+                    return true;
+                }
+            }
+        }
+        for (Component c1 : getComponents()) {
+            if (component == c1) {
+                return true;
+            }
+        }
+        return false;
+    }
 
     /**
      * Gets any currently displayed component matching a predicate.
@@ -1063,6 +1290,10 @@ public final class Canvas extends JDesktopPane {
 
         final Rectangle updateBounds = comp.getBounds();
         final JInternalFrame jif = getInternalFrame(comp);
+
+        if (jif != null) {
+            savePositionAndSize(comp, jif);
+        }
         if (jif != null && jif != comp) {
             jif.dispose();
         } else {
@@ -1099,7 +1330,7 @@ public final class Canvas extends JDesktopPane {
         dialogRemove(dialog);
         return response;
     }
-
+    
     /**
      * Displays a {@code FreeColPanel} at a generalized position.
      *
@@ -1110,10 +1341,29 @@ public final class Canvas extends JDesktopPane {
      * @return The panel.
      */
     public FreeColPanel showFreeColPanel(FreeColPanel panel,
-                                         PopupPosition popupPosition,
-                                         boolean resizable) {
+            PopupPosition popupPosition,
+            boolean resizable) {
+        return showFreeColPanel(panel, false, popupPosition, resizable);
+    }
+
+    /**
+     * Displays a {@code FreeColPanel} at a generalized position.
+     *
+     * @param panel {@code FreeColPanel}, panel to show
+     * @param toolBox Should be set to true if the resulting frame is
+     *     used as a toolbox (that is: it should not be counted as a
+     *     frame).
+     * @param popupPosition {@code PopupPosition} The generalized
+     *     position to place the panel.
+     * @param resizable Should the panel be resizable?
+     * @return The panel.
+     */
+    public FreeColPanel showFreeColPanel(FreeColPanel panel,
+            boolean toolBox,
+            PopupPosition popupPosition,
+            boolean resizable) {
         repaint();
-        addAsFrame(panel, false, popupPosition, resizable);
+        addAsFrame(panel, toolBox, popupPosition, resizable);
         panel.requestFocus();
         freeColClient.getActionManager().update();
         return panel;
@@ -1199,14 +1449,31 @@ public final class Canvas extends JDesktopPane {
      * @return True if any were added.
      */
     public boolean addMapControls() {
-        if (this.mapControls == null) return false;
-        List<Component> components
-            = this.mapControls.getComponentsToAdd(this.oldSize);
-        for (Component c : components) {
-            addToCanvas(c, JLayeredPane.MODAL_LAYER);
+        if (this.mapControls == null) {
+            return false;
+        }
+        final List<JComponent> components = this.mapControls.getComponentsToAdd(this.oldSize);
+        if (hasAtLeastOneMapControlFrom(components)) {
+            return false;
+        }
+        for (JComponent c : components) {
+            if (freeColClient.isMapEditor()) {
+                addAsFrame(c, true, null, true);
+            } else {
+                addToCanvas(c, JLayeredPane.MODAL_LAYER);
+            }
         }
         return !components.isEmpty();
     }
+    
+    private boolean hasAtLeastOneMapControlFrom(List<JComponent> components) {
+        for (JComponent c : components) {
+            if (isAddedAsPanelFrameOrIcon(c)) {
+                return true;
+            }
+        }
+        return false;
+    }
 
     /**
      * Remove the map controls.
@@ -1217,7 +1484,7 @@ public final class Canvas extends JDesktopPane {
         if (mapControls == null) {
             return false;
         }
-        final List<Component> components = mapControls.getComponentsPresent();
+        final List<JComponent> components = mapControls.getComponentsPresent();
         boolean ret = false;
         for (Component c : components) {
             removeFromCanvas(c);
@@ -1236,6 +1503,7 @@ public final class Canvas extends JDesktopPane {
     public void closeMenus() {
         for (JInternalFrame jif : getAllFrames()) {
             for (Component c : jif.getContentPane().getComponents()) {
+                savePositionAndSize(c, jif);
                 notifyClose(c, jif);
             }
             jif.dispose();
@@ -1427,14 +1695,19 @@ public final class Canvas extends JDesktopPane {
      * Display the map editor transform panel.
      */
     public void showMapEditorTransformPanel() {
-        MapEditorTransformPanel panel
-            = new MapEditorTransformPanel(this.freeColClient);
-        JInternalFrame f = addAsFrame(panel, true, PopupPosition.CENTERED,
-                                      false);
-        f.setLocation(f.getX(), 50); // move up to near the top
+        final MapEditorTransformPanel panel = new MapEditorTransformPanel(this.freeColClient);
+        addAsFrame(panel, true, null, true);
+        repaint();
+    }
+    
+    /**
+     * Display the map editor toolbox panel.
+     */
+    public void showMapEditorToolboxPanel() {
+        final MapEditorToolboxPanel panel = new MapEditorToolboxPanel(this.freeColClient);
+        addAsFrame(panel, true, null, true);
         repaint();
     }
-
 
     // Override JComponent
     public void paintJustTheMapImmediately() {
