@@ -23,6 +23,7 @@ import java.awt.Color;
 import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.Graphics2D;
+import java.awt.Point;
 import java.awt.event.ActionListener;
 import java.awt.image.BufferedImage;
 import java.io.File;
@@ -40,8 +41,11 @@ import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.FreeColClientHolder;
 import net.sf.freecol.client.control.MapTransform;
+import net.sf.freecol.client.gui.SwingGUI.PopupPosition;
 import net.sf.freecol.client.gui.dialog.FreeColDialog;
 import net.sf.freecol.client.gui.dialog.Parameters;
+import net.sf.freecol.client.gui.mapviewer.MapAsyncPainter;
+import net.sf.freecol.client.gui.mapviewer.MapViewer;
 import net.sf.freecol.client.gui.mapviewer.MapViewerRepaintManager;
 import net.sf.freecol.client.gui.panel.FreeColPanel;
 import net.sf.freecol.client.gui.panel.report.LabourData.UnitData;
@@ -1155,6 +1159,30 @@ public class GUI extends FreeColClientHolder {
      */
     public void startMapEditorGUI() {}
 
+    /**
+     * Shows the map editor toolbox panel.
+     */
+    public void enableEditorToolboxPanel(boolean shouldDisplayPanel) {}
+
+    /**
+     * Checks if the map editor toolbox panel is being displayed.
+     */
+    public boolean isShowingMapEditorToolboxPanel() {
+        return false;
+    }
+    
+    /**
+     * Shows the map editor transform panel.
+     */
+    public void enableEditorTransformPanel(boolean shouldDisplayPanel) {}
+    
+    /**
+     * Checks if the map editor transform panel is being displayed.
+     */
+    public boolean isShowingMapEditorTransformPanel() {
+        return false;
+    }
+    
 
     // Animation handling
 
@@ -1250,6 +1278,16 @@ public class GUI extends FreeColClientHolder {
     public Tile getFocus() {
         return null;
     }
+    
+    /**
+     * Gets the current focus of the visible map given in pixels.
+     * 
+     * @return The current focus map point that is between {@code 0}
+     *   and <code>map.getWidth() * tileBounds.getWidth()</code>.
+     */
+    public Point getFocusMapPoint() {
+        return null;
+    }
 
     /**
      * Set the current focus tile.
@@ -1261,7 +1299,16 @@ public class GUI extends FreeColClientHolder {
      * @param tile The new focus {@code Tile}.
      */
     public void setFocus(Tile tile) {}
-
+    
+    /**
+     * The current focus of the visible map given in pixels.
+     * 
+     * The maximum width of the map in pixels is:
+     * <code>map.getWidth() * tileBounds.getWidth()</code>.
+     * 
+     * @param pointToFocus The new focus point.
+     */
+    public void setFocusMapPoint(Point pointToFocus) {}
 
     // Path handling
 
@@ -1480,10 +1527,46 @@ public class GUI extends FreeColClientHolder {
      * Scroll the map in a given direction.
      *
      * @param direction The {@code Direction} to scroll.
+     * @param performRepaints If {@code true}, then repaints are performed
+     *      after scrolling.
      * @return True if scrolling can continue.
      */
-    public boolean scrollMap(Direction direction) { return false; }
-
+    public boolean scrollMap(Direction direction, boolean performRepaints) { return false; }
+    
+    /**
+     * Sets the scroll speed back to the initial value. This is needed
+     * when the scrolling stops, since the scrolling accelerates.
+     */
+    public void resetScrollSpeed() { }
+    
+    /**
+     * Paint the whole canvas now.
+     * 
+     * This should only be called for very special cases, like animations.
+     * Normally, use {@link #repaint()} instead.
+     */
+    public void paintImmediately() {}
+    
+    /**
+     * Enables asynchronous painting. That is, the painting of the
+     * {@link MapViewer} is performed in a thread other than the EDT
+     * (normal GUI thread).
+     * 
+     * This allows better performance and lower latency, but will sometimes
+     * produce visual artifacts when the game state changes during painting.
+     *    
+     * @see #stopMapAsyncPainter()
+     */
+    public MapAsyncPainter useMapAsyncPainter() {
+        return null;
+    }
+    
+    /**
+     * Stops asynchronous painting.
+     * 
+     * @see #useMapAsyncPainter()
+     */
+    public void stopMapAsyncPainter() {}
 
     // Tile image manipulation
 
@@ -1891,6 +1974,19 @@ public class GUI extends FreeColClientHolder {
      * @return The panel shown.
      */
     public FreeColPanel showColonyPanel(Colony colony, Unit unit) { return null; }
+    
+    /**
+     * Displays a {@code FreeColPanel} at a generalized position.
+     *
+     * @param panel {@code FreeColPanel}, panel to show
+     * @param toolBox Should be set to true if the resulting frame is
+     *     used as a toolbox (that is: it should not be counted as a
+     *     frame).
+     * @param popupPosition {@code PopupPosition} The generalized
+     *     position to place the panel.
+     * @param resizable Should the panel be resizable?
+     */
+    public void showFreeColPanel(FreeColPanel panel, boolean toolBox, PopupPosition popupPosition, boolean resizable) { }
 
     /**
      * Show a colopedia panel.
@@ -2501,9 +2597,9 @@ public class GUI extends FreeColClientHolder {
      * Run out of ColonyPanel, so the tile is already displayed.
      *
      * @param colony The {@code Colony} to display.
-     * @return The response returned by the dialog.
+     * @param dialogHandler A {@code DialogHandler} for the dialog response.
      */
-    public boolean showWarehouseDialog(Colony colony) { return false; }
+    public void showWarehouseDialog(Colony colony, DialogHandler<Boolean> dialogHandler) {  }
 
     /**
      * Show the production of a unit.
@@ -2543,4 +2639,11 @@ public class GUI extends FreeColClientHolder {
     public void emergencyPurge() {
 
     }
+
+    /**
+     * Updates the scaleFactor, if necessary, after the window has been resized. 
+     */
+    public void refreshScaleFactorIfNecessary() {
+
+    }
 }
