@@ -31,6 +31,7 @@ import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.Image;
 import java.awt.Point;
+import java.awt.RenderingHints;
 import java.awt.Toolkit;
 import java.awt.image.BufferedImage;
 import java.util.ArrayList;
@@ -43,6 +44,7 @@ import java.util.stream.Collectors;
 import javax.swing.ImageIcon;
 import javax.swing.JLabel;
 
+import net.sf.freecol.client.gui.images.ColorizedImageCreator;
 import net.sf.freecol.client.gui.images.ImageCreators;
 import net.sf.freecol.common.io.sza.SimpleZippedAnimation;
 import net.sf.freecol.common.model.Ability;
@@ -75,6 +77,7 @@ import net.sf.freecol.common.resources.ImageResource;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.resources.StringResource;
 import net.sf.freecol.common.resources.Video;
+import net.sf.freecol.common.util.ImageUtils;
 
 
 /**
@@ -396,22 +399,30 @@ public final class ImageLibrary {
      * @param goodsType The {@code GoodsType} to use.
      * @param amount The amount of goods.
      * @param location The {@code Location} for the goods.
+     * @param whiteForeground If the color should be white (or bright) so that
+     *     it works on a black background.
      * @return A suitable {@code Color}.
      */
     public static Color getGoodsColor(GoodsType goodsType, int amount,
-                                      Location location) {
+                                      Location location, boolean whiteForeground) {
+        final String foreground = (whiteForeground) ? "whiteForeground" : "foreground";
+        final String prefix = "color." + foreground + ".GoodsLabel.";
         final String key = (!goodsType.limitIgnored()
             && location instanceof Colony
             && ((Colony)location).getWarehouseCapacity() < amount)
-            ? "color.foreground.GoodsLabel.capacityExceeded"
+            ? prefix + "capacityExceeded"
             : (location instanceof Colony && goodsType.isStorable()
                 && ((Colony)location).getExportData(goodsType).getExported())
-            ? "color.foreground.GoodsLabel.exported"
+            ? prefix + "exported"
             : (amount == 0)
-            ? "color.foreground.GoodsLabel.zeroAmount"
+            ? prefix + "zeroAmount"
             : (amount < 0)
-            ? "color.foreground.GoodsLabel.negativeAmount"
-            : "color.foreground.GoodsLabel.positiveAmount";
+            ? prefix + "negativeAmount"
+            : prefix + "positiveAmount";
+        
+        if (whiteForeground) {
+            return getColor(key, Color.WHITE);
+        }
         return getColor(key, Color.BLACK);
     }
 
@@ -634,9 +645,13 @@ public final class ImageLibrary {
         return getUnscaledImage(key);
     }
 
-    public static BufferedImage getMenuBackground() {
+    public static BufferedImage getMenuBarBackground() {
         return getUnscaledImage("image.background.FreeColMenuBar");
     }
+    
+    public static BufferedImage getMenuBackground() {
+        return getUnscaledImage("image.background.FreeColMenu");
+    }
 
     public static BufferedImage getMiniMapBackground() {
         return getUnscaledImage("image.background.MiniMap");
@@ -796,6 +811,12 @@ public final class ImageLibrary {
             ? getBuildingTypeImage((BuildingType)buildable, size)
             : getUnitTypeImage((UnitType)buildable, size);
     }
+    
+    public BufferedImage getBuildableTypeImage(BuildableType buildable, Player player, Dimension size) {
+        return (buildable instanceof BuildingType)
+                ? getBuildingTypeImage((BuildingType)buildable, player, size)
+                        : getUnitTypeImage((UnitType)buildable, size);
+    }
 
     public BufferedImage getSmallBuildableTypeImage(BuildableType buildable,
                                                     Player player) {
@@ -804,6 +825,39 @@ public final class ImageLibrary {
             ? getScaledBuildingTypeImage((BuildingType)buildable, player, scale)
             : getUnitTypeImage((UnitType)buildable, scale);
     }
+    
+    public BufferedImage getSmallBuildableTypeImageWithWithSize(BuildableType buildable, Player player, Dimension maxSize) {
+        final BufferedImage image = getSmallBuildableTypeImageWithMaxSize(buildable, player, maxSize);
+        return ImageUtils.createCenteredImage(image, maxSize);
+    }
+    
+    public BufferedImage getSmallBuildableTypeImageWithMaxSize(BuildableType buildable, Player player, Dimension maxSize) {
+        final BufferedImage image = getSmallBuildableTypeImage(buildable, player);
+        final double widthProportion = image.getWidth() / maxSize.width;
+        final double heightProportion = image.getHeight() / maxSize.height;
+        if (widthProportion >= heightProportion) {
+            if (image.getWidth() > maxSize.width) {
+                return getBuildableTypeImage(buildable, player, new Dimension(maxSize.width, 0));
+            } else {
+                return image;
+            }
+        } else {
+            if (image.getHeight() > maxSize.height) {
+                return getBuildableTypeImage(buildable, player, new Dimension(0, maxSize.height));
+            } else {
+                return image;
+            }
+        }
+    }
+    
+    private BufferedImage getBuildingTypeImage(BuildingType buildingType, Player player, Dimension size) {
+        String key = getBuildingTypeKey(buildingType);
+        final String extraKey = key + "." + player.getNationResourceKey();
+        if (ResourceManager.getImageResource(extraKey, false) != null) {
+            key = extraKey;
+        }
+        return this.imageCache.getSizedImage(key, size, false);
+    }
 
     public BufferedImage getBuildingTypeImage(BuildingType buildingType,
                                               Dimension size) {
@@ -816,7 +870,7 @@ public final class ImageLibrary {
         final String key = getBuildingTypeKey(buildingType);
         return this.imageCache.getScaledImage(key, scale, false);
     }
-
+    
     private BufferedImage getScaledBuildingTypeImage(BuildingType buildingType,
                                                      Player player,
                                                      float scale) {
@@ -827,19 +881,185 @@ public final class ImageLibrary {
         }
         return this.imageCache.getScaledImage(key, scale, false);
     }
+    
+    public Dimension determineMaxSizeUsingSizeFromAllLevels(BuildingType buildingType, Player player) {        
+        int maxWidth = 0;
+        int maxHeight = 0;
+        while (buildingType.getUpgradesFrom() != null) {
+            buildingType = buildingType.getUpgradesFrom();
+        }
+        do {
+            final Image buildingImage = getScaledBuildingTypeImage(buildingType, player, getScaleFactor());
+            if (buildingImage.getWidth(null) > maxWidth) {
+                maxWidth = buildingImage.getWidth(null);
+            }
+            if (buildingImage.getHeight(null) > maxHeight) {
+                maxHeight = buildingImage.getHeight(null);
+            }
+            buildingType = buildingType.getUpgradesTo();
+        } while (buildingType != null);
+        
+        return new Dimension(maxWidth, maxHeight);
+    }
 
     public BufferedImage getScaledBuildingImage(Building building) {
+        if (building == null) {
+            return null;
+        }
         return getScaledBuildingTypeImage(building.getType(),
                                           building.getOwner(),
                                           this.scaleFactor);
     }
 
     public BufferedImage getSmallBuildingImage(Building building) {
+        if (building == null) {
+            return null;
+        }
         return getScaledBuildingTypeImage(building.getType(),
                                           building.getOwner(),
                                           this.scaleFactor * SMALL_SCALE);
     }
+    
+    public BufferedImage getScaledBuildingEmptyLandImage() {
+        final String key = "image.buildingicon.model.building.BuildingSite";
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    /**
+     * Gets an UNCACHED image of the "outside colony"-background based on the given defensive building.
+     * 
+     * The result have to be cached by the caller of this method as long as the image is needed.
+     * 
+     * @param defensiveBuilding The building to get the graphics for.
+     * @param nation The nation (if a nation specific iamge is available).
+     * @param size The size of the background.
+     * @return The image, resized to the given {@code size} without honoring the aspect ratio.
+     */
+    public static BufferedImage getUncachedOutsideColonyBackground(BuildingType defensiveBuilding, String nation, Dimension size) {
+        final String key;
+        if (defensiveBuilding == null) {
+            key = "image.buildingOutside.background.default";
+        } else {
+            key = "image.buildingOutside.background." + defensiveBuilding.getId();
+        }
+        
+        ImageResource ir = ResourceManager.getImageResource(key + "." + nation, false);
+        if (ir == null) {
+            ir = ResourceManager.getImageResource(key, false);
+        }
+        if (ir == null) {
+            return null;
+        }
+        return ir.getSizedImageIgnoringProportions(size);
+    }
 
+    // Colony Panel
+    
+    public BufferedImage getScaledCargoHold(boolean available) {
+        final String key = "image.cargohold." + (available ? "available" : "unavailable");
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    public BufferedImage getColonyDocksBackground() {
+        final String key = "image.colony.docks.background";
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    public BufferedImage getColonyDocksSkyBackground() {
+        final String key = "image.colony.docks.sky.background";
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    public BufferedImage getColonyUpperRightBackground() {
+        final String key = "image.colony.upperRight.background";
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    public BufferedImage getColonyWarehouseBackground() {
+        final String key = "image.colony.warehouse.background";
+        return this.imageCache.getScaledImage(key, this.scaleFactor, false);
+    }
+    
+    public BufferedImage createColonyTitleImage(Graphics2D g2d, String title, Player owner, boolean poles) {
+        final Color backgroundColor = owner.getNationColor();
+        final Color foregroundColor = makeForegroundColor(backgroundColor);
+        
+        final Font font = FontLibrary.getScaledFont("header-plain-large", title);
+        final FontMetrics fm = g2d.getFontMetrics(font);
+        final int textWidth = fm.stringWidth(title);
+        final int textHeight = fm.getMaxAscent() + fm.getMaxDescent();
+
+        final ColorizedImageCreator cic = imageCreators.getColorizedImageCreator();
+        final float bannerScaleFactor = FontLibrary.getFontScaling() / 2;
+        final BufferedImage westImage = cic.getColorizedImage("image.colony.banner.w", bannerScaleFactor, backgroundColor);
+        final BufferedImage centerImage = cic.getColorizedImage("image.colony.banner.center", bannerScaleFactor, backgroundColor);
+        final BufferedImage eastImage = cic.getColorizedImage("image.colony.banner.e", bannerScaleFactor, backgroundColor);
+        
+        final BufferedImage poleLeft = this.imageCache.getScaledImage("image.colony.banner.pole.w", bannerScaleFactor, false);
+        final BufferedImage poleRight = this.imageCache.getScaledImage("image.colony.banner.pole.e", bannerScaleFactor, false);
+
+        int imageWidth = (westImage != null) ? westImage.getWidth() : 0;
+        imageWidth += (eastImage != null) ? eastImage.getWidth() : 0;
+        final int numCenterImages = Math.max(0, (int) Math.ceil((textWidth - imageWidth / 2) / ((double) centerImage.getWidth())));
+        imageWidth += centerImage.getWidth() * numCenterImages;
+        
+        final int imageHeight;
+        final int y;
+        if (poles) {
+            imageHeight = poleLeft.getHeight();
+            y = (poleLeft.getHeight() - centerImage.getHeight()) / 2;
+        } else {
+            imageHeight = centerImage.getHeight();
+            y = 0;
+        }
+        
+        final BufferedImage result = ImageUtils.createBufferedImage(imageWidth, imageHeight);
+        final Graphics2D resultG2D = result.createGraphics();
+        try {
+            resultG2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+            resultG2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
+            resultG2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
+            resultG2D.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);            
+            resultG2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
+            resultG2D.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
+            resultG2D.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
+            resultG2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+            resultG2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+
+            int x = 0;
+            if (westImage != null) {
+                resultG2D.drawImage(westImage, x, y, null);
+                
+                if (poles && poleLeft != null) {
+                    resultG2D.drawImage(poleLeft, x, 0, null);
+                }
+                
+                x += westImage.getWidth();
+            }
+            for (int i=0; i<numCenterImages; i++) {
+                resultG2D.drawImage(centerImage, x, y, null);
+                x += centerImage.getWidth();
+            }
+            if (eastImage != null) {
+                resultG2D.drawImage(eastImage, x, y, null);
+                if (poles && poleRight != null) {
+                    resultG2D.drawImage(poleRight, x, 0, null);
+                }
+                x += eastImage.getWidth();
+            }
+            x = (imageWidth - textWidth) / 2;
+            final int textY = y + (centerImage.getHeight() - textHeight) / 2 + fm.getAscent();
+
+            resultG2D.setFont(font);
+            resultG2D.setColor(foregroundColor);
+            resultG2D.drawString(title, x, textY);
+        } finally {
+            resultG2D.dispose();
+        }
+
+        return result;
+    }
+   
 
     // Goods image handling
 
@@ -943,9 +1163,9 @@ public final class ImageLibrary {
      * @param pt The {@code PathType}
      * @return The {@code BufferedImage}.
      */
-    public static BufferedImage getPathImage(PathType pt) {
+    public BufferedImage getPathImage(PathType pt) {
         return (pt == null) ? null
-            : getUnscaledImage(pt.getImageKey());
+            : this.imageCache.getScaledImage(pt.getImageKey(), this.scaleFactor, false);
     }
 
     /**
@@ -954,7 +1174,7 @@ public final class ImageLibrary {
      * @param u The {@code Unit}
      * @return The {@code BufferedImage}.
      */
-    public static BufferedImage getPathImage(Unit u) {
+    public BufferedImage getPathImage(Unit u) {
         return (u == null) ? null
             : getPathImage(PathType.getPathType(u));
     }
@@ -965,9 +1185,9 @@ public final class ImageLibrary {
      * @param pt The {@code PathType}
      * @return The {@code BufferedImage}.
      */
-    private static BufferedImage getPathNextTurnImage(PathType pt) {
+    private BufferedImage getPathNextTurnImage(PathType pt) {
         return (pt == null) ? null
-            : getUnscaledImage(pt.getNextTurnImageKey());
+            : this.imageCache.getScaledImage(pt.getNextTurnImageKey(), this.scaleFactor, false);
     }
 
     /**
@@ -976,7 +1196,7 @@ public final class ImageLibrary {
      * @param u The {@code Unit}
      * @return The {@code BufferedImage}.
      */
-    public static BufferedImage getPathNextTurnImage(Unit u) {
+    public BufferedImage getPathNextTurnImage(Unit u) {
         return (u == null) ? null
             : getPathNextTurnImage(PathType.getPathType(u));
     }
@@ -1497,8 +1717,7 @@ public final class ImageLibrary {
         if (forestImage != null) {
             height = Math.max(height, forestImage.getHeight());
         }
-        BufferedImage compositeImage
-            = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
+        final BufferedImage compositeImage = ImageUtils.createBufferedImage(width, height);
         Graphics2D g = compositeImage.createGraphics();
         g.drawImage(terrainImage, 0, height - terrainImage.getHeight(), null);
         if (overlayImage != null) {
@@ -1773,16 +1992,14 @@ public final class ImageLibrary {
                                      double amount, Color fill,
                                      Color foreground,
                                      boolean filled) {
-        Font font = getScaledFont("simple-bold-tiny", null);
-        FontMetrics fm = g.getFontMetrics(font);
-        int padding = scaleInt(6);
-        BufferedImage bi = new BufferedImage(fm.stringWidth(text) + padding,
-            fm.getMaxAscent() + fm.getMaxDescent() + padding,
-            BufferedImage.TYPE_INT_ARGB);
-        Graphics2D g2 = bi.createGraphics();
+        final Font font = getScaledFont("simple-bold-tiny", null);
+        final FontMetrics fm = g.getFontMetrics(font);
+        final int padding = scaleInt(6);
+        final int width = fm.stringWidth(text) + padding;
+        final int height = fm.getMaxAscent() + fm.getMaxDescent() + padding;
+        final BufferedImage bi = ImageUtils.createBufferedImage(width, height);
+        final Graphics2D g2 = bi.createGraphics();
         g2.setFont(font);
-        int width = bi.getWidth();
-        int height = bi.getHeight();
         g2.setColor(border);
         g2.fillRect(0, 0, width, height);
         g2.setColor(background);
@@ -1845,8 +2062,10 @@ public final class ImageLibrary {
                                             Font font, FontMetrics fm) {
         final int width = fm.stringWidth(text) + 4;
         final int height = fm.getMaxAscent() + fm.getMaxDescent();
-        BufferedImage img = new BufferedImage(width, height,
-                                              BufferedImage.TYPE_INT_ARGB);
+        if (width <= 0 || height <= 0) {
+            return ImageUtils.createBufferedImage(1, 1);
+        }
+        final BufferedImage img = ImageUtils.createBufferedImage(width, height);
         // draw the string with selected color
         Graphics2D g = img.createGraphics();
         g.setColor(makeStringBorderColor(color));
