@@ -52,6 +52,8 @@ import javax.swing.JPopupMenu;
 import javax.swing.SwingUtilities;
 import javax.swing.filechooser.FileFilter;
 
+import net.miginfocom.layout.PlatformDefaults;
+import net.miginfocom.layout.UnitValue;
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
@@ -64,14 +66,18 @@ import net.sf.freecol.client.gui.dialog.ClientOptionsDialog;
 import net.sf.freecol.client.gui.dialog.FreeColDialog;
 import net.sf.freecol.client.gui.dialog.Parameters;
 import net.sf.freecol.client.gui.mapviewer.GUIMessage;
+import net.sf.freecol.client.gui.mapviewer.MapAsyncPainter;
 import net.sf.freecol.client.gui.mapviewer.MapViewer;
 import net.sf.freecol.client.gui.mapviewer.MapViewerState;
 import net.sf.freecol.client.gui.mapviewer.TileViewer;
 import net.sf.freecol.client.gui.panel.ColonyPanel;
+import net.sf.freecol.client.gui.panel.CornerMapControls;
 import net.sf.freecol.client.gui.panel.FreeColImageBorder;
 import net.sf.freecol.client.gui.panel.FreeColPanel;
 import net.sf.freecol.client.gui.panel.InformationPanel;
 import net.sf.freecol.client.gui.panel.MapControls;
+import net.sf.freecol.client.gui.panel.MapEditorToolboxPanel;
+import net.sf.freecol.client.gui.panel.MapEditorTransformPanel;
 import net.sf.freecol.client.gui.panel.PurchasePanel;
 import net.sf.freecol.client.gui.panel.RecruitPanel;
 import net.sf.freecol.client.gui.panel.StartGamePanel;
@@ -83,7 +89,9 @@ import net.sf.freecol.client.gui.panel.report.LabourData.UnitData;
 import net.sf.freecol.client.gui.plaf.FreeColLookAndFeel;
 import net.sf.freecol.client.gui.plaf.FreeColToolTipUI;
 import net.sf.freecol.common.FreeColException;
+import net.sf.freecol.common.MemoryManager;
 import net.sf.freecol.common.debug.DebugUtils;
+import net.sf.freecol.common.debug.FreeColDebugger;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColDataFile;
 import net.sf.freecol.common.metaserver.ServerInfo;
@@ -118,6 +126,7 @@ import net.sf.freecol.common.option.LanguageOption.Language;
 import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.resources.ImageCache;
+import net.sf.freecol.common.resources.ImageResource;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.util.Introspector;
 import net.sf.freecol.common.util.Utils;
@@ -134,6 +143,12 @@ public class SwingGUI extends GUI {
         CENTERED,
         CENTERED_LEFT,
         CENTERED_RIGHT,
+        LOWER_LEFT,
+        LOWER_RIGHT,
+        UPPER_LEFT,
+        UPPER_RIGHT,
+        LEFT,
+        RIGHT,
         
         /**
          * Places centered even this means overlapping components.
@@ -205,7 +220,7 @@ public class SwingGUI extends GUI {
         
         final float scaleFactor = determineScaleFactorUsingClientOptions(Utils.determineDpi(graphicsDevice));
         this.imageCache = new ImageCache();
-        this.scaledImageLibrary = new ImageLibrary(scaleFactor, this.imageCache);
+        this.scaledImageLibrary = new ImageLibrary(constrainToMaxMapScale(scaleFactor), this.imageCache);
         this.fixedImageLibrary = new ImageLibrary(scaleFactor, this.imageCache);
         this.tileViewer = new TileViewer(freeColClient, fixedImageLibrary);
         // Defer remaining initializations, possibly to startGUI
@@ -215,10 +230,32 @@ public class SwingGUI extends GUI {
         this.widgets = null;
         this.dragPoint = null;
         
+        configureMigLayout(scaleFactor);
+        
         logger.info("GUI constructed using scale factor " + scaleFactor);
     }
 
 
+    private void configureMigLayout(final float scaleFactor) {
+        /*
+         * We should avoid having platform dependent layout and behavior
+         * since this introduces platform specific bugs. Using the defaults
+         * in GNOME as the basis regardless of platform.
+         */
+        PlatformDefaults.setPlatform(PlatformDefaults.GNOME);
+        
+        /*
+         * This scales the numbers without units using the current
+         * interface scaleFactor. We can use "px" as a unit when we want to
+         * provide a size regardless of the current scaling factor.
+         */
+        PlatformDefaults.setHorizontalScaleFactor(scaleFactor);
+        PlatformDefaults.setVerticalScaleFactor(scaleFactor);
+        
+        PlatformDefaults.setMinimumButtonWidth(new UnitValue(48, UnitValue.LPX, null));
+    }
+
+
     // Internals
 
     /**
@@ -365,6 +402,10 @@ public class SwingGUI extends GUI {
     private void setDragPoint(int x, int y) {
         this.dragPoint = new Point(x, y);
     }
+    
+    private void clearDrag() {
+        this.dragPoint = null;
+    }
 
     /**
      * Is mouse movement differnce above the drag threshold?
@@ -414,6 +455,9 @@ public class SwingGUI extends GUI {
         		&& !mapViewer.getMapViewerBounds().onScreen(newUnit.getTile())) {
         	this.mapViewer.getMapViewerBounds().setFocus(newUnit.getTile());
         	mapViewer.getMapViewerRepaintManager().markAsDirty(newUnit.getTile());
+        	if (this.mapControls != null) {
+        	    this.mapControls.updateMinimap();
+        	}
         	repaint();
         }
         
@@ -523,7 +567,10 @@ public class SwingGUI extends GUI {
      */
     private void updateUnitPath() {
         final Unit active = getActiveUnit();
-        if (active == null) return;
+        if (active == null) {
+            setUnitPath(null);
+            return;
+        }
 
         Location destination = active.getDestination();
         PathNode path = null;
@@ -595,11 +642,31 @@ public class SwingGUI extends GUI {
     }
 
     /**
-     * Paint the whole canvas now.
+     * {@inheritDoc}
      */
+    @Override
     public void paintImmediately() {
         this.canvas.paintImmediately(this.canvas.getBounds());
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public MapAsyncPainter useMapAsyncPainter() {
+        canvas.updateRepaintTimer(true);
+        return mapViewer.useMapAsyncPainter();
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void stopMapAsyncPainter() {
+        canvas.updateRepaintTimer(false);
+        mapViewer.stopMapAsyncPainter();
+    }
+
 
     /**
      * Schedule a tile to be repainted.
@@ -618,6 +685,10 @@ public class SwingGUI extends GUI {
                 mapViewer.getMapViewerRepaintManager().markAsDirty(tile.getSurroundingTiles(1, 1));
             }
             
+            if (this.mapControls != null) {
+                this.mapControls.updateMinimap();
+            }
+            
             this.canvas.repaint();
         }
     }
@@ -626,7 +697,7 @@ public class SwingGUI extends GUI {
      * Reset the map zoom and refresh the canvas.
      */
     private void resetMapZoom() {
-        if (this.scaledImageLibrary.getScaleFactor() != fixedImageLibrary.getScaleFactor()) {
+        if (this.scaledImageLibrary.getScaleFactor() != constrainToMaxMapScale(fixedImageLibrary.getScaleFactor())) {
             changeMapScale(fixedImageLibrary.getScaleFactor());
         }
     }
@@ -720,6 +791,7 @@ public class SwingGUI extends GUI {
             throw new FreeColException("Unable to create main font: "
                 + fontName);
         }
+        FreeColLookAndFeel.setScaleFactor(fixedImageLibrary.getScaleFactor());
         FreeColLookAndFeel.installFont(font);
         Utility.initStyleContext(font);
     }
@@ -763,7 +835,7 @@ public class SwingGUI extends GUI {
         this.mapViewer.getMapViewerBounds().setFocus(tile);
 
         enableMapControls(false);
-        enableMapControls(getClientOptions().getBoolean(ClientOptions.DISPLAY_MAP_CONTROLS));
+        enableMapControls(getGame() != null && getGame().getMap() != null && getClientOptions().getBoolean(ClientOptions.DISPLAY_MAP_CONTROLS));
 
         refresh();
     }
@@ -795,7 +867,8 @@ public class SwingGUI extends GUI {
     public void startGUI(final Dimension desiredWindowSize) {
         final FreeColClient fcc = getFreeColClient();
         final ClientOptions opts = getClientOptions();
-        this.mapControls = MapControls.newInstance(fcc);
+        this.mapControls = new CornerMapControls(fcc);
+        
         final ActionListener al = (ActionEvent ae) -> {
             final Tile tile = mapViewer.getMapViewerState().getCursorTile();
             if (tile != null) {
@@ -845,8 +918,54 @@ public class SwingGUI extends GUI {
     @Override
     public void startMapEditorGUI() {
         resetMapZoom(); // Reset zoom to the default
+        mapViewer.getMapViewerState().setActiveUnit(null);
         this.canvas.startMapEditorGUI();
-        this.canvas.showMapEditorTransformPanel();
+        enableEditorToolboxPanel(getGame() != null && getGame().getMap() != null);
+        enableEditorTransformPanel(getGame() != null && getGame().getMap() != null);
+        enableMapControls(getGame() != null && getGame().getMap() != null && getClientOptions().getBoolean(ClientOptions.DISPLAY_MAP_CONTROLS));
+        updateMenuBar();
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void enableEditorToolboxPanel(boolean shouldDisplayPanel) {
+        final MapEditorToolboxPanel panel = this.canvas.getExistingFreeColPanel(MapEditorToolboxPanel.class);
+        if (shouldDisplayPanel && panel == null) {
+            this.canvas.showMapEditorToolboxPanel();
+        } else if (!shouldDisplayPanel && panel != null) {
+            this.canvas.removeFromCanvas(panel);
+        } // else: ignore.
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean isShowingMapEditorToolboxPanel() {
+        return this.canvas.isAddedAsPanelFrameOrIcon(MapEditorToolboxPanel.class);
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void enableEditorTransformPanel(boolean shouldDisplayPanel) {
+        final MapEditorTransformPanel panel = this.canvas.getExistingFreeColPanel(MapEditorTransformPanel.class);
+        if (shouldDisplayPanel && panel == null) {
+            this.canvas.showMapEditorTransformPanel();
+        } else if (!shouldDisplayPanel && panel != null) {
+            this.canvas.removeFromCanvas(panel);
+        } // else: ignore.
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean isShowingMapEditorTransformPanel() {
+        return this.canvas.isAddedAsPanelFrameOrIcon(MapEditorTransformPanel.class);
     }
 
 
@@ -893,6 +1012,11 @@ public class SwingGUI extends GUI {
             final List<Tile> possibleDirtyTiles = dstTile.getSurroundingTiles(0, unit.getLineOfSight());
             mapViewer.getMapViewerRepaintManager().markAsDirty(possibleDirtyTiles);
         }
+        
+        if (this.mapControls != null) {
+            this.mapControls.updateMinimap();
+        }
+        
         repaint();
     }
 
@@ -939,6 +1063,14 @@ public class SwingGUI extends GUI {
     public Tile getFocus() {
         return this.mapViewer.getMapViewerBounds().getFocus();
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public Point getFocusMapPoint() {
+        return this.mapViewer.getMapViewerBounds().getFocusMapPoint();
+    }
 
     /**
      * {@inheritDoc}
@@ -948,6 +1080,15 @@ public class SwingGUI extends GUI {
         this.mapViewer.getMapViewerBounds().setFocus(tileToFocus);
         repaint();
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setFocusMapPoint(Point pointToFocus) {
+        this.mapViewer.getMapViewerBounds().setFocusMapPoint(pointToFocus);
+        repaint();
+    }
 
 
     // MapControls
@@ -989,9 +1130,11 @@ public class SwingGUI extends GUI {
      */
     @Override
     public void miniMapToggleViewControls() {
-        if (this.mapControls == null) return;
+        if (this.mapControls == null) {
+            return;
+        }
         getFreeColClient().toggleClientOption(ClientOptions.MINIMAP_TOGGLE_BORDERS);
-        this.mapControls.repaint();
+        this.mapControls.updateMinimap();
     }
 
     /**
@@ -999,9 +1142,11 @@ public class SwingGUI extends GUI {
      */
     @Override
     public void miniMapToggleFogOfWarControls() {
-        if (this.mapControls == null) return;
+        if (this.mapControls == null) {
+            return;
+        }
         getFreeColClient().toggleClientOption(ClientOptions.MINIMAP_TOGGLE_FOG_OF_WAR);
-        this.mapControls.repaint();
+        this.mapControls.updateMinimap();
     }
 
     /**
@@ -1133,8 +1278,8 @@ public class SwingGUI extends GUI {
                 updateGotoTile(tile);
                 traverseGotoPath();
             }
-            updateUnitPath();
         }
+        updateUnitPath();
         repaint();
     }
 
@@ -1152,7 +1297,11 @@ public class SwingGUI extends GUI {
     @Override
     public void traverseGotoPath() {
         final Unit unit = getActiveUnit();
-        if (unit == null || !isGotoStarted()) return;
+        if (unit == null || !isGotoStarted()) {
+            return;
+        }
+        
+        setUnitPath(null);
 
         final PathNode path = stopGoto();
         if (path == null) {
@@ -1161,8 +1310,7 @@ public class SwingGUI extends GUI {
             igc().goToTile(unit, path);
             refresh();
         }
-        // Only update the path if the unit is still active
-        if (unit == getActiveUnit()) updateUnitPath();
+        updateUnitPath();
         repaint();
     }
 
@@ -1188,6 +1336,17 @@ public class SwingGUI extends GUI {
             stopGoto();
             repaint();
         }
+        
+        final Tile tile = tileAt(x, y);
+        final Unit dragUnit = this.mapViewer.getMapViewerState().findUnitInFront(tile);
+        if (dragUnit == null || !getMyPlayer().owns(dragUnit)) {
+            clearDrag();
+            return;
+        }
+        
+        this.mapViewer.getMapViewerState().setActiveUnit(dragUnit);
+        updateMapControls();
+        
         setDragPoint(x, y);
         this.canvas.requestFocus();
     }
@@ -1199,7 +1358,19 @@ public class SwingGUI extends GUI {
      * {@inheritDoc}
      */
     @Override
-    public boolean scrollMap(Direction direction) {
+    public boolean scrollMap(Direction direction, boolean performRepaints) {
+        if (!performRepaints
+                || this.mapViewer.getMapViewerRepaintManager().isAllDirty()
+                || !getClientOptions().isTerrainAnimationsEnabled()) {
+            return this.mapViewer.getMapViewerBounds().scrollMap(direction);
+        }
+        
+        /*
+         * TODO: We can get better performance for diagonal scrolling, when terrain
+         *       animations are disabled, by updating the back buffer and removing the
+         *       terrain animation check above.
+         */
+        
         boolean scrolled = false;
         if (Direction.longSides.contains(direction)) {
             final List<Direction> directions = toNonDiagonalDirections(direction);
@@ -1207,7 +1378,7 @@ public class SwingGUI extends GUI {
             if (scrolled) {
                 this.mapViewer.paintImmediatelyToBuffersOnly();
             }
-            scrolled |=this.mapViewer.getMapViewerBounds().scrollMap(directions.get(1));
+            scrolled |= this.mapViewer.getMapViewerBounds().scrollMap(directions.get(1));
         } else {
             scrolled |= this.mapViewer.getMapViewerBounds().scrollMap(direction);
         }
@@ -1219,6 +1390,14 @@ public class SwingGUI extends GUI {
         return scrolled;
     }
     
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void resetScrollSpeed() {
+        this.mapViewer.getMapViewerBounds().resetScrollSpeed();
+    }
+    
     private static List<Direction> toNonDiagonalDirections(Direction d) {
         switch (d) {
         case NE: return List.of(Direction.N, Direction.E);
@@ -1348,7 +1527,7 @@ public class SwingGUI extends GUI {
      */
     @Override
     public boolean canZoomInMap() {
-        return this.scaledImageLibrary.getScaleFactor() < ImageLibrary.MAX_SCALE;
+        return this.scaledImageLibrary.getScaleFactor() < getMaxScale();
     }
 
     /**
@@ -1358,6 +1537,10 @@ public class SwingGUI extends GUI {
     public boolean canZoomOutMap() {
         return this.scaledImageLibrary.getScaleFactor() > ImageLibrary.MIN_SCALE;
     }
+    
+    private float getMaxScale() {
+        return MemoryManager.isHighQualityGraphicsEnabled() ? ImageLibrary.MAX_SCALE : ImageLibrary.NORMAL_SCALE;
+    }
 
     /**
      * {@inheritDoc}
@@ -1366,7 +1549,7 @@ public class SwingGUI extends GUI {
     public void zoomInMap() {
         float scale = this.scaledImageLibrary.getScaleFactor();
         float newScale = scale + ImageLibrary.SCALE_STEP;
-        if (scale < newScale && newScale <= ImageLibrary.MAX_SCALE) {
+        if (scale < newScale && newScale <= getMaxScale()) {
             changeMapScale(newScale);
         }
     }
@@ -1384,15 +1567,26 @@ public class SwingGUI extends GUI {
     }
 
     private void changeMapScale(float newScale) {
+        newScale = constrainToMaxMapScale(newScale);
         imageCache.clear();
         if (this.mapViewer != null) {
             this.mapViewer.changeScale(newScale);
         }
+        if (this.mapControls != null) {
+            this.mapControls.updateMinimap();
+        }
         if (this.canvas != null) {
             refresh();
         }
     }
     
+    private float constrainToMaxMapScale(float newScale) {
+        if (newScale > getMaxScale()) {
+            newScale = getMaxScale();
+        }
+        return newScale;
+    }
+    
     /**
      * {@inheritDoc}
      */
@@ -1452,7 +1646,7 @@ public class SwingGUI extends GUI {
         Unit other = null;
 
         if (!tile.isExplored()) { // Select unexplored tiles
-            changeView(tile);
+            setFocus(tile);
         } else if (tile.hasSettlement()) { // Pop up settlements if any
             Settlement settlement = tile.getSettlement();
             if (settlement instanceof Colony) {
@@ -1465,7 +1659,6 @@ public class SwingGUI extends GUI {
             } else if (settlement instanceof IndianSettlement) {
                 showIndianSettlementPanel((IndianSettlement)settlement);
             }
-            changeView(tile);
         } else if ((other = this.mapViewer.getMapViewerState().findUnitInFront(tile)) != null) {
             if (getMyPlayer().owns(other)) {
                 // If there is one of the player units present, select it,
@@ -1489,10 +1682,14 @@ public class SwingGUI extends GUI {
                 }
                 changeView(other, false);
             } else { // Select the tile under the unit if it is not ours
-                changeView(tile);
+                setFocus(tile);
             }
         } else { // Otherwise select the tile in terrain mode on multiclick
-            if (count > 1) changeView(tile);
+            if (count > 1) {
+                changeView(tile);
+            } else {
+                setFocus(tile);
+            }
         }
     }    
 
@@ -1598,6 +1795,9 @@ public class SwingGUI extends GUI {
         if (this.mapViewer != null) {
             this.mapViewer.getMapViewerRepaintManager().markAsDirty();
         }
+        if (this.mapControls != null) {
+            this.mapControls.updateMinimap();
+        }
         if (this.canvas != null) {
             this.canvas.repaint(0, 0, this.canvas.getWidth(), this.canvas.getHeight());
         }
@@ -1775,15 +1975,26 @@ public class SwingGUI extends GUI {
             this.tileViewer.updateScaledVariables();
         }
         
+        configureMigLayout(scaleFactor);
+        
         FreeColImageBorder.setScaleFactor(scaleFactor);
         
         ResourceManager.setMods(getClientOptions().getActiveMods());
-        prepareResources();
+        if (getClientOptions().getRange(ClientOptions.GRAPHICS_QUALITY) == ClientOptions.GRAPHICS_QUALITY_LOWEST) {
+            ImageResource.forceLowestQuality(true);
+            reloadResources();
+        } else if (ImageResource.isForceLowestQuality()) {
+            ImageResource.forceLowestQuality(false);
+            reloadResources();
+        } else {
+            prepareResources();
+        }
         
         final int fontSize = determineMainFontSizeUsingClientOptions(dpi);
         FontLibrary.setMainFontSize(fontSize);
         
         final Font font = FontLibrary.getMainFont();
+        FreeColLookAndFeel.setScaleFactor(scaleFactor);
         FreeColLookAndFeel.installFont(font);
         FreeColToolTipUI.setFontScaling(FontLibrary.getFontScaling());
         Utility.initStyleContext(font);
@@ -1800,6 +2011,8 @@ public class SwingGUI extends GUI {
             resetMapControls();
         }
         
+        canvas.refreshAllFrames();
+        
         refresh();
     }
 
@@ -1824,23 +2037,28 @@ public class SwingGUI extends GUI {
         }
         
         final int displayScaling = getClientOptions().getInteger(ClientOptions.DISPLAY_SCALING);
-        if (displayScaling == 0) {
-            int fontSize = (int) ((FontLibrary.DEFAULT_UNSCALED_MAIN_FONT_SIZE * dpi) / DEFAULT_DPI);
-            
-            final int screenHeight = graphicsDevice.getDisplayMode().getHeight();
-            if (screenHeight < 900) {
-                fontSize = Math.min(14, fontSize);
-            } else if (screenHeight < 1050) {
-                fontSize = Math.min(18, fontSize);
-            }
-                    
-            logger.info("Automatic font size: " + fontSize + " (reported DPI: " + dpi + ", screen height: " + screenHeight + ")");        
-            return fontSize;
+        final float scaleFactor = determineScaleFactorUsingClientOptions(dpi);
+        final int fontSizeUsingScaling = (int) (FontLibrary.DEFAULT_UNSCALED_MAIN_FONT_SIZE * scaleFactor);
+        
+        if (displayScaling != 0) {
+            logger.info("Font size based on manual display scaling: " + fontSizeUsingScaling + " (reported DPI: " + dpi + ")");
+            return fontSizeUsingScaling;
         }
         
-        final int fontSize = (int) (FontLibrary.DEFAULT_UNSCALED_MAIN_FONT_SIZE * (displayScaling / 100f));
-        logger.info("Font size based on manual display scaling: " + fontSize + " (reported DPI: " + dpi + ")");        
-        return fontSize;
+        int fontSizeUsingDpi = (int) ((FontLibrary.DEFAULT_UNSCALED_MAIN_FONT_SIZE * dpi) / DEFAULT_DPI);
+        final int screenHeight = graphicsDevice.getDisplayMode().getHeight();
+        if (screenHeight < 900) {
+            fontSizeUsingDpi = Math.min(14, fontSizeUsingDpi);
+        } else if (screenHeight < 1050) {
+            fontSizeUsingDpi = Math.min(18, fontSizeUsingDpi);
+        }
+        
+        if (fontSizeUsingDpi >= fontSizeUsingScaling * 0.25f) {
+            return fontSizeUsingScaling;
+        }
+                
+        logger.info("Automatic font size: " + fontSizeUsingDpi + " (reported DPI: " + dpi + ", screen height: " + screenHeight + ")");        
+        return fontSizeUsingDpi;        
     }
 
     private float determineScaleFactorUsingClientOptions(final int dpi) {
@@ -1860,8 +2078,15 @@ public class SwingGUI extends GUI {
                 scaleFactor = 2F;
             }
             
-            final int screenHeight = graphicsDevice.getDisplayMode().getHeight();
-            if (screenHeight < 900) {
+            final int screenHeight;
+            if (FreeColDebugger.isAutomaticRescalingOnWindowResize()) {
+                screenHeight = (canvas != null && canvas.getParentFrame() != null) ? canvas.getParentFrame().getHeight() : graphicsDevice.getDisplayMode().getHeight();
+            } else {
+                screenHeight = graphicsDevice.getDisplayMode().getHeight();
+            }
+            if (screenHeight < 720) {
+                scaleFactor = 0.75F;
+            } else if (screenHeight < 900) {
                 scaleFactor = 1F;
             } else if (screenHeight < 1050) {
                 scaleFactor = Math.min(1.25F, scaleFactor);
@@ -1905,6 +2130,18 @@ public class SwingGUI extends GUI {
         if (unit != null) panel.setSelectedUnit(unit);
         return panel;
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void showFreeColPanel(FreeColPanel panel, boolean toolBox, PopupPosition popupPosition, boolean resizable) {
+        if (canvas.isAddedAsPanelFrameOrIcon(panel)) {
+            panel.requestFocus();
+        } else {
+            canvas.showFreeColPanel(panel, toolBox, popupPosition, resizable);
+        }
+    }
 
     /**
      * {@inheritDoc}
@@ -1951,7 +2188,17 @@ public class SwingGUI extends GUI {
      */
     @Override
     public void showDeclarationPanel(Runnable afterClosing) {
-        this.widgets.showDeclarationPanel(afterClosing);
+        mapViewer.getMapViewerRepaintManager().setRepaintsBlocked(true);
+        try {
+            this.widgets.showDeclarationPanel(() -> {
+                mapViewer.getMapViewerRepaintManager().setRepaintsBlocked(false);
+                refresh();
+                afterClosing.run();
+            });
+        } catch (RuntimeException e) {
+            mapViewer.getMapViewerRepaintManager().setRepaintsBlocked(false);
+            throw e;
+        }
     }
 
     /**
@@ -2267,7 +2514,13 @@ public class SwingGUI extends GUI {
      */
     @Override
     public FreeColPanel showNewPanel(Specification spec) {
-        return this.widgets.showNewPanel(spec);
+        try {
+            return this.widgets.showNewPanel(spec);
+        } catch (RuntimeException e) {
+            logger.log(Level.WARNING, "Exception while showing new game panel", e);
+            showErrorPanel(e, StringTemplate.key("error.unspecified"));
+            return null;
+        }
     }
 
     /**
@@ -2612,8 +2865,8 @@ public class SwingGUI extends GUI {
      * {@inheritDoc}
      */
     @Override
-    public boolean showWarehouseDialog(Colony colony) {
-        return this.widgets.showWarehouseDialog(colony);
+    public void showWarehouseDialog(Colony colony, DialogHandler<Boolean> handler) {
+        this.widgets.showWarehouseDialog(colony, handler);
     }
 
     /**
@@ -2639,4 +2892,19 @@ public class SwingGUI extends GUI {
     public boolean canGameChangingModsBeAdded() {
         return getFreeColClient().getGame() == null;
     }
+    
+    @Override
+    public void refreshScaleFactorIfNecessary() {
+        if (!FreeColDebugger.isAutomaticRescalingOnWindowResize()) {
+            return;
+        }
+        
+        final int dpi = Utils.determineDpi(graphicsDevice);
+        final float scaleFactor = determineScaleFactorUsingClientOptions(dpi);
+        if (Math.abs(getFixedImageLibrary().getScaleFactor() - scaleFactor) <= 0.1) {
+            return;
+        }
+        
+        refreshGuiUsingClientOptions();
+    }
 }
