@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,10 +23,8 @@ import java.awt.Component;
 import java.awt.Container;
 import java.awt.Font;
 import java.awt.event.ActionEvent;
-import java.util.Collections;
 import java.util.List;
 import java.util.logging.Logger;
-import java.util.stream.Collectors;
 
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
@@ -36,7 +34,6 @@ import javax.swing.JSeparator;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.action.UnloadAction;
-import net.sf.freecol.client.gui.panel.ReportPanel;
 import net.sf.freecol.client.gui.panel.Utility;
 import net.sf.freecol.common.debug.DebugUtils;
 import net.sf.freecol.common.debug.FreeColDebugger;
@@ -52,6 +49,7 @@ import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Unit.UnitState;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
 
 
@@ -69,116 +67,114 @@ public final class TilePopup extends JPopupMenu {
     public static final int UNIT_LINES_IN_OTHER_MENUS = 19;
 
     private final FreeColClient freeColClient;
-    private final SwingGUI gui;
-    private final Canvas canvas;
+    private final GUI gui;
+    private Font font;
     private boolean hasAnItem = false;
 
 
     /**
      * The constructor that will insert the MenuItems.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
-     * @param canvas The Canvas.
-     * @param tile The <code>Tile</code> to create a popup for.
-     *       The popup menu also appears near this <code>Tile</code>.
+     * @param freeColClient The {@code FreeColClient} for the game.
+     * @param tile The {@code Tile} to create a popup for.
+     *       The popup menu also appears near this {@code Tile}.
      */
-    public TilePopup(final FreeColClient freeColClient, final Canvas canvas,
-                     final Tile tile) {
+    public TilePopup(final FreeColClient freeColClient, final Tile tile) {
         super(Messages.message(tile.getSimpleLabel()));
 
         this.freeColClient = freeColClient;
-        this.gui = (SwingGUI)freeColClient.getGUI();
-        this.canvas = canvas;
-
+        this.gui = freeColClient.getGUI();
+        this.font = FontLibrary.getScaledFont("normal-bold-tiny");
+        final InGameController igc = freeColClient.getInGameController();
         final Player player = freeColClient.getMyPlayer();
+
         final Unit activeUnit = gui.getActiveUnit();
-        final boolean owned = player != null && activeUnit != null
-            && player.owns(activeUnit);
-        Tile unitTile;
-        if (activeUnit != null && owned
-            && (unitTile = activeUnit.getTile()) != null) {
-            JMenuItem gotoMenuItem = null;
-            if (activeUnit.isOffensiveUnit()
-                && unitTile.isAdjacent(tile)
-                && activeUnit.getMoveType(tile).isAttack()) {
-                gotoMenuItem = Utility.localizedMenuItem(activeUnit
-                    .getCombatLabel(tile));
-            } else if (activeUnit.getSimpleMoveType(unitTile, tile).isLegal()) {
-                gotoMenuItem = Utility.localizedMenuItem("goToThisTile");
-            }
-            if (gotoMenuItem != null) {
-                gotoMenuItem.addActionListener((ActionEvent ae) -> {
-                        if (!freeColClient.currentPlayerIsMyPlayer()) return;
-                        Tile currTile = activeUnit.getTile();
-                        if (currTile == tile) return; // already at destination
-                        freeColClient.getInGameController()
-                            .goToTile(activeUnit, tile);
-                        // if unit did not move, we should show the goto path
-                        if (activeUnit.getTile() == currTile) {
-                            canvas.updateCurrentPathForActiveUnit();
-                        }
-                    });
-                add(gotoMenuItem);
-            }
-
-            // Add move to Europe entry if the unit can do so
-            final InGameController igc = freeColClient.getInGameController();
-            if (unitTile == tile && activeUnit.hasHighSeasMove()) {
-                JMenuItem europeMenuItem = Utility.localizedMenuItem(StringTemplate
-                    .template("goToEurope"));
-                europeMenuItem.addActionListener((ActionEvent ae) -> {
-                        if (!freeColClient.currentPlayerIsMyPlayer()) return;
-                        igc.moveTo(activeUnit, player.getEurope());
-                    });
-                add(europeMenuItem);
-                hasAnItem = true;
-            }
-
-            // Add state changes if present
-            if (unitTile == tile) {
-                JMenuItem ji = null;
-                if (activeUnit.checkSetState(UnitState.ACTIVE)) {
-                    ji = Utility.localizedMenuItem("activateUnit");
-                    ji.addActionListener((ActionEvent ae) -> {
-                            igc.changeState(activeUnit, Unit.UnitState.ACTIVE);
-                        });
-                    add(ji);
-                    hasAnItem = true;
+        if (activeUnit != null) {
+            final boolean owned = player != null && player.owns(activeUnit);
+            Tile unitTile = activeUnit.getTile();
+            if (owned && unitTile != null) {
+                // Check for unit to tile interactions: attack, goto
+                JMenuItem gotoMenuItem = null;
+                if (activeUnit.isOffensiveUnit()
+                    && unitTile.isAdjacent(tile)
+                    && activeUnit.getMoveType(tile).isAttack()) {
+                    gotoMenuItem = Utility.localizedMenuItem(activeUnit
+                        .getCombatLabel(tile));
+                } else if (activeUnit.getSimpleMoveType(unitTile, tile).isLegal()) {
+                    gotoMenuItem = Utility.localizedMenuItem("goToThisTile");
                 }
-                if (activeUnit.checkSetState(UnitState.FORTIFYING)) {
-                    ji = Utility.localizedMenuItem("fortify");
-                    ji.addActionListener((ActionEvent ae) -> {
-                            igc.changeState(activeUnit, Unit.UnitState.FORTIFYING);
+                if (gotoMenuItem != null) {
+                    gotoMenuItem.addActionListener((ActionEvent ae) -> {
+                            if (!freeColClient.currentPlayerIsMyPlayer())
+                                return;
+                            TilePopup.this.gui.performGoto(tile);
                         });
-                    add(ji);
-                    hasAnItem = true;
-                }
-                if (activeUnit.checkSetState(UnitState.SKIPPED)) {
-                    ji = Utility.localizedMenuItem("skip");
-                    ji.addActionListener((ActionEvent ae) -> {
-                            igc.changeState(activeUnit, Unit.UnitState.SKIPPED);
-                        });
-                    add(ji);
-                    hasAnItem = true;
-                }
-                if (activeUnit.canCarryTreasure()
-                    && activeUnit.canCashInTreasureTrain()) {
-                    ji = Utility.localizedMenuItem("cashInTreasureTrain");
-                    ji.addActionListener((ActionEvent ae) -> {
-                            igc.checkCashInTreasureTrain(activeUnit);
-                        });
-                    ji.setEnabled(true);
-                    add(ji);
-                    hasAnItem = true;
+                    add(gotoMenuItem);
                 }
 
-                if (activeUnit.getDestination() != null) {
-                    ji = Utility.localizedMenuItem("clearOrders");
-                    ji.addActionListener((ActionEvent ae) -> {
-                            igc.clearOrders(activeUnit);
-                        });
-                    add(ji);
-                    hasAnItem = true;
+                // Special cases for active unit on the tile
+                if (unitTile == tile) {
+                    // Add move to Europe entry if the unit can do so
+                    if (activeUnit.canMoveToHighSeas()) {
+                        JMenuItem europeMenuItem
+                            = Utility.localizedMenuItem(StringTemplate.template("goToEurope"));
+                        europeMenuItem.addActionListener((ActionEvent ae) -> {
+                                if (!freeColClient.currentPlayerIsMyPlayer())
+                                    return;
+                                igc.moveTo(activeUnit, player.getEurope());
+                            });
+                        add(europeMenuItem);
+                        hasAnItem = true;
+                    }
+
+                    // Add state changes if present
+                    JMenuItem ji = null;
+                    if (activeUnit.checkSetState(UnitState.ACTIVE)) {
+                        ji = Utility.localizedMenuItem("activateUnit");
+                        ji.addActionListener((ActionEvent ae) -> {
+                                igc.changeState(activeUnit, Unit.UnitState.ACTIVE);
+                            });
+                        add(ji);
+                        hasAnItem = true;
+                    }
+                    if (activeUnit.checkSetState(UnitState.FORTIFYING)) {
+                        ji = Utility.localizedMenuItem("fortify");
+                        ji.addActionListener((ActionEvent ae) -> {
+                                igc.changeState(activeUnit, Unit.UnitState.FORTIFYING);
+                            });
+                        add(ji);
+                        hasAnItem = true;
+                    }
+                    if (activeUnit.checkSetState(UnitState.SKIPPED)) {
+                        ji = Utility.localizedMenuItem("skip");
+                        ji.addActionListener((ActionEvent ae) -> {
+                                igc.changeState(activeUnit, Unit.UnitState.SKIPPED);
+                            });
+                        add(ji);
+                        hasAnItem = true;
+                    }
+
+                    // Treasure train cashin
+                    if (activeUnit.canCarryTreasure()
+                        && activeUnit.canCashInTreasureTrain()) {
+                        ji = Utility.localizedMenuItem("cashInTreasureTrain");
+                        ji.addActionListener((ActionEvent ae) -> {
+                                igc.checkCashInTreasureTrain(activeUnit);
+                            });
+                        ji.setEnabled(true);
+                        add(ji);
+                        hasAnItem = true;
+                    }
+                    
+                    // Clear orders is possible if there is a destination
+                    if (activeUnit.getDestination() != null) {
+                        ji = Utility.localizedMenuItem("clearOrders");
+                        ji.addActionListener((ActionEvent ae) -> {
+                                igc.clearOrders(activeUnit);
+                            });
+                        add(ji);
+                        hasAnItem = true;
+                    }
                 }
             }
             if (hasAnItem) addSeparator();
@@ -194,7 +190,7 @@ public final class TilePopup extends JPopupMenu {
             if (hasAnItem) addSeparator();
         }
 
-        if (tile.isExplored()) addTile(tile);
+        addTile(tile);
         addSeparator();
 
         int lineCount = 0;
@@ -202,9 +198,7 @@ public final class TilePopup extends JPopupMenu {
         Container currentMenu = this;
         boolean moreUnits = false;
         Unit firstUnit = tile.getFirstUnit();
-        for (Unit u : tile.getUnitList().stream()
-                 .sorted(Unit.typeRoleComparator)
-                 .collect(Collectors.toList())) {
+        for (Unit u : sort(tile.getUnits(), Unit.typeRoleComparator)) {
             if (lineCount > maxUnits) {
                 JMenu more = Utility.localizedMenu("more");
                 more.setFont(more.getFont().deriveFont(Font.ITALIC));
@@ -223,12 +217,8 @@ public final class TilePopup extends JPopupMenu {
             JMenuItem activateAllItem = Utility.localizedMenuItem(StringTemplate
                 .template("activateAllUnits"));
             activateAllItem.addActionListener((ActionEvent ae) -> {
-                    Unit lastUnit = null;
-                    for (Unit unit : tile.getUnitList()) {
-                        freeColClient.getInGameController().clearOrders(unit);
-                        lastUnit = unit;
-                    }
-                    gui.setActiveUnit(lastUnit);
+                    for (Unit unit : tile.getUnitList()) igc.clearOrders(unit);
+                    gui.changeView(tile.getFirstUnit(), false);
                 });
             add(activateAllItem);
         }
@@ -247,11 +237,11 @@ public final class TilePopup extends JPopupMenu {
     /**
      * Build the debug entries for the TilePopup.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
-     * @param tile The <code>Tile</code> to build menu items for.
+     * @param freeColClient The {@code FreeColClient} for the game.
+     * @param tile The {@code Tile} to build menu items for.
      */
-    public void addDebugItems(final FreeColClient freeColClient, 
-                              final Tile tile) {
+    private void addDebugItems(final FreeColClient freeColClient, 
+                               final Tile tile) {
         addSeparator();
 
         JMenu changeOwnership = new JMenu("Change ownership");
@@ -355,27 +345,19 @@ public final class TilePopup extends JPopupMenu {
                     if (currTile == tile) return;
                     final Map map = activeUnit.getGame().getMap();
                     LogBuilder lb = new LogBuilder(512);
-                    PathNode path = map.findPath(activeUnit,
-                        activeUnit.getTile(), tile, activeUnit.getCarrier(),
-                        null, lb);
-                    gui.showInformationMessage(lb.toString());
-                    canvas.setCurrentPath(path);
+                    PathNode path = activeUnit.findPath(activeUnit.getTile(),
+                        tile, activeUnit.getCarrier(), null, lb);
+                    gui.showInformationPanel(lb.toString());
+                    gui.setUnitPath(path);
                     gui.refresh();                        
                 });
             add(menuItem);
         }
 
-        for (Unit u : tile.getUnitList()) {
-            if (u.canCarryGoods() && u.hasSpaceLeft()) {
-                JMenuItem addg = new JMenuItem("Add goods");
-                addg.setOpaque(false);
-                final Unit unit = u;
-                addg.addActionListener((ActionEvent ae) -> {
-                        DebugUtils.addUnitGoods(freeColClient, unit);
-                    });
-                add(addg);
-                break;
-            }
+        Unit unit = find(tile.getUnits(),
+                         u -> u.canCarryGoods() && u.hasSpaceLeft());
+        if (unit != null) {
+            DebugUtils.addGoodsAdditionEntry(freeColClient, unit, this);
         }
 
         JMenuItem dumpItem = new JMenuItem("Dump tile");
@@ -388,12 +370,13 @@ public final class TilePopup extends JPopupMenu {
 
     /**
      * Adds a unit entry to this popup.
-     * @param menu a <code>Container</code> value
+     *
+     * @param menu a {@code Container} value
      * @param unit The unit that will be represented on the popup.
      * @param enabled The initial state for the menu item.
-     * @param indent Should be <code>true</code> if the text should be
+     * @param indent Should be {@code true} if the text should be
      *      indented on the menu.
-     * @return an <code>int</code> value
+     * @return an {@code int} value
      */
     private int addUnit(Container menu, final Unit unit, boolean enabled,
                         boolean indent) {
@@ -401,13 +384,11 @@ public final class TilePopup extends JPopupMenu {
             = unit.getOccupationLabel(freeColClient.getMyPlayer(), true);
         String text = (indent ? "    " : "")
             + unit.getDescription(Unit.UnitLabelType.NATIONAL)
-            + " (" + Messages.message(occ) + ")";
+            + " (" + Messages.message(occ) + ')';
         JMenuItem menuItem = new JMenuItem(text);
-        menuItem.setFont(FontLibrary.createFont(FontLibrary.FontType.NORMAL,
-            FontLibrary.FontSize.TINY, Font.BOLD,
-            gui.getImageLibrary().getScaleFactor()));
+        menuItem.setFont(this.font);
         menuItem.addActionListener((ActionEvent ae) -> {
-                gui.setActiveUnit(unit);
+                gui.changeView(unit, false);
             });
         if (indent) {
             menuItem.setFont(menuItem.getFont().deriveFont(Font.ITALIC));
@@ -415,8 +396,8 @@ public final class TilePopup extends JPopupMenu {
         menuItem.setEnabled(enabled);
         menu.add(menuItem);
 
-        int lineCount = 1 + unit.getUnitList().stream()
-            .mapToInt(u -> addUnit(menu, u, true, true)).sum();
+        int lineCount = 1 + sum(unit.getUnits(),
+                                u -> addUnit(menu, u, true, true));
         boolean hasGoods = false;
         for (Goods goods: unit.getGoodsList()) {
             text = (indent ? "         " : "     ")
@@ -432,6 +413,8 @@ public final class TilePopup extends JPopupMenu {
         if (hasGoods) {
             JMenuItem dumpItem = Utility.localizedMenuItem("dumpCargo");
             dumpItem.setAction(new UnloadAction(freeColClient, unit));
+            // setAction changes the label
+            dumpItem.setText(Messages.message("dumpCargo"));
             menu.add(dumpItem);
             lineCount++;
         }
@@ -448,9 +431,7 @@ public final class TilePopup extends JPopupMenu {
         StringTemplate name
             = colony.getLocationLabelFor(freeColClient.getMyPlayer());
         JMenuItem menuItem = Utility.localizedMenuItem(name);
-        menuItem.setFont(FontLibrary.createFont(FontLibrary.FontType.NORMAL,
-            FontLibrary.FontSize.TINY, Font.BOLD,
-            gui.getImageLibrary().getScaleFactor()));
+        menuItem.setFont(this.font);
         menuItem.addActionListener((ActionEvent ae) -> {
                 gui.showColonyPanel(colony, null);
             });
@@ -470,18 +451,16 @@ public final class TilePopup extends JPopupMenu {
     /**
      * Adds an indian settlement entry to this popup.
      *
-     * @param settlement The Indian settlement that will be
+     * @param is The {@code IndianSettlement} that will be
      *     represented on the popup.
      */
-    private void addIndianSettlement(final IndianSettlement settlement) {
+    private void addIndianSettlement(final IndianSettlement is) {
         StringTemplate name
-            = settlement.getLocationLabelFor(freeColClient.getMyPlayer());
+            = is.getLocationLabelFor(freeColClient.getMyPlayer());
         JMenuItem menuItem = Utility.localizedMenuItem(name);
-        menuItem.setFont(FontLibrary.createFont(FontLibrary.FontType.NORMAL,
-            FontLibrary.FontSize.TINY, Font.BOLD,
-            gui.getImageLibrary().getScaleFactor()));
+        menuItem.setFont(this.font);
         menuItem.addActionListener((ActionEvent ae) -> {
-                gui.showIndianSettlementPanel(settlement);
+                gui.showIndianSettlementPanel(is);
             });
         add(menuItem);
         hasAnItem = true;
@@ -493,10 +472,12 @@ public final class TilePopup extends JPopupMenu {
      * @param tile The tile that will be represented on the popup.
      */
     private void addTile(final Tile tile) {
-        JMenuItem menuItem = new JMenuItem(Messages.getName(tile));
-        menuItem.addActionListener((ActionEvent ae) -> {
-                gui.showTilePanel(tile);
-            });
+        JMenuItem menuItem = new JMenuItem(Messages.message(tile.getLabel()));
+        if (tile.isExplored()) {
+            menuItem.addActionListener((ActionEvent ae) -> {
+                    gui.showTilePanel(tile);
+                });
+        }
 
         add(menuItem);
         /**
