@@ -30,7 +30,7 @@ import net.sf.freecol.client.FreeColClient;
 /**
  * An action to toggle the display of national borders.
  */
-public class DisplayBordersAction extends SelectableAction {
+public class DisplayBordersAction extends SelectableOptionAction {
 
     public static final String id = "displayBordersAction";
 
