@@ -370,8 +370,7 @@ public abstract class FreeColAction extends AbstractAction
      * {@link #shouldBeEnabled}.
      */
     public void update() {
-        boolean b = this.shouldBeEnabled();
-        if (this.isEnabled() != b) this.setEnabled(b);
+        this.setEnabled(this.shouldBeEnabled());
     }
 
 
