@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,6 +22,7 @@ package net.sf.freecol.client.gui.action;
 import java.awt.event.ActionEvent;
 
 import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.common.model.Game.LogoutReason;
 
 
 /**
@@ -33,9 +34,9 @@ public class ReconnectAction extends FreeColAction {
 
 
     /**
-     * Creates a new <code>DeclareIndependenceAction</code>.
+     * Creates a new {@code DeclareIndependenceAction}.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
+     * @param freeColClient The {@code FreeColClient} for the game.
      */
     public ReconnectAction(FreeColClient freeColClient) {
         super(freeColClient, id);
