@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -35,7 +35,7 @@ public class ReportHighScoresAction extends FreeColAction {
     /**
      * Creates this action.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
+     * @param freeColClient The {@code FreeColClient} for the game.
      */
     public ReportHighScoresAction(FreeColClient freeColClient) {
         super(freeColClient, id);
@@ -49,6 +49,6 @@ public class ReportHighScoresAction extends FreeColAction {
      */
     @Override
     public void actionPerformed(ActionEvent ae) {
-        igc().displayHighScores(null);
+        igc().highScore(null);
     }
 }
