@@ -19,7 +19,6 @@
 
 package net.sf.freecol.client.gui.mapviewer;
 
-import static net.sf.freecol.common.util.StringUtils.lastPart;
 import static net.sf.freecol.common.util.Utils.now;
 
 import java.awt.AlphaComposite;
@@ -28,10 +27,12 @@ import java.awt.Color;
 import java.awt.Composite;
 import java.awt.Dimension;
 import java.awt.Font;
+import java.awt.FontMetrics;
 import java.awt.Graphics2D;
 import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
+import java.awt.Shape;
 import java.awt.Stroke;
 import java.awt.event.ActionListener;
 import java.awt.font.TextLayout;
@@ -39,6 +40,7 @@ import java.awt.geom.AffineTransform;
 import java.awt.geom.Ellipse2D;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D.Float;
+import java.awt.geom.Rectangle2D;
 import java.awt.geom.RoundRectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.RescaleOp;
@@ -48,18 +50,24 @@ import java.util.EnumMap;
 import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.stream.Collectors;
+
+import javax.swing.SwingUtilities;
 
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.FreeColClientHolder;
+import net.sf.freecol.client.control.MapEditorController;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.GUI.ViewMode;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.SwingGUI;
 import net.sf.freecol.common.debug.FreeColDebugger;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.model.Ability;
+import net.sf.freecol.common.model.Area;
 import net.sf.freecol.common.model.BuildableType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Direction;
@@ -73,6 +81,7 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.option.GameOptions;
+import net.sf.freecol.common.util.ImageUtils;
 import net.sf.freecol.server.ai.AIObject;
 import net.sf.freecol.server.ai.EuropeanAIPlayer;
 import net.sf.freecol.server.ai.military.DefensiveMap;
@@ -143,6 +152,12 @@ public final class MapViewer extends FreeColClientHolder {
     
     private List<Rectangle> fullyRepaintedAreas = new ArrayList<>();
     
+    /**
+     * An asynchronous painter. If enabled (not null), then the {@code asyncPainter} will be
+     * used when painting this {@code MapViewer}.
+     */
+    private MapAsyncPainter asyncPainter = null;
+    
 
     /**
      * The constructor to use.
@@ -217,6 +232,34 @@ public final class MapViewer extends FreeColClientHolder {
     public Tile convertToMapTile(int x, int y) {
         return mapViewerBounds.convertToMapTile(getMap(), x, y);
     }
+
+    /**
+     * This method should only be called using {@link GUI#useMapAsyncPainter()}.
+     */
+    public MapAsyncPainter useMapAsyncPainter() {
+        assert SwingUtilities.isEventDispatchThread();
+        
+        if (asyncPainter != null && !asyncPainter.isStopped()) {
+            return asyncPainter;
+        }
+        
+        asyncPainter = new MapAsyncPainter(this);
+        
+        return asyncPainter;
+    }
+    
+    /**
+     * This method should only be called using {@link GUI#stopMapAsyncPainter()}.
+     */
+    public void stopMapAsyncPainter() {
+        assert SwingUtilities.isEventDispatchThread();
+        
+        final MapAsyncPainter theAsyncPainter = asyncPainter;
+        if (theAsyncPainter != null) {
+            theAsyncPainter.stop();
+        }
+        asyncPainter = null;
+    }
     
     /**
      * Displays the Map.
@@ -228,65 +271,148 @@ public final class MapViewer extends FreeColClientHolder {
     @SuppressFBWarnings(value="NP_LOAD_OF_KNOWN_NULL_VALUE",
                         justification="lazy load of extra tiles")
     public boolean displayMap(Graphics2D g2d, Dimension size) {
-        final long startMs = now();
-        final Rectangle clipBounds = g2d.getClipBounds();
-        if (mapViewerBounds.getFocus() == null) {
-            paintBlackBackground(g2d, clipBounds);
+        final MapAsyncPainter thePainter = asyncPainter;
+        if (thePainter != null) {
+            final BufferedImage backBufferImage = thePainter.getBackBufferImage();
+            if (backBufferImage == null) {
+                rpm.markAsDirty();
+                return paintMap(g2d, size, mapViewerBounds, true);
+            }
+            
+            g2d.setColor(Color.BLACK);
+            g2d.drawImage(backBufferImage, 0, 0, null);
+            
+            mapViewerState.getChatDisplay().display(g2d, mapViewerBounds.getSize());
+            
             return false;
         }
 
         if (rpm.isRepaintsBlocked(size)) {
             final VolatileImage backBufferImage = rpm.getBackBufferImage();
+            g2d.setColor(Color.black);
+            g2d.fillRect(0, 0, size.width, size.height);
             g2d.drawImage(backBufferImage, 0, 0, null);
+            
+            mapViewerState.getChatDisplay().display(g2d, mapViewerBounds.getSize());
+            
             return false;
         }
 
-        boolean fullMapRenderedWithoutUsingBackBuffer = rpm.prepareBuffers(mapViewerBounds, mapViewerBounds.getFocus());
-        final Rectangle dirtyClipBounds = rpm.getDirtyClipBounds();
-        if (rpm.isAllDirty()) {
+        return paintMap(g2d, size, mapViewerBounds, true);
+    }
+    
+    /**
+     * Displays the Map.
+     *
+     * @param g2d The {@code Graphics2D} object on which to draw the Map.
+     * @param size The size of the map.
+     * @param mapViewerBounds The bounds to be used when drawing the map. This can be
+     *      a different object than {@link #getMapViewerBounds()} when painting to
+     *      buffers etc.
+     * 
+     * @return {@code true} if the entire map has been repainted.
+     */
+    @SuppressFBWarnings(value="NP_LOAD_OF_KNOWN_NULL_VALUE",
+                        justification="lazy load of extra tiles")
+    public boolean paintMap(Graphics2D g2d, Dimension size, MapViewerBounds mapViewerBounds) {
+        return paintMap(g2d, size, mapViewerBounds, false);
+    }
+    
+    private boolean paintMap(Graphics2D g2d, Dimension size, MapViewerBounds mapViewerBounds, boolean useBuffers) {
+        final long startMs = now();
+        
+        final Rectangle clipBounds = (useBuffers) ? g2d.getClipBounds() : new Rectangle(0, 0, size.width, size.height);
+        if (mapViewerBounds.getFocus() == null) {
+            if (g2d != null) {
+                paintBlackBackground(g2d, clipBounds);
+            }
+            return false;
+        }
+        final Rectangle dirtyClipBounds;
+        boolean fullMapRenderedWithoutUsingBackBuffer;
+        if (useBuffers) {
+            fullMapRenderedWithoutUsingBackBuffer = rpm.prepareBuffers(mapViewerBounds, mapViewerBounds.getFocus());
+            dirtyClipBounds = rpm.getDirtyClipBounds();
+            if (rpm.isAllDirty()) {
+                fullMapRenderedWithoutUsingBackBuffer = true;
+            }
+        } else {
+            dirtyClipBounds = clipBounds;
             fullMapRenderedWithoutUsingBackBuffer = true;
         }
         
-        final VolatileImage backBufferImage = rpm.getBackBufferImage();
-        final BufferedImage nonAnimationBufferImage = rpm.getNonAnimationBufferImage();
+        final VolatileImage backBufferImage;
+        final BufferedImage nonAnimationBufferImage;
+        final Graphics2D backBufferG2d;
+        final Graphics2D nonAnimationG2d;
+        if (useBuffers) {
+            backBufferImage = rpm.getBackBufferImage();
+            nonAnimationBufferImage = rpm.getNonAnimationBufferImage();
+            backBufferG2d = backBufferImage.createGraphics();
+            nonAnimationG2d = nonAnimationBufferImage.createGraphics();
+        } else {
+            backBufferImage = null;
+            nonAnimationBufferImage = null;
+            backBufferG2d = g2d;
+            nonAnimationG2d = g2d;
+        }
+        
+        applyRenderingHints(g2d);
+        applyRenderingHints(backBufferG2d);
+        applyRenderingHints(nonAnimationG2d);
         
-        final Graphics2D backBufferG2d = backBufferImage.createGraphics();
         final AffineTransform backBufferOriginTransform = backBufferG2d.getTransform();
 
         final Map map = getMap();
 
-        final Rectangle allRenderingClipBounds;
+        Rectangle allRenderingClipBounds;
         if (dirtyClipBounds.isEmpty()) {
             allRenderingClipBounds = clipBounds;
         } else {
             allRenderingClipBounds = clipBounds.union(dirtyClipBounds);
         }
+        
+        if (!getClientOptions().isTerrainAnimationsEnabled()) {
+            allRenderingClipBounds = dirtyClipBounds;
+        }
+        
         paintBlackBackground(backBufferG2d, allRenderingClipBounds);
         
         // Display the animated base tiles:
-        final TileClippingBounds animatedBaseTileTcb = new TileClippingBounds(map, allRenderingClipBounds);
+        final TileClippingBounds animatedBaseTileTcb = new TileClippingBounds(mapViewerBounds, map, allRenderingClipBounds);
         final long initMs = now();
-        backBufferG2d.setClip(allRenderingClipBounds);
+        if (useBuffers) {
+            backBufferG2d.setClip(allRenderingClipBounds);
+        }
         backBufferG2d.translate(animatedBaseTileTcb.clipLeftX, animatedBaseTileTcb.clipTopY);
         paintEachTile(backBufferG2d, animatedBaseTileTcb, (tileG2d, tile) -> this.tv.displayAnimatedBaseTiles(tileG2d, tile, false));
+        if (!useBuffers) {
+            backBufferG2d.translate(-animatedBaseTileTcb.clipLeftX, -animatedBaseTileTcb.clipTopY);
+        }
                
         // Display everything else:
         final long animatedBaseMs = now();
         if (!dirtyClipBounds.isEmpty()) {
-            displayToNonAnimationBufferImage(dirtyClipBounds, nonAnimationBufferImage, map);
+            displayToNonAnimationBufferImage(mapViewerBounds, dirtyClipBounds, nonAnimationG2d, map, useBuffers);
+        }
+        
+        if (useBuffers) {
+            nonAnimationG2d.dispose();
+        }
+        
+        final long nonAnimatedMs = now();
+        if (useBuffers) {
+            backBufferG2d.setTransform(backBufferOriginTransform);
+            backBufferG2d.setClip(allRenderingClipBounds);
+            backBufferG2d.drawImage(nonAnimationBufferImage, 0, 0, null);
+            backBufferG2d.dispose();
+            g2d.drawImage(backBufferImage, 0, 0, null);
         }
         
-        final long nonAnimatedMs = now();   
-        backBufferG2d.setTransform(backBufferOriginTransform);
-        backBufferG2d.setClip(allRenderingClipBounds);
-        backBufferG2d.drawImage(nonAnimationBufferImage, 0, 0, null);
-        backBufferG2d.dispose();        
-
-        g2d.drawImage(backBufferImage, 0, 0, null);      
         final long useBuffersMs = now();
         
         // Display cursor for selected tile or active unit
-        final Tile cursorTile = getVisibleCursorTile();
+        final Tile cursorTile = getVisibleCursorTile(mapViewerBounds);
         if (cursorTile != null && mapViewerState.getCursor().isActive() && !mapViewerState.getUnitAnimator().isUnitsOutForAnimation()) {
             /*
              * The cursor is hidden when units are animated. 
@@ -299,10 +425,10 @@ public final class MapViewer extends FreeColClientHolder {
         final long cursorTileMs = now();
         
         // Display goto path
-        if (mapViewerState.getUnitPath() != null) {
-            displayPath(g2d, mapViewerState.getUnitPath());
-        } else if (mapViewerState.getGotoPath() != null) {
-            displayPath(g2d, mapViewerState.getGotoPath());
+        if (mapViewerState.getGotoPath() != null) {
+            displayPath(g2d, mapViewerState.getGotoPath(), mapViewerBounds);
+        } else if (mapViewerState.getUnitPath() != null) {
+            displayPath(g2d, mapViewerState.getUnitPath(), mapViewerBounds);
         }
         final long gotoPathMs = now();
 
@@ -340,7 +466,9 @@ public final class MapViewer extends FreeColClientHolder {
             fullyRepaintedAreas.clear();
         }
         
-        verifyAndMarkAsClean(size, clipBounds);
+        if (useBuffers) {
+            verifyAndMarkAsClean(size, clipBounds);
+        }
         
         /*
          * Remove the check for "fullMapRenderedWithoutUsingBackBuffer" to get every repaint
@@ -367,6 +495,21 @@ public final class MapViewer extends FreeColClientHolder {
         return fullMapRenderedWithoutUsingBackBuffer;
     }
 
+    private void applyRenderingHints(Graphics2D g2d) {
+        if (getClientOptions().getRange(ClientOptions.GRAPHICS_QUALITY) == ClientOptions.GRAPHICS_QUALITY_LOWEST) {
+            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
+            g2d.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
+            g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_SPEED);
+            g2d.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_DISABLE);
+            g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
+            g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
+            g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
+            g2d.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_DEFAULT);
+            g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
+        }
+    }
+
+
     /**
      * Paints the dirty tiles to the buffers. The screen will
      * be updated the next time {@link #displayMap(Graphics2D, Dimension)}
@@ -391,24 +534,24 @@ public final class MapViewer extends FreeColClientHolder {
         
         final BufferedImage nonAnimationBufferImage = rpm.getNonAnimationBufferImage();
         final Map map = getMap();
-        displayToNonAnimationBufferImage(dirtyClipBounds, nonAnimationBufferImage, map);
+        final Graphics2D nonAnimationG2d = nonAnimationBufferImage.createGraphics();
+        displayToNonAnimationBufferImage(mapViewerBounds, dirtyClipBounds, nonAnimationG2d, map, true);
+        nonAnimationG2d.dispose();
         if (FreeColDebugger.debugRendering()) {
             fullyRepaintedAreas.add(dirtyClipBounds);
         }
         rpm.markAsClean();
     }
 
-    private void displayToNonAnimationBufferImage(final Rectangle dirtyClipBounds, final BufferedImage nonAnimationBufferImage,
-            final Map map) {
-        final TileClippingBounds tcb = new TileClippingBounds(map, dirtyClipBounds);
-        final Graphics2D nonAnimationG2d = nonAnimationBufferImage.createGraphics();
-        displayNonAnimationImages(nonAnimationG2d, dirtyClipBounds, tcb);
-        nonAnimationG2d.dispose();
+    private void displayToNonAnimationBufferImage(MapViewerBounds mapViewerBounds, Rectangle dirtyClipBounds, Graphics2D nonAnimationG2d, Map map, boolean useBuffers) {
+        final TileClippingBounds tcb = new TileClippingBounds(mapViewerBounds, map, dirtyClipBounds);
+        displayNonAnimationImages(nonAnimationG2d, dirtyClipBounds, tcb, useBuffers);
     }
 
     private void displayNonAnimationImages(Graphics2D nonAnimationG2d,
             Rectangle clipBounds,
-            TileClippingBounds tcb) {
+            TileClippingBounds tcb,
+            boolean useBuffers) {
         
         long t0 = now();
         final Player player = getMyPlayer(); // Check, can be null in map editor
@@ -418,10 +561,12 @@ public final class MapViewer extends FreeColClientHolder {
         to be drawn in north to prevent missing parts on partial redraws,
         as they can reach below their tiles, see BR#2580 */
         
-        nonAnimationG2d.setComposite(AlphaComposite.Clear);
-        nonAnimationG2d.fill(clipBounds);
-        nonAnimationG2d.setComposite(AlphaComposite.SrcOver);
-        nonAnimationG2d.setClip(clipBounds);
+        if (useBuffers) {
+            nonAnimationG2d.setComposite(AlphaComposite.Clear);
+            nonAnimationG2d.fill(clipBounds);
+            nonAnimationG2d.setComposite(AlphaComposite.SrcOver);
+            nonAnimationG2d.setClip(clipBounds);
+        }
         nonAnimationG2d.translate(tcb.clipLeftX, tcb.clipTopY);
         
         long t1 = now();
@@ -562,6 +707,8 @@ public final class MapViewer extends FreeColClientHolder {
         });
         
         displayDebugAiDefensiveMap(nonAnimationG2d, tcb);
+        
+        displayAreasInMapEditor(nonAnimationG2d, tcb);
 
         // Display the colony names, if needed
         long t14 = now();
@@ -578,6 +725,10 @@ public final class MapViewer extends FreeColClientHolder {
         
         long t15 = now();
         
+        if (!useBuffers) {
+            nonAnimationG2d.translate(-tcb.clipLeftX, -tcb.clipTopY);
+        }
+        
         if (logger.isLoggable(Level.FINEST)) {
             final long gap = now() - t0;
             final Map.Position bottomRight = tcb.getBottomRightDirtyTile();
@@ -609,6 +760,71 @@ public final class MapViewer extends FreeColClientHolder {
         }
     }
 
+
+    private void displayAreasInMapEditor(Graphics2D nonAnimationG2d, TileClippingBounds tcb) {
+        if (!getFreeColClient().isMapEditor()) {
+            return;
+        }
+        final MapEditorController mec = getFreeColClient().getMapEditorController();
+        if (!mec.isDisplayAreas()) {
+            return;
+        }
+        
+        final Object oldAntialiasingHint = nonAnimationG2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
+        nonAnimationG2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
+
+        final Composite oldComposite = nonAnimationG2d.getComposite();
+        nonAnimationG2d.setColor(Color.BLACK);
+        nonAnimationG2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
+         
+        final Color oldColor = nonAnimationG2d.getColor();
+        
+        final GeneralPath baseTileOutline = mapViewerScaledUtils.getBaseTileOutline();
+        paintEachTile(nonAnimationG2d, tcb, (tileG2d, tile) -> {
+            // This can easily be optimized if slow on some systems.
+            final List<Area> areas = getGame().getAreas().stream()
+                    .filter(a -> a.containsTile(tile))
+                    .filter(a -> mec.isAreaVisible(a))
+                    .collect(Collectors.toList());
+            if (areas.isEmpty()) {
+                return;
+            }
+            if (areas.size() == 1) {
+                final Area area = areas.get(0);
+                tileG2d.setColor(area.getColor());
+                tileG2d.fill(baseTileOutline);
+                return;
+            }
+            
+            final Shape oldClip = tileG2d.getClip();
+            final Stroke oldStroke = tileG2d.getStroke();
+            tileG2d.setClip(baseTileOutline);
+            
+            final int stepSize = lib.scaleInt(4);
+            tileG2d.setStroke(new BasicStroke(stepSize));
+            
+            int step = 0;
+            int index = 0;
+            while (step * stepSize < tileBounds.getHeight()) {
+                final Area area = areas.get(index);
+                tileG2d.setColor(area.getColor());
+                tileG2d.drawLine(0, step*stepSize, tileBounds.getWidth(), step*stepSize);
+                
+                step++;
+                index++;
+                if (index == areas.size()) {
+                    index = 0;
+                }
+            }
+            
+            tileG2d.setClip(oldClip);
+            tileG2d.setStroke(oldStroke);
+        });
+        nonAnimationG2d.setColor(oldColor);
+        nonAnimationG2d.setComposite(oldComposite);
+        nonAnimationG2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, oldAntialiasingHint);
+    }
+
     private void displayDebugAiDefensiveMap(Graphics2D nonAnimationG2d, TileClippingBounds tcb) {
         if (FreeColDebugger.debugShowDefenceMapForPlayer() != null
                 && getFreeColServer() != null
@@ -669,7 +885,7 @@ public final class MapViewer extends FreeColClientHolder {
             }
             g2d.setTransform(baseTransform);
         }
-    }    
+    }
     
     private void paintBlackBackground(Graphics2D g2d, final Rectangle rectangle) {
         g2d.setColor(Color.black);
@@ -799,7 +1015,7 @@ public final class MapViewer extends FreeColClientHolder {
         path.closePath();
 
         // draw everything
-        BufferedImage bi = new BufferedImage(extent, extent, BufferedImage.TYPE_INT_ARGB);
+        BufferedImage bi = ImageUtils.createBufferedImage(extent, extent);
         Graphics2D g2d = bi.createGraphics();
         g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
                              RenderingHints.VALUE_RENDER_QUALITY);
@@ -869,8 +1085,7 @@ public final class MapViewer extends FreeColClientHolder {
 
         int radius = Math.min(hPadding, vPadding);
 
-        BufferedImage bi = new BufferedImage(width, height,
-                                             BufferedImage.TYPE_INT_ARGB);
+        BufferedImage bi = ImageUtils.createBufferedImage(width, height);
         Graphics2D g2 = bi.createGraphics();
         g2.setRenderingHint(RenderingHints.KEY_RENDERING,
                             RenderingHints.VALUE_RENDER_QUALITY);
@@ -929,7 +1144,7 @@ public final class MapViewer extends FreeColClientHolder {
         cross.lineTo(offset + bar + 1, padding + bar + inset);
 
         // draw everything
-        BufferedImage bi = new BufferedImage(extent, extent, BufferedImage.TYPE_INT_ARGB);
+        BufferedImage bi = ImageUtils.createBufferedImage(extent, extent);
         Graphics2D g2d = bi.createGraphics();
         g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
         g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
@@ -947,62 +1162,78 @@ public final class MapViewer extends FreeColClientHolder {
         g2d.dispose();
         return bi;
     }
-
-
+    
     /**
      * Display a path.
      *
      * @param g2d The {@code Graphics2D} to display on.
      * @param path The {@code PathNode} to display.
      */
-    private void displayPath(Graphics2D g2d, PathNode path) {
-        final boolean debug = FreeColDebugger
-            .isInDebugMode(FreeColDebugger.DebugMode.PATHS);
-
-        for (PathNode p = path; p != null; p = p.next) {
-            Tile tile = p.getTile();
-            if (tile == null) continue;
-            Point point = mapViewerBounds.calculateTilePosition(tile, false);
-            if (point == null) continue;
-
-            BufferedImage image = (p.isOnCarrier())
-                ? ImageLibrary.getPathImage(ImageLibrary.PathType.NAVAL)
-                : (mapViewerState.getActiveUnit() != null)
-                ? ImageLibrary.getPathImage(mapViewerState.getActiveUnit())
-                : null;
-
-            BufferedImage turns = null;
-            if (mapViewerScaledUtils.getFontTiny() != null) {
-                if (debug) { // More detailed display
-                    if (mapViewerState.getActiveUnit() != null) {
-                        image = ImageLibrary.getPathNextTurnImage(mapViewerState.getActiveUnit());
-                    }
-                    turns = this.lib.getStringImage(g2d,
-                        Integer.toString(p.getTurns())
-                        + "/" + Integer.toString(p.getMovesLeft()),
-                        Color.WHITE, mapViewerScaledUtils.getFontTiny());
-                } else {
-                    turns = (p.getTurns() <= 0) ? null
-                        : this.lib.getStringImage(g2d,
-                            Integer.toString(p.getTurns()),
-                            Color.WHITE, mapViewerScaledUtils.getFontTiny());
-                }
-                g2d.setColor((turns == null) ? Color.GREEN : Color.RED);
-            }
-
-            g2d.translate(point.x, point.y);
-            if (image == null) {
-                g2d.fillOval(tileBounds.getHalfWidth(), tileBounds.getHalfHeight(), 10, 10);
-                g2d.setColor(Color.BLACK);
-                g2d.drawOval(tileBounds.getHalfWidth(), tileBounds.getHalfHeight(), 10, 10);
-            } else {
-                this.tv.displayCenteredImage(g2d, image);
-                if (turns != null) {
-                    this.tv.displayCenteredImage(g2d, turns);
-                }
-            }
-            g2d.translate(-point.x, -point.y);
+    private void displayPath(Graphics2D g2d, PathNode path, MapViewerBounds mapViewerBounds) {
+        if (path == null || path.next == null) {
+            return;
         }
+        
+        final Stroke defaultStroke = g2d.getStroke();
+        final Color oldColor = g2d.getColor();
+        final Font oldFont = g2d.getFont();
+        g2d.setStroke(new BasicStroke(lib.scaleInt(5), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
+        g2d.setFont(mapViewerScaledUtils.getFontNormal());
+
+        Point previousPoint = null;
+        for (PathNode p = path; p != null; p = p.next) {
+            final Tile newTile = p.getTile();
+            if (newTile == null) {
+                previousPoint = null;
+                continue;
+            }
+            
+            final Point nextPoint = mapViewerBounds.calculateCenterTilePosition(newTile);
+            if (nextPoint == null) {
+                previousPoint = null;
+                continue;
+            }
+            
+            final Color pathLineColor = (p.getTurns() > 0) ? new Color(255, 255, 0, 128) : new Color(0, 255, 0, 128);
+            g2d.setColor(pathLineColor);
+            
+            if (previousPoint != null) {
+                g2d.drawLine(previousPoint.x, previousPoint.y, nextPoint.x, nextPoint.y);
+            }
+            previousPoint = nextPoint;
+
+        }
+        
+        g2d.setStroke(new BasicStroke(lib.scaleInt(2)));
+        final FontMetrics fm = g2d.getFontMetrics();
+        final int fontCenterY = (fm.getAscent() - fm.getDescent() - fm.getLeading()) / 2;
+        for (PathNode p = path; p != null; p = p.next) {
+            final Tile newTile = p.getTile();
+            if (newTile == null) {
+                continue;
+            }
+            final Point newP = mapViewerBounds.calculateCenterTilePosition(newTile);
+            if (newP == null) {
+                continue;
+            }
+           
+            final int r = lib.scaleInt(20);
+            final Color pathBgColor = (p.getTurns() > 0) ? new Color(255, 255, 0, 255) : new Color(0, 255, 0, 255);
+            g2d.setColor(pathBgColor);
+            g2d.fillOval(newP.x - r/2, newP.y - r/2, r, r);
+            g2d.setColor(new Color(0, 0, 0, 128));
+            g2d.drawOval(newP.x - r/2, newP.y - r/2, r, r);
+            
+            if (mapViewerScaledUtils.getFontTiny() != null && p.getTurns() > 0) {
+                final String text = Integer.toString(p.getTurns());
+                final Rectangle2D bounds = fm.getStringBounds(text, g2d);
+                g2d.setColor(new Color(0, 0, 0));
+                g2d.drawString(text, newP.x - (int) bounds.getWidth() / 2, newP.y + fontCenterY);
+            }
+        }
+        g2d.setStroke(defaultStroke);
+        g2d.setColor(oldColor);
+        g2d.setFont(oldFont);
     }
 
     /**
@@ -1076,22 +1307,55 @@ public final class MapViewer extends FreeColClientHolder {
         // FOR DEBUGGING
         net.sf.freecol.server.ai.AIUnit au;
         if (FreeColDebugger.isInDebugMode(FreeColDebugger.DebugMode.MENUS)
-            && player != null && !player.owns(unit)
-            && unit.getOwner().isAI()
-            && getFreeColServer() != null
-            && getFreeColServer().getAIMain() != null
-            && (au = getFreeColServer().getAIMain().getAIUnit(unit)) != null) {
+                && player != null && !player.owns(unit)
+                && unit.getOwner().isAI()
+                && getFreeColServer() != null
+                && getFreeColServer().getAIMain() != null
+                && (au = getFreeColServer().getAIMain().getAIUnit(unit)) != null) {
             if (FreeColDebugger.debugShowMission()) {
-                g2d.setColor(Color.WHITE);
-                g2d.drawString((!au.hasMission()) ? "No mission"
-                    : lastPart(au.getMission().getClass().toString(), "."),
-                    0, 0);
+                String missionString = (!au.hasMission())
+                        ? "No mission"
+                        : au.getMission().getClass().getSimpleName().replaceAll("Mission$", "");
+                final Font origFont = g2d.getFont();
+                if (FreeColDebugger.debugShowMissionInfo() && au.hasMission()) {
+                    missionString += "\n" + au.getMission().toStringForDebugExtraMissionInfo();
+                    g2d.setFont(origFont.deriveFont(origFont.getSize2D() * 2 / 3));
+                }
+                drawCenteredMultilineDebugText(g2d, missionString, 0, 0);
+                g2d.setFont(origFont);
             }
-            if (FreeColDebugger.debugShowMissionInfo() && au.hasMission()) {
-                g2d.setColor(Color.WHITE);
-                g2d.drawString(au.getMission().toString(), 0, 25);
+        }
+    }
+    
+    private void drawCenteredMultilineDebugText(Graphics2D g2d, String text, int x, int y) {
+        final FontMetrics fm = g2d.getFontMetrics();
+        final String[] lines = text.split("\n");
+        final Rectangle2D[] lineTextBoundingBoxes = new Rectangle2D[lines.length];
+        
+        Dimension allTextBoundingBox = new Dimension(0, 0);
+        int backgroundOffsetY = 0;
+        for (int i=0; i<lines.length; i++) {
+            Rectangle2D lineTextBoundingBox = fm.getStringBounds(lines[i], g2d);
+            lineTextBoundingBoxes[i] = lineTextBoundingBox;
+            allTextBoundingBox = new Dimension((int) Math.max(allTextBoundingBox.width, lineTextBoundingBox.getWidth()),
+                    (int) (allTextBoundingBox.height + lineTextBoundingBox.getHeight()));
+            if ((int) lineTextBoundingBox.getY() < backgroundOffsetY) {
+                backgroundOffsetY = (int) lineTextBoundingBox.getY(); 
             }
         }
+        
+        final int tileCenterX = (tileBounds.getWidth() - allTextBoundingBox.width) / 2;
+        
+        g2d.setColor(new Color(0, 0, 0, 128));
+        g2d.fillRect(x + tileCenterX, y + backgroundOffsetY, allTextBoundingBox.width, allTextBoundingBox.height);
+        
+        int offsetY = 0;
+        for (int i=0; i<lines.length; i++) {
+            g2d.setColor(Color.WHITE);
+            final int centerX = (int) (allTextBoundingBox.getWidth() - lineTextBoundingBoxes[i].getWidth()) / 2;
+            g2d.drawString(lines[i], x + centerX + tileCenterX, y + offsetY);
+            offsetY += (int) lineTextBoundingBoxes[i].getHeight();
+        }
     }
 
     /**
@@ -1231,7 +1495,7 @@ public final class MapViewer extends FreeColClientHolder {
      *
      * @return The {@code Tile} found or null.
      */
-    private Tile getVisibleCursorTile() {
+    private Tile getVisibleCursorTile(MapViewerBounds mapViewerBounds) {
         Tile ret = mapViewerState.getCursorTile();
         return (mapViewerBounds.isTileVisible(ret)) ? ret : null;
     }
@@ -1283,8 +1547,7 @@ public final class MapViewer extends FreeColClientHolder {
     public MapViewerRepaintManager getMapViewerRepaintManager() {
         return rpm;
     }
-    
-    
+
     /**
      * Paints a single tile using the provided callback.
      * 
@@ -1374,7 +1637,6 @@ public final class MapViewer extends FreeColClientHolder {
         g2d.translate(-xt0, -yt0);
     }
     
-    
     /**
      * Calculates the tile clipping bounds from a Graphics' clipBounds.
      * 
@@ -1388,8 +1650,7 @@ public final class MapViewer extends FreeColClientHolder {
      * {@link #getExtendedTiles() extendedTiles} and
      * {@link #getSuperExtendedTiles() superExtendedTiles}.
      */
-    private final class TileClippingBounds {
-        
+    private static final class TileClippingBounds {
         private final Map.Position topLeftDirtyTile;
         private final Map.Position bottomRightDirtyTile;
         private final int clipLeftX;
@@ -1400,8 +1661,8 @@ public final class MapViewer extends FreeColClientHolder {
         private final List<Tile> extendedTiles;
         private final List<Tile> superExtendedTiles;
         
-        private TileClippingBounds(Map map, Rectangle clipBounds) {
-            // Determine which tiles need to be redrawn          
+        private TileClippingBounds(MapViewerBounds mapViewerBounds, Map map, Rectangle clipBounds) {
+            final TileBounds tileBounds = mapViewerBounds.getTileBounds();
             final int firstRowTiles = (clipBounds.y - mapViewerBounds.getTopLeftVisibleTilePoint().y) / tileBounds.getHalfHeight() - 1;
             this.clipTopY = mapViewerBounds.getTopLeftVisibleTilePoint().y + firstRowTiles * tileBounds.getHalfHeight();
             final int firstRow = mapViewerBounds.getTopLeftVisibleTile().getY() + firstRowTiles;
@@ -1412,12 +1673,19 @@ public final class MapViewer extends FreeColClientHolder {
             
             final int lastRowTiles = (clipBounds.y + clipBounds.height - mapViewerBounds.getTopLeftVisibleTilePoint().y) / tileBounds.getHalfHeight();
             final int lastRow = mapViewerBounds.getTopLeftVisibleTile().getY() + lastRowTiles;
-            
             final int lastColumnTiles = (clipBounds.x + clipBounds.width - mapViewerBounds.getTopLeftVisibleTilePoint().x) / tileBounds.getWidth();
             final int lastColumn = mapViewerBounds.getTopLeftVisibleTile().getX() + lastColumnTiles;
-            
+
             this.topLeftDirtyTile = new Map.Position(firstColumn, firstRow);
             this.bottomRightDirtyTile = new Map.Position(lastColumn, lastRow);
+          
+            /* For testing MapViewerBounds -- just ignore the logic above, and do:
+            this.topLeftDirtyTile = mapViewerBounds.getTopLeftVisibleTile();
+            //this.bottomRightDirtyTile = new Map.Position(mapViewerBounds.getTopLeftVisibleTile().x + 50, mapViewerBounds.getTopLeftVisibleTile().y + 50);
+            this.bottomRightDirtyTile = mapViewerBounds.getBottomRightVisibleTile();
+            this.clipLeftX = mapViewerBounds.getTopLeftVisibleTilePoint().x;
+            this.clipTopY = mapViewerBounds.getTopLeftVisibleTilePoint().y;
+            */
             
             final int subMapWidth = bottomRightDirtyTile.getX() - topLeftDirtyTile.getX() + 1;
             final int subMapHeight = bottomRightDirtyTile.getY() - topLeftDirtyTile.getY() + 1;
