@@ -57,6 +57,7 @@ import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TileItem;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.util.ImageUtils;
 
 
 /**
@@ -156,8 +157,7 @@ public final class TileViewer extends FreeColClientHolder {
             = (overlayImage != null) ? overlayImage.getHeight()
             : (tileType.isForested()) ? this.lib.getForestedTileSize().height
             : this.tileHeight;
-        BufferedImage image = new BufferedImage(this.tileWidth, compoundHeight,
-                                                BufferedImage.TYPE_INT_ARGB);
+        BufferedImage image = ImageUtils.createBufferedImage(this.tileWidth, compoundHeight);
         Graphics2D g2d = image.createGraphics();
         g2d.translate(0, compoundHeight - this.tileHeight);
         displayAnimatedBaseTiles(g2d, tile, false);
@@ -184,8 +184,7 @@ public final class TileViewer extends FreeColClientHolder {
             : (tileType.isForested())
             ? this.lib.getForestedTileSize().height
             : this.tileHeight;
-        BufferedImage image = new BufferedImage(this.tileWidth, compoundHeight,
-                                                BufferedImage.TYPE_INT_ARGB);
+        BufferedImage image = ImageUtils.createBufferedImage(this.tileWidth, compoundHeight);
         Graphics2D g2d = image.createGraphics();
         g2d.translate(0, compoundHeight - this.tileHeight);
         displayTile(g2d, tile, player, overlayImage);
@@ -215,8 +214,7 @@ public final class TileViewer extends FreeColClientHolder {
             : ((tileType.isForested())
                 ? this.lib.getForestedTileSize().height
                 : this.tileHeight);
-        BufferedImage image = new BufferedImage(this.tileWidth, compoundHeight,
-                                                BufferedImage.TYPE_INT_ARGB);
+        BufferedImage image = ImageUtils.createBufferedImage(this.tileWidth, compoundHeight);
         Graphics2D g2d = image.createGraphics();
         g2d.translate(0, compoundHeight - this.tileHeight);
         displayColonyTile(g2d, tile, colony, overlayImage);
@@ -710,11 +708,8 @@ public final class TileViewer extends FreeColClientHolder {
                                  BufferedImage overlayImage) {
         // ATTENTION: we assume that only overlays and forests
         // might be taller than a tile.
-        BufferedImage image = new BufferedImage(this.tileWidth,
-            this.tileHeight+this.halfHeight, BufferedImage.TYPE_INT_ARGB);
-        Graphics2D g1 = (Graphics2D)image.getGraphics();
-        g1.translate(0, this.halfHeight);
-        // layer additions and improvements according to zIndex
+        
+         // layer additions and improvements according to zIndex
         List<TileItem> tileItems = new ArrayList<>(tile.getCompleteItems());
         Collections.sort(tileItems, (a, b) -> {
             return Integer.valueOf(a.getZIndex()).compareTo(b.getZIndex());
@@ -722,7 +717,7 @@ public final class TileViewer extends FreeColClientHolder {
         int startIndex = 0;
         for (int index = startIndex; index < tileItems.size(); index++) {
             if (tileItems.get(index).getZIndex() < Tile.OVERLAY_ZINDEX) {
-                displayTileItem(g1, tile, tileItems.get(index));
+                displayTileItem(g2d, tile, rop, tileItems.get(index));
                 startIndex = index + 1;
             } else {
                 startIndex = index;
@@ -731,12 +726,12 @@ public final class TileViewer extends FreeColClientHolder {
         }
         // Tile Overlays (eg. hills and mountains)
         if (overlayImage != null) {
-            g1.drawImage(overlayImage,
-                0, (this.tileHeight - overlayImage.getHeight()), null);
+            g2d.drawImage(overlayImage, rop,
+                0, (this.tileHeight - overlayImage.getHeight()));
         }
         for (int index = startIndex; index < tileItems.size(); index++) {
             if (tileItems.get(index).getZIndex() < Tile.FOREST_ZINDEX) {
-                displayTileItem(g1, tile, tileItems.get(index));
+                displayTileItem(g2d, tile, rop, tileItems.get(index));
                 startIndex = index + 1;
             } else {
                 startIndex = index;
@@ -747,17 +742,15 @@ public final class TileViewer extends FreeColClientHolder {
         if (tile.isForested()) {
             BufferedImage forestImage = this.lib.getScaledForestImage(tile.getType(),
                 tile.getRiverStyle());
-            g1.drawImage(forestImage, 0, (this.tileHeight - forestImage.getHeight()), null);
+            g2d.drawImage(forestImage, rop, 0, (this.tileHeight - forestImage.getHeight()));
         } else if (getClientOptions().getRange(ClientOptions.GRAPHICS_QUALITY) >= ClientOptions.GRAPHICS_QUALITY_HIGH) {
-            drawForestCornerImages(tile, g1);
+            drawForestCornerImages(tile, g2d);
         }
         
         // draw all remaining items
         for (TileItem ti : tileItems.subList(startIndex, tileItems.size())) {
-            displayTileItem(g1, tile, ti);
+            displayTileItem(g2d, tile, rop, ti);
         }
-        g1.dispose();
-        g2d.drawImage(image, rop, 0, -this.halfHeight);
     }
 
 
@@ -794,14 +787,15 @@ public final class TileViewer extends FreeColClientHolder {
      *
      * @param g2d The {@code Graphics} to draw to.
      * @param tile The {@code Tile} to draw from.
+     * @param rop An optional RescaleOp for fog of war.
      * @param item The {@code TileItem} to draw.
      */
-    private void displayTileItem(Graphics2D g2d, Tile tile, TileItem item) {
+    private void displayTileItem(Graphics2D g2d, Tile tile, RescaleOp rop, TileItem item) {
         if (item instanceof TileImprovement) {
             TileImprovement ti = (TileImprovement)item;
             if (!ti.isComplete()) return;
             if (ti.isRoad()) {
-                this.rp.displayRoad(g2d, tile);
+                this.rp.displayRoad(g2d, tile, rop);
             } else if (ti.isRiver()) {
                 if (getClientOptions().isRiverAnimationEnabled()) {
                     // Painted elsewhere.
@@ -809,19 +803,20 @@ public final class TileViewer extends FreeColClientHolder {
                 }
                 final BufferedImage img = this.lib.getAnimatedScaledRiverTerrainImage(tile, 0);
                 if (img != null) {
-                    g2d.drawImage(img, 0, 0, null);
+                    g2d.drawImage(img, rop, 0, 0);
                 }
             } else {
-                BufferedImage img
-                    = this.lib.getTileImprovementImage(ti.getType().getId());
-                if (img != null) g2d.drawImage(img, 0, 0, null);
+                BufferedImage img = this.lib.getTileImprovementImage(ti.getType().getId());
+                if (img != null) g2d.drawImage(img, rop, 0, 0);
             }
         } else if (item instanceof LostCityRumour) {
             displayCenteredImage(g2d,
-                this.lib.getScaledImage(ImageLibrary.LOST_CITY_RUMOUR));
+                this.lib.getScaledImage(ImageLibrary.LOST_CITY_RUMOUR),
+                rop);
         } else if (item instanceof Resource) {
             displayCenteredImage(g2d,
-                this.lib.getScaledResourceImage((Resource)item));
+                this.lib.getScaledResourceImage((Resource)item),
+                rop);
         }
     }
 }
