@@ -45,6 +45,7 @@ import net.sf.freecol.client.gui.tooltip.*;
 import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ProductionInfo;
 import net.sf.freecol.common.model.Unit;
@@ -67,6 +68,8 @@ public class BuildingPanel extends MigPanel implements PropertyChangeListener {
 
     /** Labels for any units present. */
     private final List<UnitLabel> unitLabels = new ArrayList<>();
+    
+    private final Dimension unscaledMinimumWorkerAndProductionSize;
 
 
     /**
@@ -76,14 +79,28 @@ public class BuildingPanel extends MigPanel implements PropertyChangeListener {
      * @param building The building to display information from.
      */
     public BuildingPanel(FreeColClient freeColClient, Building building) {
-        super(new MigLayout("", "[32][32][32]", "[32][44]"));
-
+        super(new MigLayout("ins 0 0 0 0, gap 0 0", layoutConstraintString(building), "0:push[32!, bottom]0[58!, bottom]"));
         this.freeColClient = freeColClient;
         this.building = building;
+        this.unscaledMinimumWorkerAndProductionSize = new Dimension(Math.max(64, building.getUnitCapacity() * 32), 90);
 
         setToolTipText(" ");
     }
 
+    private static String layoutConstraintString(Building building) {
+        return "push" + layoutWorkerString(building) + "push";
+    }
+
+    private static String layoutWorkerString(Building building) {
+        if (building.getUnitCapacity() < 2) {
+            return "[64!]";
+        }
+        final StringBuilder sb = new StringBuilder();
+        for (int i=0; i<building.getUnitCapacity(); i++) {
+            sb.append("[32!]");
+        }
+        return sb.toString();
+    }
 
     private ImageLibrary getImageLibrary() {
         return this.freeColClient.getGUI().getFixedImageLibrary();
@@ -176,15 +193,22 @@ public class BuildingPanel extends MigPanel implements PropertyChangeListener {
             unitLabels.add(unitLabel);
             add(unitLabel);
         }
-
-        ImageLibrary lib = getImageLibrary();
-        Image buildingImage = lib.getScaledBuildingImage(building);
-        setPreferredSize(new Dimension(buildingImage.getWidth(null), 
-                                       buildingImage.getHeight(null)));
+        
+        final Dimension minimumWorkerAndProductionSize = getImageLibrary().scale(unscaledMinimumWorkerAndProductionSize);
+        
+        final BufferedImage image = getImageLibrary().getScaledBuildingImage(building);
+        final Dimension minimumSizeForImage = new Dimension(image.getWidth(), image.getHeight());
+        setMinimumSize(largestWidthAndHeight(minimumWorkerAndProductionSize, minimumSizeForImage));
+        
+        final Dimension preferredSizeForImage = getImageLibrary().determineMaxSizeUsingSizeFromAllLevels(building.getType(), building.getOwner());
+        setPreferredSize(largestWidthAndHeight(minimumWorkerAndProductionSize, preferredSizeForImage));
         revalidate();
         repaint();
     }
 
+    private Dimension largestWidthAndHeight(Dimension d1, Dimension d2) {
+        return new Dimension(Math.max(d1.width, d2.width), Math.max(d1.height, d2.height));
+    }
 
     /**
      * Get the building this panel displays.
@@ -239,8 +263,13 @@ public class BuildingPanel extends MigPanel implements PropertyChangeListener {
      */
     @Override
     public void paintComponent(Graphics g) {
-        ImageLibrary lib = getImageLibrary();
-        g.drawImage(lib.getScaledBuildingImage(building), 0, 0, this);
+        final ImageLibrary lib = getImageLibrary();
+        final BufferedImage image = lib.getScaledBuildingImage(building);
+        final Dimension size = getSize();
+        g.drawImage(image,
+                (size.width - image.getWidth()) / 2,
+                size.height - image.getHeight(),
+                this);
     }
 
 
