@@ -28,6 +28,7 @@ import java.net.InetAddress;
 import java.util.List;
 import java.util.logging.Logger;
 
+import javax.swing.AbstractAction;
 import javax.swing.ButtonGroup;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
@@ -47,11 +48,11 @@ import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColModFile;
 import net.sf.freecol.common.io.FreeColRules;
-import net.sf.freecol.common.io.FreeColTcFile;
 import net.sf.freecol.common.metaserver.MetaServerUtils;
 import net.sf.freecol.common.metaserver.ServerInfo;
 import net.sf.freecol.common.model.NationOptions.Advantages;
 import net.sf.freecol.common.model.Specification;
+import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.option.OptionGroup;
 
 
@@ -191,15 +192,15 @@ public final class NewPanel extends FreeColPanel implements ItemListener {
         this.buttonGroup.add(single);
         single.setActionCommand(String.valueOf(NewPanelAction.SINGLE));
         single.addActionListener(ae -> enableComponents());
-        this.buttonGroup.add(join);
-        join.setActionCommand(String.valueOf(NewPanelAction.JOIN));
-        join.addActionListener(ae -> enableComponents());
         this.buttonGroup.add(start);
         start.setActionCommand(String.valueOf(NewPanelAction.START));
         start.addActionListener(ae -> enableComponents());
         this.buttonGroup.add(meta);
         meta.setActionCommand(String.valueOf(NewPanelAction.META_SERVER));
         meta.addActionListener(ae -> enableComponents());
+        this.buttonGroup.add(join);
+        join.setActionCommand(String.valueOf(NewPanelAction.JOIN));
+        join.addActionListener(ae -> enableComponents());
         single.setSelected(true);
 
         String name = getClientOptions().getText(ClientOptions.NAME);
@@ -300,7 +301,12 @@ public final class NewPanel extends FreeColPanel implements ItemListener {
                     gui.showMainPanel(null);
                 }
             });
-        setCancelComponent(cancel);
+        setEscapeAction(new AbstractAction() {
+            @Override
+            public void actionPerformed(ActionEvent ae) {
+                cancel.doClick();
+            }
+        });
 
         // Add all the components
         add(Utility.localizedHeader("newPanel.newGamePanel",
@@ -560,48 +566,52 @@ public final class NewPanel extends FreeColPanel implements ItemListener {
         final GUI gui = getGUI();
         final String command = ae.getActionCommand();
 
-        switch (Enum.valueOf(NewPanelAction.class, command)) {
-        case OK:
-            FreeCol.setName(getSelectedName());
-            FreeCol.setAdvantages(getSelectedAdvantages());
-            FreeCol.setRules(getSelectedRules().getId());
-
-            NewPanelAction action = Enum.valueOf(NewPanelAction.class,
-                buttonGroup.getSelection().getActionCommand());
-            switch (action) {
-            case SINGLE:
-                this.specification.prepare(getSelectedAdvantages(),
-                                           this.difficulty);
-                if (cc.startSinglePlayerGame(this.specification)) return;
-                break;
-            case JOIN:
-                int joinPort = getSelectedPort(this.joinPortField);
-                if (joinPort < 0) break;
-                if (cc.joinMultiplayerGame(this.joinNameField.getText(),
-                                           joinPort)) return;
-                break;
-            case START:
-                int serverPort = getSelectedPort(this.serverPortField);
-                if (serverPort < 0) break;
-                this.specification.prepare(getSelectedAdvantages(),
-                                           this.difficulty);
-                final InetAddress serverAddress = (InetAddress) this.serverAddressBox.getSelectedItem();
-                final boolean publicServerValue = this.publicServer.isSelected()
-                        && !serverAddress.isLoopbackAddress();
-                if (cc.startMultiplayerGame(this.specification,
-                        publicServerValue, serverAddress, serverPort)) return;
-                break;
-            case META_SERVER:
-                List<ServerInfo> servers = MetaServerUtils.getServerList();
-                if (servers != null) gui.showServerListPanel(servers);
+        try {
+            switch (Enum.valueOf(NewPanelAction.class, command)) {
+            case OK:
+                FreeCol.setName(getSelectedName());
+                FreeCol.setAdvantages(getSelectedAdvantages());
+                FreeCol.setRules(getSelectedRules().getId());
+    
+                NewPanelAction action = Enum.valueOf(NewPanelAction.class,
+                    buttonGroup.getSelection().getActionCommand());
+                switch (action) {
+                case SINGLE:
+                    this.specification.prepare(getSelectedAdvantages(),
+                                               this.difficulty);
+                    if (cc.startSinglePlayerGame(this.specification)) return;
+                    break;
+                case JOIN:
+                    int joinPort = getSelectedPort(this.joinPortField);
+                    if (joinPort < 0) break;
+                    if (cc.joinMultiplayerGame(this.joinNameField.getText(),
+                                               joinPort)) return;
+                    break;
+                case START:
+                    int serverPort = getSelectedPort(this.serverPortField);
+                    if (serverPort < 0) break;
+                    this.specification.prepare(getSelectedAdvantages(),
+                                               this.difficulty);
+                    final InetAddress serverAddress = (InetAddress) this.serverAddressBox.getSelectedItem();
+                    final boolean publicServerValue = this.publicServer.isSelected()
+                            && !serverAddress.isLoopbackAddress();
+                    if (cc.startMultiplayerGame(this.specification,
+                            publicServerValue, serverAddress, serverPort)) return;
+                    break;
+                case META_SERVER:
+                    List<ServerInfo> servers = MetaServerUtils.getServerList();
+                    if (servers != null) gui.showServerListPanel(servers);
+                    break;
+                default:
+                    break;
+                }
                 break;
             default:
+                super.actionPerformed(ae);
                 break;
             }
-            break;
-        default:
-            super.actionPerformed(ae);
-            break;
+        } catch (RuntimeException e) {
+            gui.showErrorPanel(e, StringTemplate.key("error.unspecified"));
         }
     }
 
