@@ -31,6 +31,7 @@ import javax.swing.UIManager;
 import javax.swing.UnsupportedLookAndFeelException;
 import javax.swing.plaf.ColorUIResource;
 import javax.swing.plaf.metal.DefaultMetalTheme;
+import javax.swing.plaf.metal.MetalIconFactory;
 import javax.swing.plaf.metal.MetalLookAndFeel;
 
 import net.sf.freecol.client.gui.ImageLibrary;
@@ -114,16 +115,20 @@ public class FreeColLookAndFeel extends MetalLookAndFeel {
 
     private static final Class uiClasses[] = {
         FreeColButtonUI.class,
+        FreeColCheckBoxMenuItemUI.class,
         FreeColCheckBoxUI.class,
         FreeColComboBoxUI.class,
         FreeColFileChooserUI.class,
+        FreeColInternalFrameUI.class,
         FreeColLabelUI.class,
         FreeColListUI.class,
         FreeColMenuBarUI.class,
         FreeColMenuItemUI.class,
+        FreeColMenuUI.class,
         FreeColOptionPaneUI.class,
         FreeColPanelUI.class,
         FreeColPopupMenuUI.class,
+        FreeColRadioButtonMenuItemUI.class,
         FreeColRadioButtonUI.class,
         FreeColScrollPaneUI.class,
         FreeColTableHeaderUI.class,
@@ -135,6 +140,8 @@ public class FreeColLookAndFeel extends MetalLookAndFeel {
         FreeColTransparentPanelUI.class,
         FreeColSpinnerUI.class
     };
+    
+    private static float scaleFactor = 1;
 
 
     /**
@@ -211,7 +218,14 @@ public class FreeColLookAndFeel extends MetalLookAndFeel {
         u.put("cursor.go", ImageLibrary.getCursor());
 
         u.put("CheckBox.icon", (LazyValue) t -> FreeColCheckBoxUI.createCheckBoxIcon());
+        u.put("CheckBoxMenuItem.checkIcon", (LazyValue) t -> FreeColCheckBoxUI.createCheckBoxIcon());
         
+        u.put("RadioButton.icon", (LazyValue) t -> FreeColRadioButtonUI.createRadioButtonIcon());
+        u.put("RadioButtonMenuItem.checkIcon", (LazyValue) t -> FreeColRadioButtonUI.createRadioButtonIcon());
+        
+        // TODO: We might want to allow overriding font colors for the menu:
+        //u.put("Menu.foreground", java.awt.Color.WHITE);
+
         return u;
     }
 
@@ -247,6 +261,23 @@ public class FreeColLookAndFeel extends MetalLookAndFeel {
             }
         }
     }
+    
+    public static void setScaleFactor(float scaleFactor) {
+        FreeColLookAndFeel.scaleFactor = scaleFactor;
+        final UIDefaults u = UIManager.getLookAndFeelDefaults();
+        final int scrollbarWidth = (int) (17 * scaleFactor);
+        u.put("ScrollBar.width", scrollbarWidth);
+        
+        int internalFrameIconSize = (int) (16 * scaleFactor);
+        u.put("InternalFrame.closeIcon", (LazyValue) t -> MetalIconFactory.getInternalFrameCloseIcon(internalFrameIconSize));
+        u.put("InternalFrame.maximizeIcon", (LazyValue) t -> MetalIconFactory. getInternalFrameMaximizeIcon(internalFrameIconSize));
+        u.put("InternalFrame.iconifyIcon", (LazyValue) t -> MetalIconFactory.getInternalFrameMinimizeIcon(internalFrameIconSize));
+        u.put("InternalFrame.minimizeIcon", (LazyValue) t -> MetalIconFactory.getInternalFrameAltMaximizeIcon(internalFrameIconSize));
+    }
+    
+    public static float getScaleFactor() {
+        return scaleFactor;
+    }
 
     /**
      * Gets a one line description of this Look and Feel.
