@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,6 +24,10 @@ package net.sf.freecol.common;
  * The Exception thrown by the FreeCol application.
  */
 public final class FreeColException extends Exception {
+
+    /** Should this exception allow debug runs to continue? */
+    private boolean allowDebug = false;
+    
     
     /**
      * Build a new FreeCol specific exception with the given message.
@@ -38,9 +42,38 @@ public final class FreeColException extends Exception {
      * Build a new FreeCol specific exception with the given message.
      *
      * @param message The message for this exception.
-     * @param throwable The <code>Throwable</code> cause for the exception.
+     * @param throwable The {@code Throwable} cause for the exception.
      */
     public FreeColException(String message, Throwable throwable) {
         super(message, throwable);
     }
+
+    /**
+     * Build a new FreeCol specific exception with the given message.
+     * 
+     * @param throwable The {@code Throwable} cause for the exception.
+     */
+    public FreeColException(Throwable throwable) {
+        super(throwable);
+    }
+
+
+    /**
+     * Make this exception *not* cause debug runs to end.
+     *
+     * @return This exception.
+     */
+    public FreeColException preserveDebug() {
+        this.allowDebug = true;
+        return this;
+    }
+
+    /**
+     * Get the debug state.
+     *
+     * @return Whether debug runs should continue following this exception.
+     */
+    public boolean debugAllowed() {
+        return this.allowDebug;
+    }
 }
