@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,9 +26,9 @@ import java.io.OutputStreamWriter;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.io.Writer;
+import java.nio.charset.StandardCharsets;
 
 import java.util.Collection;
-import java.util.Locale;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -36,20 +36,20 @@ import javax.xml.namespace.NamespaceContext;
 import javax.xml.stream.XMLOutputFactory;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamWriter;
-import javax.xml.transform.OutputKeys;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.stream.StreamSource;
 
 import net.sf.freecol.common.model.FreeColObject;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Player;
+import static net.sf.freecol.common.util.StringUtils.*;
+import net.sf.freecol.common.util.Utils;
 
 
 /**
- * A wrapper for <code>XMLStreamWriter</code> and potentially an
+ * A wrapper for {@code XMLStreamWriter} and potentially an
  * underlying stream.  Adds on many useful utilities for writing
  * XML and FreeCol values.
  *
@@ -64,130 +64,155 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
 
     private static final Logger logger = Logger.getLogger(FreeColXMLWriter.class.getName());
 
-    /** Magic properties to indent files if supported. */
-    private static final String[] indentProps = {
-        OutputKeys.INDENT, "yes",
-        "{http://xml.apache.org/xslt}indent-amount", "2"
-    };
-
     /** The scope of a FreeCol object write. */
-    public static enum WriteScope {
+    public static enum WriteScopeType {
         CLIENT,  // Only the client-visible information
         SERVER,  // Full server-visible information
-        SAVE;    // Absolutely everything needed to save the game state
+        SAVE     // Absolutely everything needed to save the game state
+    };
 
-        private Player player = null; // The player to write to.
+    public static class WriteScope {
+
+        private final WriteScopeType scopeType;
+        
+        private final Player player;
 
 
+        WriteScope(WriteScopeType scopeType, Player player) {
+            this.scopeType = scopeType;
+            this.player = player;
+        }
+        
         public static WriteScope toClient(Player player) {
-            if (player == null) {
-                throw new IllegalArgumentException("Null player.");
-            }
-            WriteScope ret = WriteScope.CLIENT;
-            ret.player = player;
-            return ret;
+            return new WriteScope(WriteScopeType.CLIENT, player);
         }            
 
         public static WriteScope toServer() {
-            return WriteScope.SERVER;
+            return new WriteScope(WriteScopeType.SERVER, null);
         }
 
         public static WriteScope toSave() {
-            return WriteScope.SAVE;
+            return new WriteScope(WriteScopeType.SAVE, null);
         }
 
         public boolean isValid() {
-            return (this == WriteScope.CLIENT) == (player != null);
+            return (this.scopeType == WriteScopeType.CLIENT)
+                == (this.player != null);
         }
 
         public boolean validForSave() {
-            return this == WriteScope.SAVE;
+            return this.scopeType == WriteScopeType.SAVE;
         }
 
         public boolean validFor(Player player) {
-            return this != WriteScope.CLIENT || this.player == player;
+            return this.scopeType != WriteScopeType.CLIENT
+                || this.player == player;
         }
 
         public Player getClient() {
             return this.player;
         }
+
+        public String toString() {
+            String ret = this.scopeType.toString();
+            if (this.scopeType == WriteScopeType.CLIENT) {
+                ret += ":" + ((this.player == null) ? "INVALID"
+                    : this.player.getId());
+            }
+            return ret;
+        }
     }
 
 
-    /** The stream to write to. */
+    /** The internal XML writer to write XML to. */
     private final XMLStreamWriter xmlStreamWriter;
 
-    /** Internal writer to accumulate into when pretty printing. */
-    private StringWriter stringWriter = null;
-    /** The final output writer to use when pretty printing. */
-    private Writer outputWriter = null;
+    /** An internal writer to accumulate XML into. */
+    private final StringWriter stringWriter;
+
+    /** An optional transformer to handle indentation. */
+    private final Transformer transformer;
+
+    /** The writer that receives the final output. */
+    private final Writer outputWriter;
 
     /** A write scope to use for FreeCol object writes. */
     private WriteScope writeScope;
 
 
     /**
-     * Creates a new <code>FreeColXMLWriter</code>.
+     * Creates a new {@code FreeColXMLWriter}.
      *
-     * @param outputStream The <code>OutputStream</code> to create
-     *     an <code>FreeColXMLWriter</code> for.
-     * @param scope The <code>WriteScope</code> to use for FreeCol
+     * @param outputStream The {@code OutputStream} to create
+     *     an {@code FreeColXMLWriter} for.
+     * @param scope The {@code WriteScope} to use for FreeCol
      *     object writes.
      * @param indent If true, produce indented output if supported.
-     * @exception IOException if thrown while creating the
-     *     <code>XMLStreamWriter</code>.
+     * @exception IOException if there is a problem while creating the
+     *     {@code FreeColXMLWriter}.
      */
     public FreeColXMLWriter(OutputStream outputStream, WriteScope scope,
                             boolean indent) throws IOException {
-        this(new OutputStreamWriter(outputStream, "UTF-8"), scope, indent);
+        this(new OutputStreamWriter(outputStream, StandardCharsets.UTF_8),
+             scope, indent);
     }
 
     /**
-     * Creates a new <code>FreeColXMLWriter</code>.
+     * Creates a new {@code FreeColXMLWriter}.
      *
-     * @param writer A <code>Writer</code> to create an
-     *     <code>FreeColXMLWriter</code> for.
-     * @param scope The <code>WriteScope</code> to use for FreeCol
-     *     object writes.
-     * @exception IOException if thrown while creating the
-     *     <code>XMLStreamWriter</code>.
+     * @param writer A {@code Writer} to create an 
+     *     {@code FreeColXMLWriter} for.
+     * @exception IOException if there is a problem while creating the
+     *     {@code FreeColXMLWriter}.
      */
-    public FreeColXMLWriter(Writer writer, WriteScope scope) throws IOException {
+    public FreeColXMLWriter(Writer writer) throws IOException {
+        this(writer, WriteScope.toSave());
+    }
+
+    /**
+     * Creates a new {@code FreeColXMLWriter}.
+     *
+     * @param writer A {@code Writer} to create an
+     *     {@code FreeColXMLWriter} for.
+     * @param scope The {@code WriteScope} to use for FreeCol objects.
+     * @exception IOException if there is a problem while creating the
+     *     {@code FreeColXMLWriter}.
+     */
+    public FreeColXMLWriter(Writer writer,
+                            WriteScope scope) throws IOException {
         this(writer, scope, false);
     }
 
     /**
-     * Creates a new <code>FreeColXMLWriter</code>.
+     * Creates a new {@code FreeColXMLWriter}.
      *
-     * @param scope The <code>WriteScope</code> to use for FreeCol
-     *     object writes.
+     * @param writer A {@code Writer} to create an
+     *     {@code FreeColXMLWriter} for.
+     * @param scope The {@code WriteScope} to use for FreeCol objects.
      * @param indent If true, produce indented output if supported.
-     * @exception IOException if thrown while creating the
-     *     <code>FreeColXMLWriter</code>.
+     * @exception IOException if there is a problem while creating the
+     *     {@code FreeColXMLWriter}.
      */
-    private FreeColXMLWriter(Writer writer, WriteScope scope,
-                             boolean indent) throws IOException {
+    public FreeColXMLWriter(Writer writer, WriteScope scope,
+                            boolean indent) throws IOException {
+        this.outputWriter = writer;
         try {
-            if (indent) {
-                this.outputWriter = writer;
-                this.stringWriter = new StringWriter(1024);
-                this.xmlStreamWriter = getFactory()
-                    .createXMLStreamWriter(this.stringWriter);
-            } else {
-                this.xmlStreamWriter = getFactory()
-                    .createXMLStreamWriter(writer);
-            }
+            this.stringWriter = new StringWriter(1024);
+            this.xmlStreamWriter = getFactory()
+                .createXMLStreamWriter(this.stringWriter);
         } catch (XMLStreamException e) {
             throw new IOException(e);
         }
-        this.writeScope = scope;
+        this.transformer = (indent) ? Utils.makeTransformer(false, true)
+            : null;
+        this.writeScope = (scope == null) ? WriteScope.toSave() : scope;
     }
 
 
     /**
-     * Get the <code>XMLOutputFactory</code> to create the output stream with.
+     * Get the {@code XMLOutputFactory} to create the output stream with.
      *
-     * @return An <code>XMLOutputFactory</code>.
+     * @return An {@code XMLOutputFactory}.
      */
     private XMLOutputFactory getFactory() {
         return XMLOutputFactory.newInstance();
@@ -205,53 +230,88 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     /**
      * Set the write scope prevailing on this stream.
      *
-     * @param writeScope The new <code>WriteScope</code>.
+     * @param writeScope The new {@code WriteScope}.
      */     
     public void setWriteScope(WriteScope writeScope) {
         this.writeScope = writeScope;
     }
 
     /**
-     * Closes both the <code>XMLStreamWriter</code> and
-     * the underlying stream if any.
+     * Replace the scope.
      *
-     * Implement interface Closeable.
+     * @param newWriteScope The {@code WriteScope} to push.
+     * @return The previous {@code WriteScope}.
      */
-    @Override
-    public void close() {
-        if (xmlStreamWriter != null) {
-            try {
-                xmlStreamWriter.close();
-            } catch (XMLStreamException xse) {
-                logger.log(Level.WARNING, "Error closing stream.", xse);
-            }
-        }
+    public WriteScope replaceScope(WriteScope newWriteScope) {
+        WriteScope ret = this.writeScope;
+        this.writeScope = newWriteScope;
+        return ret;
+    }
 
-        if (this.outputWriter != null) {
-            TransformerFactory factory;
-            Transformer transformer;
-            StreamSource source;
-            StreamResult result;
-            try {
-                source = new StreamSource(new StringReader(this.stringWriter
-                                                               .toString()));
-                result = new StreamResult(this.outputWriter);
-                factory = TransformerFactory.newInstance();
-                transformer = factory.newTransformer();
-                for (int i = 0; i < indentProps.length; i += 2) {
-                    transformer.setOutputProperty(indentProps[i],
-                                                  indentProps[i+1]);
+    /**
+     * Internal flush, returning what was written.
+     *
+     * @return The internal buffer containing the flushed data.
+     * @exception XMLStreamException on stream error.
+     */
+    public StringBuffer flushBuffer() throws XMLStreamException {
+        this.xmlStreamWriter.flush();
+
+        StringBuffer sb = this.stringWriter.getBuffer();
+        if (sb.length() > 0) {
+            String str = sb.toString();
+            if (this.transformer == null) {
+                try {
+                    this.outputWriter.write(str);
+                } catch (IOException ioe) {
+                    logger.log(Level.WARNING, "Flush-write fail:" + str, ioe);
+                }
+            } else {
+                StreamSource source = new StreamSource(new StringReader(str));
+                StreamResult result = new StreamResult(this.outputWriter);
+                try {
+                    this.transformer.transform(source, result);
+                } catch (TransformerException te) {
+                    logger.log(Level.WARNING, "Transform fail:" + str, te);
                 }
-                transformer.transform(source, result);
-            } catch (TransformerException te) {
-                logger.log(Level.WARNING, "Transformer fail", te);
             }
+
             try {
                 this.outputWriter.flush();
             } catch (IOException ioe) {
                 logger.log(Level.WARNING, "Flush fail", ioe);
             }
         }
+        return sb;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void flush() throws XMLStreamException {
+        // Clear the underlying buffer after flushing it
+        flushBuffer().setLength(0);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void close() {
+        try {
+            flush();
+        } catch (XMLStreamException xse) {} // Ignore flush fail on close
+        
+        if (this.xmlStreamWriter != null) {
+            try {
+                this.xmlStreamWriter.close();
+            } catch (XMLStreamException xse) {
+                logger.log(Level.WARNING, "Error closing stream.", xse);
+            }
+            // TODO: Do not set this.xmlStreamWriter = null.  Need to build
+            // robustness.
+        }
     }
 
 
@@ -262,7 +322,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * @param value A boolean to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, boolean value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, boolean value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName, String.valueOf(value));
     }
 
@@ -273,7 +334,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * @param value A float to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, float value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, float value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName, String.valueOf(value));
     }
 
@@ -284,7 +346,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * @param value An integer to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, int value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, int value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName, String.valueOf(value));
     }
 
@@ -295,7 +358,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * @param value A long to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, long value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, long value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName, String.valueOf(value));
     }
 
@@ -303,22 +367,24 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * Write an enum attribute to the stream.
      *
      * @param attributeName The attribute name.
-     * @param value The <code>Enum</code> to write.
+     * @param value The {@code Enum} to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, Enum<?> value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, Enum<?> value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName,
-            value.toString().toLowerCase(Locale.US));
+                                       downCase(value.toString()));
     }
 
     /**
      * Write an Object attribute to the stream.
      *
      * @param attributeName The attribute name.
-     * @param value The <code>Object</code> to write.
+     * @param value The {@code Object} to write.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, Object value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, Object value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(attributeName, String.valueOf(value));
     }
 
@@ -326,12 +392,16 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * Write the identifier attribute of a non-null FreeColObject to the stream.
      *
      * @param attributeName The attribute name.
-     * @param value The <code>FreeColObject</code> to write the identifier of.
+     * @param value The {@code FreeColObject} to write the identifier of.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeAttribute(String attributeName, FreeColObject value) throws XMLStreamException {
+    public void writeAttribute(String attributeName, FreeColObject value)
+        throws XMLStreamException {
         if (value != null) {
-            xmlStreamWriter.writeAttribute(attributeName, value.getId());
+            String id = value.getId();
+            if (id != null) {
+                xmlStreamWriter.writeAttribute(attributeName, value.getId());
+            }
         }
     }
 
@@ -339,36 +409,41 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
      * Write the identifier attribute of a non-null Location to the stream.
      *
      * @param attributeName The attribute name.
-     * @param value The <code>Location</code> to write the identifier of.
+     * @param value The {@code Location} to write the identifier of.
      * @exception XMLStreamException if a write error occurs.
      */
-    public void writeLocationAttribute(String attributeName, Location value) throws XMLStreamException {
+    public void writeLocationAttribute(String attributeName, Location value)
+        throws XMLStreamException {
         writeAttribute(attributeName, (FreeColObject)value);
     }
 
     /**
      * Writes an XML-representation of a collection object to the given stream.
      *
-     * @param tag The tag for the array <code>Element</code>.
+     * @param <T> The collection type.
+     * @param tag The tag for the array.
      * @param members The members of the array.
      * @exception XMLStreamException if a problem was encountered
      *      while writing.
      */
     public <T extends FreeColObject> void writeToListElement(String tag,
         Collection<T> members) throws XMLStreamException {
+        if (members.isEmpty()) return;
+        
         writeStartElement(tag);
 
         writeAttribute(FreeColObject.ARRAY_SIZE_TAG, members.size());
 
         int i = 0;
         for (T t : members) {
-            writeAttribute("x" + i, t);
+            writeAttribute(FreeColObject.arrayKey(i), t);
             i++;
         }
 
         writeEndElement();
     }
 
+
     // Delegations to the WriteScope.
 
     public Player getClientPlayer() {
@@ -387,13 +462,9 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
         return writeScope.validFor(player);
     }
 
-    // Simple delegations to the XMLStreamWriter.  All should be
-    // present here except close which is supplied above.
 
-    @Override
-    public void flush() throws XMLStreamException {
-        xmlStreamWriter.flush();
-    }
+    // Simple delegations to the XMLStreamWriter.  All should be
+    // present here except close and flush which are supplied above.
 
     @Override
     public NamespaceContext getNamespaceContext() {
@@ -416,7 +487,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void setNamespaceContext(NamespaceContext context) throws XMLStreamException {
+    public void setNamespaceContext(NamespaceContext context)
+        throws XMLStreamException {
         xmlStreamWriter.setNamespaceContext(context);
     }
 
@@ -426,17 +498,21 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeAttribute(String localName, String value) throws XMLStreamException {
+    public void writeAttribute(String localName, String value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(localName, value);
     }
 
     @Override
-    public void writeAttribute(String namespaceURI, String localName, String value) throws XMLStreamException {
+    public void writeAttribute(String namespaceURI, String localName,
+                               String value) throws XMLStreamException {
         xmlStreamWriter.writeAttribute(namespaceURI, localName, value);
     }
 
     @Override
-    public void writeAttribute(String prefix, String namespaceURI, String localName, String value) throws XMLStreamException {
+    public void writeAttribute(String prefix, String namespaceURI,
+                               String localName, String value)
+        throws XMLStreamException {
         xmlStreamWriter.writeAttribute(prefix, namespaceURI, localName, value);
     }
 
@@ -446,7 +522,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeCharacters(char[] text, int start, int len) throws XMLStreamException {
+    public void writeCharacters(char[] text, int start, int len)
+        throws XMLStreamException {
         xmlStreamWriter.writeCharacters(text, start, len);
     }
 
@@ -461,7 +538,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeDefaultNamespace(String namespaceURI) throws XMLStreamException {
+    public void writeDefaultNamespace(String namespaceURI)
+        throws XMLStreamException {
         xmlStreamWriter.writeDefaultNamespace(namespaceURI);
     }
 
@@ -476,12 +554,15 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeEmptyElement(String namespaceURI, String localName) throws XMLStreamException {
+    public void writeEmptyElement(String namespaceURI, String localName)
+        throws XMLStreamException {
         xmlStreamWriter.writeEmptyElement(namespaceURI, localName);
     }
 
     @Override
-    public void writeEmptyElement(String prefix, String localName, String namespaceURI) throws XMLStreamException {
+    public void writeEmptyElement(String prefix, String localName,
+                                  String namespaceURI)
+        throws XMLStreamException {
         xmlStreamWriter.writeEmptyElement(prefix, localName, namespaceURI);
     }
 
@@ -501,17 +582,20 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeNamespace(String prefix, String namespaceURI) throws XMLStreamException {
+    public void writeNamespace(String prefix, String namespaceURI)
+        throws XMLStreamException {
         xmlStreamWriter.writeNamespace(prefix, namespaceURI);
     }
 
     @Override
-    public void writeProcessingInstruction(String target) throws XMLStreamException {
+    public void writeProcessingInstruction(String target)
+        throws XMLStreamException {
         xmlStreamWriter.writeProcessingInstruction(target);
     }
 
     @Override
-    public void writeProcessingInstruction(String target, String data) throws XMLStreamException {
+    public void writeProcessingInstruction(String target, String data)
+        throws XMLStreamException {
         xmlStreamWriter.writeProcessingInstruction(target, data);
     }
 
@@ -526,7 +610,8 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeStartDocument(String encoding, String version) throws XMLStreamException {
+    public void writeStartDocument(String encoding, String version)
+        throws XMLStreamException {
         xmlStreamWriter.writeStartDocument(encoding, version);
     }
 
@@ -536,12 +621,15 @@ public class FreeColXMLWriter implements Closeable, XMLStreamWriter {
     }
 
     @Override
-    public void writeStartElement(String namespaceURI, String localName) throws XMLStreamException {
+    public void writeStartElement(String namespaceURI, String localName)
+        throws XMLStreamException {
         xmlStreamWriter.writeStartElement(namespaceURI, localName);
     }
 
     @Override
-    public void writeStartElement(String prefix, String localName, String namespaceURI) throws XMLStreamException {
+    public void writeStartElement(String prefix, String localName,
+                                  String namespaceURI)
+        throws XMLStreamException {
         xmlStreamWriter.writeStartElement(prefix, localName, namespaceURI);
     }
 }
