@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,23 +19,31 @@
 
 package net.sf.freecol.common.model;
 
+import static net.sf.freecol.common.util.CollectionUtils.concat;
+import static net.sf.freecol.common.util.CollectionUtils.count;
+import static net.sf.freecol.common.util.CollectionUtils.sort;
+
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
-import java.util.Set;
 import java.util.logging.Logger;
+import java.util.stream.Stream;
 
-import javax.xml.stream.XMLStreamConstants;
+import javax.swing.JLabel;
 import javax.xml.stream.XMLStreamException;
 
+import net.sf.freecol.client.gui.ImageLibrary;
+import net.sf.freecol.client.gui.panel.Utility;
+import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
 import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.option.GameOptions;
 
 
 /**
- * Represents Europe in the game.  Each <code>Player</code> has it's
- * own <code>Europe</code>.
+ * Represents Europe in the game.  Each {@code Player} has it's
+ * own {@code Europe}.
  *
  * In Europe, you can recruit, train and purchase new units.  You can
  * also equip units, as well as sell and buy goods.
@@ -45,6 +53,11 @@ public class Europe extends UnitLocation
 
     private static final Logger logger = Logger.getLogger(Europe.class.getName());
 
+    /** Class index for Europe. */
+    private static final int EUROPE_CLASS_INDEX = 30;
+
+    public static final String TAG = "europe";
+
     /** The initial recruit price. */
     private static final int RECRUIT_PRICE_INITIAL = 200;
 
@@ -130,17 +143,18 @@ public class Europe extends UnitLocation
         }
     }
 
+
     /**
      * This list represents the types of the units that can be
      * recruited in Europe.
      */
-    protected final List<UnitType> recruitables = new ArrayList<>();
+    protected final List<AbstractUnit> recruitables = new ArrayList<>();
 
     /** Prices for trainable or purchasable units. */
     protected final java.util.Map<UnitType, Integer> unitPrices = new HashMap<>();
 
     /** Current price to recruit a unit. */
-    protected int recruitPrice;
+    protected int baseRecruitPrice;
 
     /** The lower bound on recruitment price. */
     protected int recruitLowerCap;
@@ -155,24 +169,23 @@ public class Europe extends UnitLocation
     /**
      * Constructor for ServerEurope.
      *
-     * @param game The enclosing <code>Game</code>.
-     * @param owner The owning <code>Player</code>.
+     * @param game The enclosing {@code Game}.
+     * @param owner The owning {@code Player}.
      */
     protected Europe(Game game, Player owner) {
         super(game);
 
         this.owner = owner;
-        this.recruitPrice = RECRUIT_PRICE_INITIAL;
+        this.baseRecruitPrice = RECRUIT_PRICE_INITIAL;
         this.recruitLowerCap = LOWER_CAP_INITIAL;
     }
 
     /**
-     * Creates a new <code>Europe</code> with the given identifier.
-     * The object should later be initialized by calling either
-     * {@link #readFromXML(FreeColXMLReader)} or
-     * {@link #readFromXMLElement(Element)}.
+     * Creates a new {@code Europe} with the given identifier.
+     (
+     * The object should be initialized later.
      *
-     * @param game The enclosing <code>Game</code>.
+     * @param game The enclosing {@code Game}.
      * @param id The object identifier.
      */
     public Europe(Game game, String id) {
@@ -183,30 +196,96 @@ public class Europe extends UnitLocation
     /**
      * Get a list of the current recruitables.
      *
-     * @return A list of recruitable <code>UnitType</code>s.
+     * @return A list of recruitable {@code AbstractUnit}s.
      */
-    public List<UnitType> getRecruitables() {
-        return new ArrayList<>(recruitables);
+    protected List<AbstractUnit> getRecruitables() {
+        return this.recruitables;
     }
 
     /**
-     * Add a recruitable unit type.
+     * Get an expanded list of recruitable abstract units, with single counts
+     * and limited to valid indices.
      *
-     * @param unitType The recruitable <code>UnitType</code> to add.
+     * @param full If true, return the full list, otherwise limit to the
+     *     standard number of migrants.
+     * @return A list of {@code AbstractUnit}s.
+     */
+    public List<AbstractUnit> getExpandedRecruitables(boolean full) {
+        final Specification spec = getSpecification();
+        List<AbstractUnit> ret = new ArrayList<>();
+        int c = 0;
+        for (AbstractUnit au : this.recruitables) {
+            for (int n = au.getNumber(); n > 0; n--) {
+                if (!full && c >= MigrationType.MIGRANT_COUNT) break;
+                ret.add(new AbstractUnit(au.getType(spec), au.getRoleId(), 1));
+                c++;
+            }
+        }
+        return ret;
+    }
+
+    /**
+     * Set the recruitables list.
+     *
+     * @param recruitables The new list of recruitables {@code AbstractUnit}s.
+     */
+    private void setRecruitables(List<AbstractUnit> recruitables) {
+        this.recruitables.clear();
+        this.recruitables.addAll(recruitables);
+    }
+
+    /**
+     * Add a recruitable abstract unit.
+     *
+     * @param au The recruitable {@code AbstractUnit} to add.
+     * @param force If true, add beyond the standard limit.
      * @return True if the recruitable was added.
      */
-    protected boolean addRecruitable(UnitType unitType) {
-        if (recruitables.size() < MigrationType.MIGRANT_COUNT) {
-            recruitables.add(unitType);
+    protected boolean addRecruitable(AbstractUnit au, boolean force) {
+        if (force || this.recruitables.size() < MigrationType.MIGRANT_COUNT) {
+            this.recruitables.add(au);
             return true;
         }
         return false;
     }
 
+    /**
+     * Add a recruitable unit type.
+     *
+     * @param unitType The recruitable {@code UnitType} to add.
+     * @param force If true, add beyond the standard limit.
+     * @return True if the recruitable was added.
+     */
+    protected boolean addRecruitable(UnitType unitType, boolean force) {
+        Role role = unitType.getDefaultRole();
+        if (role == null) role = getSpecification().getDefaultRole();
+        return addRecruitable(new AbstractUnit(unitType, role.getId(), 1),
+                              force);
+    }
+
+    /**
+     * Get the unit price map.
+     *
+     * @return The map of {@code UnitType} to its price.
+     */
+    protected java.util.Map<UnitType, Integer> getUnitPrices() {
+        return this.unitPrices;
+    }
+
+    /**
+     * Set the unit price map.
+     *
+     * @param unitPrices The new map of {@code UnitType}s to price.
+     */
+    protected void setUnitPrices(java.util.Map<UnitType, Integer> unitPrices) {
+        this.unitPrices.clear();
+        this.unitPrices.putAll(unitPrices);
+    }
+    
     /**
      * Gets the price of a unit in Europe.
      *
-     * @param unitType The <code>UnitType</code> to price.
+     * @param unitType The {@code UnitType} to price.
      * @return The price of this unit when trained/purchased in Europe,
      *     or UNDEFINED on failure.
      */
@@ -215,20 +294,38 @@ public class Europe extends UnitLocation
         return (price != null) ? price : unitType.getPrice();
     }
 
+    /**
+     * Get the base recruit price.
+     *
+     * @return The base recruit price.
+     */
+    protected int getBaseRecruitPrice() {
+        return this.baseRecruitPrice;
+    }
+    
     /**
      * Gets the current price for a recruit.
      *
-     * @return The current price of the recruit in this <code>Europe</code>.
+     * @return The current price of the recruit in this {@code Europe}.
      */
-    public int getRecruitPrice() {
+    public int getCurrentRecruitPrice() {
         if (!owner.isColonial()) return -1;
         int required = owner.getImmigrationRequired();
         int immigration = owner.getImmigration();
         int difference = Math.max(required - immigration, 0);
-        return Math.max((recruitPrice * difference) / required,
+        return Math.max((getBaseRecruitPrice() * difference) / required,
                         recruitLowerCap);
     }
 
+    /**
+     * Get the recruit lower cap price.
+     *
+     * @return The lower cap on the recruit price.
+     */
+    protected int getRecruitLowerCap() {
+        return this.recruitLowerCap;
+    }
+
     /**
      * Get any immigration produced in Europe.
      *
@@ -242,10 +339,7 @@ public class Europe extends UnitLocation
      */
     public int getImmigration(int production) {
         final Specification spec = getSpecification();
-        int n = 0;
-        for (Unit u : getUnitList()) {
-            if (u.isPerson()) n++;
-        }
+        int n = count(getUnits(), Unit::isPerson);
         n *= spec.getInteger(GameOptions.EUROPEAN_UNIT_IMMIGRATION_PENALTY);
         n += spec.getInteger(GameOptions.PLAYER_IMMIGRATION_BONUS);
         // Do not allow total production to be negative.
@@ -254,26 +348,6 @@ public class Europe extends UnitLocation
     }
 
 
-    // Override FreeColObject
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public FeatureContainer getFeatureContainer() {
-        return featureContainer;
-    }
-
-    // Override FreeColGameObject
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public FreeColGameObject getLinkTarget(Player player) {
-        return (getOwner() == player) ? this : null;
-    }
-
     // Interface Location (from UnitLocation)
     // Inheriting:
     //   FreeColObject.getId()
@@ -341,6 +415,14 @@ public class Europe extends UnitLocation
         return "Europe";
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String getLocationImageKey() {
+        return ImageLibrary.getNationKey(this.getOwner().getNation());
+    }
+
 
     // Interface UnitLocation
     // Inheriting
@@ -354,19 +436,21 @@ public class Europe extends UnitLocation
      * {@inheritDoc}
      */
     @Override
-    public int priceGoods(List<AbstractGoods> goods) {
-        Player player = getOwner();
-        Market market = player.getMarket();
+    public int priceGoods(List<AbstractGoods> goods) throws FreeColException {
+        final Player player = getOwner();
+        final Market market = player.getMarket();
         int price = 0;
         for (AbstractGoods ag : goods) {
-            if (ag.getAmount() <= 0) continue;
             GoodsType goodsType = ag.getType();
             // Refuse to trade in boycotted goods
             if (!player.canTrade(goodsType)) {
-                price = -1;
-                break;
+                throw new FreeColException("Can not trade " + goodsType);
+            }
+            if (ag.getAmount() > 0) {
+                price += market.getBidPrice(goodsType, ag.getAmount());
+            } else {
+                price += market.getSalePrice(goodsType, ag.getAmount());
             }
-            price += market.getBidPrice(goodsType, ag.getAmount());
         }
         return price;
     }
@@ -376,7 +460,7 @@ public class Europe extends UnitLocation
      */
     @Override
     public boolean equipForRole(Unit unit, Role role, int roleCount) {
-        throw new RuntimeException("Only valid in the server.");
+        throw new RuntimeException("Only valid in the server: " + this);
     }
 
 
@@ -416,7 +500,7 @@ public class Europe extends UnitLocation
      * {@inheritDoc}
      */
     @Override
-    public int getGoodsCount(GoodsType goodsType) {
+    public int getAvailableGoodsCount(GoodsType goodsType) {
         return GoodsContainer.HUGE_CARGO_SIZE;
     }
 
@@ -425,7 +509,8 @@ public class Europe extends UnitLocation
      */
     @Override
     public int getExportAmount(GoodsType goodsType, int turns) {
-        return (getOwner().canTrade(goodsType)) ? GoodsContainer.HUGE_CARGO_SIZE
+        return (getOwner().canTrade(goodsType))
+            ? GoodsContainer.HUGE_CARGO_SIZE
             : 0;
     }
 
@@ -434,10 +519,35 @@ public class Europe extends UnitLocation
      */
     @Override
     public int getImportAmount(GoodsType goodsType, int turns) {
-        return (getOwner().canTrade(goodsType)) ? GoodsContainer.HUGE_CARGO_SIZE
+        return (getOwner().canTrade(goodsType))
+            ? GoodsContainer.HUGE_CARGO_SIZE
             : 0;
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String getLocationName(TradeLocation tradeLocation) {
+        return ((Europe) tradeLocation).getNameKey();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public JLabel getNameAsJlabel() {
+        return Utility.localizedLabel(getLocationName(this));
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean canBeInput() {
+        return true;
+    }
+
 
     // Override FreeColGameObject
 
@@ -455,6 +565,14 @@ public class Europe extends UnitLocation
         super.disposeResources();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public FreeColGameObject getLinkTarget(Player player) {
+        return (getOwner() == player) ? this : null;
+    }
+
 
     // Override FreeColObject
 
@@ -462,14 +580,46 @@ public class Europe extends UnitLocation
      * {@inheritDoc}
      */
     @Override
-    public Set<Ability> getAbilities(String id, FreeColGameObjectType fcgot,
-                                     Turn turn) {
-        Set<Ability> result = super.getAbilities(id, fcgot, turn);
-        // Always able to dress a missionary.
-        if (id == null || Ability.DRESS_MISSIONARY.equals(id)) {
-            result.add(ABILITY_DRESS_MISSIONARY);
-        }
-        return result;
+    public FeatureContainer getFeatureContainer() {
+        return this.featureContainer;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public Stream<Ability> getAbilities(String id, FreeColSpecObjectType fcgot,
+                                        Turn turn) {
+        return concat(super.getAbilities(id, fcgot, turn),
+            // Always able to dress a missionary.
+            ((id == null || Ability.DRESS_MISSIONARY.equals(id))
+                ? Stream.of(ABILITY_DRESS_MISSIONARY)
+                : Stream.<Ability>empty()));
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public int getClassIndex () {
+        return EUROPE_CLASS_INDEX;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        Europe o = copyInCast(other, Europe.class);
+        if (o == null || !super.copyIn(o)) return false;
+        final Game game = getGame();
+        this.setRecruitables(o.getRecruitables());
+        this.setUnitPrices(o.getUnitPrices());
+        this.baseRecruitPrice = o.getBaseRecruitPrice();
+        this.recruitLowerCap = o.getRecruitLowerCap();
+        this.owner = game.updateRef(o.getOwner());
+        this.featureContainer.copy(o.getFeatureContainer());
+        return true;
     }
 
 
@@ -477,15 +627,17 @@ public class Europe extends UnitLocation
 
     private static final String OWNER_TAG = "owner";
     private static final String PRICE_TAG = "price";
-    private static final String RECRUIT_TAG = "recruit";
-    private static final String RECRUIT_ID_TAG = "id";
     private static final String RECRUIT_LOWER_CAP_TAG = "recruitLowerCap";
+    private static final String RECRUIT_NUMBER_TAG = "number";
     private static final String RECRUIT_PRICE_TAG = "recruitPrice";
+    private static final String RECRUIT_ROLE_TAG = "role";
+    private static final String RECRUIT_TAG = "recruit";
+    private static final String RECRUIT_TYPE_TAG = "type";
     private static final String UNIT_PRICE_TAG = "unitPrice";
     private static final String UNIT_TYPE_TAG = "unitType";
-    // @compat 0.10.0
-    private static final String UNITS_TAG = "units";
-    // end @compat 0.10.0
+    // @compat 0.11.x
+    private static final String RECRUIT_ID_TAG = "id";
+    // end @compat 0.11.x
 
 
     /**
@@ -496,7 +648,7 @@ public class Europe extends UnitLocation
         super.writeAttributes(xw);
 
         if (xw.validFor(getOwner())) {
-            xw.writeAttribute(RECRUIT_PRICE_TAG, recruitPrice);
+            xw.writeAttribute(RECRUIT_PRICE_TAG, baseRecruitPrice);
 
             xw.writeAttribute(RECRUIT_LOWER_CAP_TAG, recruitLowerCap);
 
@@ -521,7 +673,7 @@ public class Europe extends UnitLocation
                 modifier.toXML(xw);
             }
 
-            for (UnitType unitType : getSortedCopy(unitPrices.keySet())) {
+            for (UnitType unitType : sort(unitPrices.keySet())) {
                 xw.writeStartElement(UNIT_PRICE_TAG);
 
                 xw.writeAttribute(UNIT_TYPE_TAG, unitType);
@@ -531,17 +683,16 @@ public class Europe extends UnitLocation
                 xw.writeEndElement();
             }
 
-            for (UnitType unitType : recruitables) {
+            for (AbstractUnit au : recruitables) {
                 xw.writeStartElement(RECRUIT_TAG);
-                xw.writeAttribute(RECRUIT_ID_TAG, unitType.getId());
+                xw.writeAttribute(RECRUIT_ID_TAG, au.getId());
+                xw.writeAttribute(RECRUIT_ROLE_TAG, au.getRoleId());
+                xw.writeAttribute(RECRUIT_NUMBER_TAG, au.getNumber());
                 xw.writeEndElement();
             }
         }
     }
 
-    // @compat 0.10.7
-    private boolean clearRecruitables = true;
-
     /**
      * {@inheritDoc}
      */
@@ -549,24 +700,11 @@ public class Europe extends UnitLocation
     public void readAttributes(FreeColXMLReader xr) throws XMLStreamException {
         super.readAttributes(xr);
 
-        final Specification spec = getSpecification();
-
-        // @compat 0.10.7
-        for (int index = 0; index < 3; index++) {
-            UnitType unitType = xr.getType(spec, RECRUIT_TAG + index,
-                                           UnitType.class, (UnitType)null);
-            if (unitType != null) {
-                addRecruitable(unitType);
-                clearRecruitables = false;
-            }
-        }
-        // end @compat
-
         owner = xr.findFreeColGameObject(getGame(), OWNER_TAG,
                                          Player.class, (Player)null, true);
 
-        recruitPrice = xr.getAttribute(RECRUIT_PRICE_TAG,
-                                       RECRUIT_PRICE_INITIAL);
+        baseRecruitPrice = xr.getAttribute(RECRUIT_PRICE_TAG,
+                                           RECRUIT_PRICE_INITIAL);
 
         recruitLowerCap = xr.getAttribute(RECRUIT_LOWER_CAP_TAG,
                                           LOWER_CAP_INITIAL);
@@ -580,23 +718,9 @@ public class Europe extends UnitLocation
         // Clear containers.
         unitPrices.clear();
         featureContainer.clear();
-
-        // @compat 0.10.7
-        if (clearRecruitables) {
-            // in future, always clear
-            recruitables.clear();
-        }
-        // end @compat
+        recruitables.clear();
 
         super.readChildren(xr);
-
-        // @compat 0.10.1
-        // Sometimes units in a Europe element have a missing
-        // location.  It should always be this Europe instance.
-        for (Unit u : getUnitList()) {
-            if (u.getLocation() == null) u.setLocationNoUpdate(this);
-        }
-        // end @compat
     }
 
     /**
@@ -607,16 +731,29 @@ public class Europe extends UnitLocation
         final Specification spec = getSpecification();
         final String tag = xr.getLocalName();
 
-        if (Ability.getXMLElementTagName().equals(tag)) {
+        if (Ability.TAG.equals(tag)) {
             addAbility(new Ability(xr, spec));
 
-        } else if (Modifier.getXMLElementTagName().equals(tag)) {
+        } else if (Modifier.TAG.equals(tag)) {
             addModifier(new Modifier(xr, spec));
 
         } else if (RECRUIT_TAG.equals(tag)) {
-            UnitType unitType = xr.getType(spec, RECRUIT_ID_TAG,
+            UnitType unitType = xr.getType(spec, RECRUIT_TYPE_TAG,
                                            UnitType.class, (UnitType)null);
-            if (unitType != null) addRecruitable(unitType);
+            // @compat 0.11.x
+            if (unitType == null) {
+                unitType = xr.getType(spec, RECRUIT_ID_TAG,
+                                      UnitType.class, (UnitType)null);
+            }
+            // end @compat 0.11.x
+            if (unitType != null) {
+                Role role = xr.getType(spec, RECRUIT_ROLE_TAG, Role.class,
+                                       unitType.getDefaultRole());
+                int n = xr.getAttribute(RECRUIT_NUMBER_TAG, 1);
+                
+                addRecruitable(new AbstractUnit(unitType, role.getId(), n),
+                               true);
+            }
             xr.closeTag(RECRUIT_TAG);
 
         } else if (UNIT_PRICE_TAG.equals(tag)) {
@@ -628,13 +765,6 @@ public class Europe extends UnitLocation
             unitPrices.put(unitType, price);
             xr.closeTag(UNIT_PRICE_TAG);
 
-        // @compat 0.10.0
-        } else if (UNITS_TAG.equals(tag)) {
-            while (xr.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                super.readChild(xr);
-            }
-        // end @compat 0.10.0
-
         } else {
             super.readChild(xr);
         }
@@ -643,23 +773,16 @@ public class Europe extends UnitLocation
     /**
      * {@inheritDoc}
      */
-    @Override
-    public String toString() {
-        return "Europe";
-    }
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "europe".
-     */
-    public static String getXMLElementTagName() {
-        return "europe";
+    public String toString() {
+        return "Europe";
     }
 }
