@@ -18,6 +18,13 @@
  */
 package net.sf.freecol.common.model;
 
+import static net.sf.freecol.common.util.CollectionUtils.first;
+import static net.sf.freecol.common.util.CollectionUtils.forEachMapEntry;
+import static net.sf.freecol.common.util.CollectionUtils.iterable;
+import static net.sf.freecol.common.util.CollectionUtils.removeInPlace;
+import static net.sf.freecol.common.util.CollectionUtils.sort;
+import static net.sf.freecol.common.util.CollectionUtils.transform;
+
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -26,8 +33,6 @@ import java.util.Set;
 import java.util.logging.Logger;
 import java.util.stream.Stream;
 
-import static net.sf.freecol.common.util.CollectionUtils.*;
-
 
 /**
  * A container to hold abilities and modifiers for some FreeColObject-subclass.
@@ -105,7 +110,12 @@ public final class FeatureContainer {
         }
         return ret;
     }
-    protected Collection<Modifier> getModifierValues() {
+    
+    /**
+     * Gets all modifiers with any filtering applied.
+     * @return All the modifiers stored in this {@code FeatureContainer}.
+     */
+    public Collection<Modifier> getModifierValues() {
         Set<Modifier> ret = new HashSet<>();
         synchronized (modifiersLock) {
             if (modifiers != null) {
