@@ -459,17 +459,21 @@ public class SettlementType extends FreeColSpecObjectType {
             convertThreshold = xr.getAttribute(CONVERT_THRESHOLD_TAG, convertThreshold);
     }
 
+    /**
+    * {@inheritDoc}
+    */
+    @Override
+    protected void clearContainers(FreeColXMLReader xr) throws XMLStreamException {
+        super.clearContainers(xr);
+        this.plunderTypes = null;
+        this.gifts = null;
+    }
+    
     /**
      * {@inheritDoc}
      */
     @Override
     protected void readChildren(FreeColXMLReader xr) throws XMLStreamException {
-        // Clear containers.
-        if (xr.shouldClearContainers()) {
-            this.plunderTypes = null;
-            this.gifts = null;
-        }
-
         super.readChildren(xr);
     }
 
