@@ -19,6 +19,25 @@
 
 package net.sf.freecol.common.model;
 
+import static net.sf.freecol.common.model.Constants.INFINITY;
+import static net.sf.freecol.common.util.CollectionUtils.all;
+import static net.sf.freecol.common.util.CollectionUtils.any;
+import static net.sf.freecol.common.util.CollectionUtils.cacheInt;
+import static net.sf.freecol.common.util.CollectionUtils.cachingDoubleComparator;
+import static net.sf.freecol.common.util.CollectionUtils.concat;
+import static net.sf.freecol.common.util.CollectionUtils.count;
+import static net.sf.freecol.common.util.CollectionUtils.find;
+import static net.sf.freecol.common.util.CollectionUtils.flatten;
+import static net.sf.freecol.common.util.CollectionUtils.forEachMapEntry;
+import static net.sf.freecol.common.util.CollectionUtils.isNotNull;
+import static net.sf.freecol.common.util.CollectionUtils.matchKey;
+import static net.sf.freecol.common.util.CollectionUtils.max;
+import static net.sf.freecol.common.util.CollectionUtils.maximize;
+import static net.sf.freecol.common.util.CollectionUtils.none;
+import static net.sf.freecol.common.util.CollectionUtils.sum;
+import static net.sf.freecol.common.util.CollectionUtils.transform;
+import static net.sf.freecol.common.util.RandomUtils.randomShuffle;
+
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -39,11 +58,11 @@ import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
-import static net.sf.freecol.common.model.Constants.*;
-import static net.sf.freecol.common.util.CollectionUtils.*;
+import net.sf.freecol.common.model.Constants.IntegrityType;
+import net.sf.freecol.common.model.production.TileProductionCalculator;
+import net.sf.freecol.common.model.production.WorkerAssignment;
 import net.sf.freecol.common.util.LogBuilder;
 import net.sf.freecol.common.util.RandomChoice;
-import static net.sf.freecol.common.util.RandomUtils.*;
 
 
 /**
@@ -1264,10 +1283,10 @@ public final class Tile extends UnitLocation implements Named, Ownable {
      * @param mountains The mountain tile type.
      * @return True if this is a good potential elevated tile.
      */
-    public boolean isGoodMountainTile(TileType mountains) {
+    public boolean isGoodMountainTile() {
         return isGoodHillTile()
             // Not too close to an existing mountain range
-            && none(getSurroundingTiles(1, 3), t -> t.getType() == mountains);
+            && none(getSurroundingTiles(1, 3), t -> t.getType().isMountains());
     }
 
     /**
@@ -1281,7 +1300,7 @@ public final class Tile extends UnitLocation implements Named, Ownable {
     public boolean isGoodRiverTile(TileImprovementType riverType) {
         return riverType.isTileTypeAllowed(getType())
             // check the river source/spring is not too close to the ocean
-            && all(getSurroundingTiles(1, 2), Tile::isLand);
+            && all(getSurroundingTiles(1, 1), Tile::isLand);
     }
 
     /**
@@ -1712,12 +1731,23 @@ public final class Tile extends UnitLocation implements Named, Ownable {
      */
     public int getPotentialProduction(GoodsType goodsType,
                                       UnitType unitType) {
-        if (!canProduce(goodsType, unitType)) return 0;
 
-        int amount = getBaseProduction(null, goodsType, unitType);
-        amount = (int)applyModifiers(amount, getGame().getTurn(),
-                                     getProductionModifiers(goodsType, unitType));
-        return (amount < 0) ? 0 : amount;
+        final TileProductionCalculator tpc = new TileProductionCalculator(null, 0);
+        final ProductionType productionType = ProductionType.getBestProductionType(goodsType,
+                getType().getAvailableProductionTypes(unitType == null));
+        
+        final boolean colonyCenterTile = (unitType == null);
+        
+        final ProductionInfo pi = tpc.getBasicProductionInfo(this,
+                getGame().getTurn(),
+                new WorkerAssignment(unitType, productionType),
+                colonyCenterTile);
+        
+        return pi.getProduction().stream()
+                .filter(a -> a.getType().equals(goodsType))
+                .map(AbstractGoods::getAmount)
+                .findFirst()
+                .orElse(0);
     }
 
     /**
@@ -1758,10 +1788,12 @@ public final class Tile extends UnitLocation implements Named, Ownable {
         // Try applying all possible non-natural improvements.
         final List<TileImprovementType> improvements
             = getSpecification().getTileImprovementTypeList();
-        for (TileImprovementType ti : transform(improvements, ti ->
-                (!ti.isNatural() && ti.isTileTypeAllowed(tileType)
-                    && ti.getBonus(goodsType) > 0))) {
-            potential = ti.getProductionModifier(goodsType).applyTo(potential);
+        if (canProduce(goodsType, unitType)) {
+            for (TileImprovementType ti : transform(improvements, ti ->
+                    (!ti.isNatural() && ti.isTileTypeAllowed(tileType)
+                        && ti.getBonus(goodsType) > 0))) {
+                potential = ti.getProductionModifier(goodsType).applyTo(potential);
+            }
         }
         return (int)potential;
     }
@@ -1796,12 +1828,21 @@ public final class Tile extends UnitLocation implements Named, Ownable {
     }
 
     /**
-     * Sort possible goods types according to potential.
+     * Sort possible goods types according to potential of autoproduction.
+     *
+     * @return A list of goods, highest potential production first.
+     */
+    public List<AbstractGoods> getSortedAutoPotential() {
+        return getSortedPotential(null, null, false);
+    }
+    
+    /**
+     * Sort possible goods types according to potential with an expert working the tile.
      *
      * @return A list of goods, highest potential production first.
      */
     public List<AbstractGoods> getSortedPotential() {
-        return getSortedPotential(null, null);
+        return getSortedPotential(null, null, true);
     }
 
     /**
@@ -1811,7 +1852,7 @@ public final class Tile extends UnitLocation implements Named, Ownable {
      * @return A list of goods, highest potential production first.
      */
     public List<AbstractGoods> getSortedPotential(Unit unit) {
-        return getSortedPotential(unit.getType(), unit.getOwner());
+        return getSortedPotential(unit.getType(), unit.getOwner(), false);
     }
 
     /**
@@ -1819,15 +1860,27 @@ public final class Tile extends UnitLocation implements Named, Ownable {
      *
      * @param unitType The {@code UnitType} to do the work.
      * @param owner the {@code Player} owning the unit.
+     * @param useExperts Use the best expert for the given goods if
+     *      {@code unitType == null}.
      * @return A list of goods, highest potential production first.
      */
-    public List<AbstractGoods> getSortedPotential(UnitType unitType,
-                                                  Player owner) {
+    private List<AbstractGoods> getSortedPotential(UnitType unitType,
+                                                  Player owner,
+                                                  boolean useExperts) {
         // Defend against calls while partially read.
         if (getType() == null) return Collections.<AbstractGoods>emptyList();
         
-        final ToIntFunction<GoodsType> productionMapper = cacheInt(gt ->
-            getPotentialProduction(gt, unitType));
+        final ToIntFunction<GoodsType> productionMapper = cacheInt(gt -> {
+            final UnitType ut;
+            if (unitType != null) {
+                ut = unitType;
+            } else if (useExperts) {
+                ut = getSpecification().getExpertForProducing(gt);
+            } else {
+                ut = null;
+            }
+            return getPotentialProduction(gt, ut);
+        });
         final Predicate<GoodsType> productionPred = gt ->
             productionMapper.applyAsInt(gt) > 0;
         final Function<GoodsType, AbstractGoods> goodsMapper = gt ->
@@ -1850,11 +1903,24 @@ public final class Tile extends UnitLocation implements Named, Ownable {
      */
     public AbstractGoods getBestFoodProduction() {
         final Comparator<AbstractGoods> goodsComp
-            = Comparator.comparingInt(ag ->
-                getPotentialProduction(ag.getType(), null));
-        return maximize(flatten(getType().getAvailableProductionTypes(true),
-                                ProductionType::getOutputs),
-                        AbstractGoods::isFoodType, goodsComp);
+        = Comparator.comparingInt(ag ->
+            getPotentialProduction(ag.getType(), null));
+    return maximize(flatten(getType().getAvailableProductionTypes(true),
+                            ProductionType::getOutputs),
+                    AbstractGoods::isFoodType, goodsComp);
+    }
+    
+    /**
+     * Get the best food type to produce here with an appropriate expert.,
+     *
+     * @return The {@code AbstractGoods} to produce.
+     */
+    public AbstractGoods getMaximumPotentialFoodProductionWithExpert() {
+        return getSpecification().getFoodGoodsTypeList().stream()
+                .map(goodsType -> new AbstractGoods(goodsType,
+                        getMaximumPotential(goodsType, getSpecification().getExpertForProducing(goodsType))))
+                .max(Comparator.comparingInt(AbstractGoods::getAmount))
+                .orElse(null);
     }
 
 
