@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,27 +20,33 @@
 package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Collections;
 import java.util.List;
 import java.util.logging.Logger;
-import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import javax.xml.stream.XMLStreamException;
 
+import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
 import static net.sf.freecol.common.util.CollectionUtils.*;
 import static net.sf.freecol.common.util.StringUtils.*;
 
-import org.w3c.dom.Element;
-
 
 /**
- * The <code>UnitLocation</code> is a place where a <code>Unit</code>
+ * The {@code UnitLocation} is a place where a {@code Unit}
  * can be put.  The UnitLocation can not store any other Locatables,
  * such as {@link Goods}, or {@link TileItem}s.
  *
+ * Be *very* careful with use of the units list for low level routines
+ * as distinct from use of getUnits()/getUnitList()/getUnitCount()
+ * etc.  Beware that Colony is a UnitLocation, but it does *not* store
+ * its units in the unit list, they live in its WorkLocations.
+ * Therefore Colony overrides several such routines to do the right thing,
+ * which can lead to confusion especially when debugging.
+ *
  * @see Locatable
  */
 public abstract class UnitLocation extends FreeColGameObject implements Location {
@@ -107,7 +113,12 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
         /**
          * Either unclaimed or claimed but could be acquired.
          */
-        CLAIM_REQUIRED;
+        CLAIM_REQUIRED,
+        
+        /**
+         * The worker is damaged (not at maximum hitpoints).
+         */
+        WORKER_DAMAGED;
 
         /**
          * Get a message key describing this reason.
@@ -133,46 +144,59 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
 
 
     /**
-     * Creates a new <code>UnitLocation</code> instance.
+     * Creates a new {@code UnitLocation} instance.
      *
-     * @param game The enclosing <code>Game</code>.
+     * @param game The enclosing {@code Game}.
      */
     public UnitLocation(Game game) {
         super(game);
     }
 
     /**
-     * Creates a new <code>UnitLocation</code> instance.
+     * Creates a new {@code UnitLocation} instance.
      *
-     * @param game The enclosing <code>Game</code>.
+     * @param game The enclosing {@code Game}.
      * @param id The object identifier.
      */
     public UnitLocation(Game game, String id) {
         super(game, id);
     }
 
+
     /**
-     * Creates a new <code>UnitLocation</code> instance.
-     * Only Unit needs this.
-     *
-     * FIXME: make it go away, its a noop.
-     *
-     * @param game The enclosing <code>Game</code>.
-     * @param e The <code>Element</code> to read from.
+     * {@inheritDoc}
      */
-    public UnitLocation(Game game, Element e) {
-        super(game, null);
+    @Override
+    public void intern() {
+        super.intern();
+        for (Unit u : this.units) u.intern();
+    }
+    
+    /**
+     * Internal addition of a unit to this location.
+     *
+     * @param u The {@code Unit} to add.
+     * @return True if the unit is added.
+     */
+    private boolean addUnit(Unit u) {
+        if (u == null) return false;
+        boolean ret = false;
+        u.setLocationNoUpdate(this);
+        synchronized (this.units) {
+            ret = this.units.add(u);
+        }
+        return ret;
     }
 
 
     // Some useful utilities, marked final as they will work as long
-    // as working implementations of getUnitList(), getUnitCount(),
+    // as working implementations of getUnits/List(), getUnitCount(),
     // getUnitCapacity() and getSettlement() are provided.
 
     /**
      * Is this unit location empty?
      *
-     * @return a <code>boolean</code> value
+     * @return a {@code boolean} value
      */
     public final boolean isEmpty() {
         return getUnitCount() == 0;
@@ -190,18 +214,16 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     /**
      * Gets the first Unit at this unit location.
      *
-     * @return The first <code>Unit</code>.
+     * @return The first {@code Unit}.
      */
     public final Unit getFirstUnit() {
-        if (isEmpty()) return null;
-        List<Unit> units = getUnitList();
-        return units.get(0);
+        return first(getUnits());
     }
 
     /**
      * Gets the last Unit at this unit location.
      *
-     * @return The last <code>Unit</code>.
+     * @return The last {@code Unit}.
      */
     public final Unit getLastUnit() {
         if (isEmpty()) return null;
@@ -213,12 +235,11 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * Gets the total amount of Units at this Location, including
      * units on a carrier.
      *
-     * @return The total number of <code>Unit</code>s at this
-     *     <code>Location</code>.
+     * @return The total number of {@code Unit}s at this
+     *     {@code Location}.
      */
     public int getTotalUnitCount() {
-        return getUnitList().stream()
-            .mapToInt(u -> 1 + u.getUnitCount()).sum();
+        return sum(getUnits(), u -> 1 + u.getUnitCount());
     }
 
     /**
@@ -230,31 +251,27 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * @see Unit#isCarrier
      */
     public boolean hasCarrierWithSpace(int space) {
-        return any(getUnitList(),
-            u -> u.isCarrier() && !u.isDamaged() && u.getSpaceLeft() >= space);
+        return any(getUnits(), u -> u.isCarrier() && !u.isDamagedAndUnderForcedRepair()
+            && u.getSpaceLeft() >= space);
     }
 
     /**
      * Gets a list of all naval units here.
      *
-     * @return A list of naval <code>Unit</code>s present.
+     * @return A list of naval {@code Unit}s present.
      */
     public List<Unit> getNavalUnits() {
-        return getUnitList().stream()
-            .filter(Unit::isNaval).collect(Collectors.toList());
+        return transform(getUnits(), Unit::isNaval);
     }
 
     /**
      * Gets a carrier for the supplied unit, if one exists.
      *
-     * @param unit The <code>Unit</code> to carry.
+     * @param unit The {@code Unit} to carry.
      * @return A suitable carrier or null if none found.
      */
     public Unit getCarrierForUnit(Unit unit) {
-        for (Unit u : getUnitList()) {
-            if (u.couldCarry(unit)) return u;
-        }
-        return null;
+        return find(getUnits(), u -> u.couldCarry(unit));
     }
 
 
@@ -264,13 +281,11 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * {@inheritDoc}
      */
     @Override
-    public List<FreeColGameObject> getDisposeList() {
-        List<FreeColGameObject> objects = new ArrayList<>();
-        synchronized (units) {
-            for (Unit u : units) objects.addAll(u.getDisposeList());
+    public Stream<FreeColGameObject> getDisposables() {
+        synchronized (this.units) {
+            return concat(flatten(this.units, Unit::getDisposables),
+                          super.getDisposables());
         }
-        objects.addAll(super.getDisposeList());
-        return objects;
     }
 
     /**
@@ -278,8 +293,8 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      */
     @Override
     public void disposeResources() {
-        synchronized (units) {
-            units.clear();
+        synchronized (this.units) {
+            this.units.clear();
         }
         super.disposeResources();
     }
@@ -324,11 +339,7 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
             if (contains(unit)) {
                 return true;
             } else if (canAdd(unit)) {
-                synchronized (units) {
-                    if (!units.add(unit)) return false;
-                    unit.setLocationNoUpdate(this);
-                    return true;
-                }
+                return addUnit(unit);
             }
         } else if (locatable instanceof Goods) {
             // dumping goods is a valid action
@@ -350,11 +361,11 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     public boolean remove(Locatable locatable) {
         if (locatable instanceof Unit) {
             Unit unit = (Unit)locatable;
-            synchronized (units) {
-                if (!units.remove(unit)) return false;
-                unit.setLocationNoUpdate(null);
-                return true;
+            synchronized (this.units) {
+                if (!this.units.remove(unit)) return false;
             }
+            unit.setLocationNoUpdate(null);
+            return true;
         } else {
             logger.warning("Tried to remove non-Unit " + locatable
                            + " from UnitLocation: " + getId());
@@ -368,8 +379,8 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     @Override
     public boolean contains(Locatable locatable) {
         if (!(locatable instanceof Unit)) return false;
-        synchronized (units) {
-            return units.contains(locatable);
+        synchronized (this.units) {
+            return this.units.contains(locatable);
         }
     }
 
@@ -386,8 +397,18 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      */
     @Override
     public int getUnitCount() {
-        synchronized (units) {
-            return units.size();
+        synchronized (this.units) {
+            return this.units.size();
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public Stream<Unit> getUnits() {
+        synchronized (this.units) {
+            return this.units.stream();
         }
     }
 
@@ -396,20 +417,12 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      */
     @Override
     public List<Unit> getUnitList() {
-        synchronized (units) {
-            return new ArrayList<>(units);
+        synchronized (this.units) {
+            return (this.units.isEmpty()) ? Collections.<Unit>emptyList()
+                : new ArrayList<>(this.units);
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public final Iterator<Unit> getUnitIterator() {
-        // Marked final as this will always work if getUnitList() does.
-        return getUnitList().iterator();
-    }
-
     /**
      * {@inheritDoc}
      */
@@ -430,7 +443,7 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * {@inheritDoc}
      */
     @Override
-    public final Colony getColony() {
+    public Colony getColony() {
         // Final as this will always work if getSettlement() does.
         Settlement settlement = getSettlement();
         return (settlement instanceof Colony) ? (Colony)settlement : null;
@@ -440,7 +453,7 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * {@inheritDoc}
      */
     @Override
-    public final IndianSettlement getIndianSettlement() {
+    public IndianSettlement getIndianSettlement() {
         // Final as this will always work if getSettlement() does.
         Settlement settlement = getSettlement();
         return (settlement instanceof IndianSettlement)
@@ -459,18 +472,17 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * @return The sum of the space taken by the units in this location.
      */
     public int getSpaceTaken() {
-        return getUnitList().stream()
-            .mapToInt(u -> u.getSpaceTaken()).sum();
+        return sum(getUnits(), Unit::getSpaceTaken);
     }
 
     /**
      * Move the given unit to the front of the units list.
      *
-     * @param u The <code>Unit</code> to move to the front.
+     * @param u The {@code Unit} to move to the front.
      */
     public void moveToFront(Unit u) {
-        synchronized (units) {
-            if (units.remove(u)) units.add(0, u);
+        synchronized (this.units) {
+            if (this.units.remove(u)) this.units.add(0, u);
         }
     }
 
@@ -478,13 +490,13 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      * Clear the units from this container.
      */
     protected void clearUnitList() {
-        synchronized (units) {
-            units.clear();
+        synchronized (this.units) {
+            this.units.clear();
         }
     }
 
     /**
-     * Gets the reason why a given <code>Locatable</code> can not be
+     * Gets the reason why a given {@code Locatable} can not be
      * added to this Location.
      *
      * Be careful to test for unit presence last before success
@@ -495,7 +507,7 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
      *
      * FIXME: consider moving this up to Location?
      *
-     * @param locatable The <code>Locatable</code> to test.
+     * @param locatable The {@code Locatable} to test.
      * @return The reason why adding would fail.
      */
     public NoAddReason getNoAddReason(Locatable locatable) {
@@ -512,7 +524,7 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     }
 
     /**
-     * Gets the maximum number of <code>Units</code> this Location
+     * Gets the maximum number of {@code Units} this Location
      * can hold.  To be overridden by subclasses.
      *
      * @return Integer.MAX_VALUE, denoting no effective limit.
@@ -524,19 +536,20 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     /**
      * What would the price be for a collection of goods at this location?
      *
-     * @param goods The list of <code>AbstractGoods</code> to check for.
-     * @return Negative if the goods are unavailable, otherwise the
-     *     price (may be zero).
+     * @param goods The list of {@code AbstractGoods} to check for.
+     * @return The net cost of acquiring the goods at this location.
+     * @exception FreeColException if the goods are unavailable for any reason.
      */
-    public int priceGoods(List<AbstractGoods> goods) {
-        return -1;
+    public int priceGoods(List<AbstractGoods> goods) throws FreeColException {
+        if (goods.isEmpty()) return 0;
+        throw new FreeColException("Not available: " + goods.get(0));
     }
 
     /**
      * Equip a unit for a role using resources at this location.
      *
-     * @param unit The <code>Unit</code> to equip.
-     * @param role The <code>Role</code> to build for.
+     * @param unit The {@code Unit} to equip.
+     * @param role The {@code Role} to build for.
      * @param roleCount The role count.
      * @return True if the equipping succeeded.
      */
@@ -545,6 +558,25 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     }
 
 
+    // Overide FreeColObject
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        UnitLocation o = copyInCast(other, UnitLocation.class);
+        if (o == null || !super.copyIn(o)) return false;
+        final Game game = getGame();
+        clearUnitList();
+        for (Unit u : o.units) { // Allow creation, unit may be new
+            Unit nu = game.update(u, true);
+            addUnit(nu);
+        }
+        return true;
+    }
+
+
     // Serialization
 
     /**
@@ -554,10 +586,12 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     protected void writeChildren(FreeColXMLWriter xw) throws XMLStreamException {
         super.writeChildren(xw);
 
-        synchronized (units) {
-            for (Unit unit : units) {
+        synchronized (this.units) {
+            // Do *not* use getUnits/List here, because Colony lies!
+            for (Unit unit : this.units) {
                 if (unit.getLocation() != this) {
-                    logger.warning("UnitLocation contains unit " + unit
+                    logger.warning("UnitLocation " + this
+                        + " contains unit " + unit
                         + " with bogus location " + unit.getLocation()
                         + ", fixing.");
                     unit.setLocationNoUpdate(this);
@@ -585,20 +619,13 @@ public abstract class UnitLocation extends FreeColGameObject implements Location
     protected void readChild(FreeColXMLReader xr) throws XMLStreamException {
         final String tag = xr.getLocalName();
 
-        if (Unit.getXMLElementTagName().equals(tag)) {
-            Unit u = xr.readFreeColGameObject(getGame(), Unit.class);
-            if (u.getLocation() != this) {
-                logger.warning("Fixing bogus unit location for " + u.getId()
-                    + ", expected " + this.getId()
-                    + " but found " + u.getLocation());
-                u.setLocationNoUpdate(this);
-            }
-            if (u != null) synchronized (units) {
-                units.add(u);
-            }
+        if (Unit.TAG.equals(tag)) {
+            addUnit(xr.readFreeColObject(getGame(), Unit.class));
 
         } else {
             super.readChild(xr);
         }
     }
+
+    // getXMLTagName left to subclasses
 }
