@@ -869,7 +869,7 @@ public final class UnitType extends BuildableType implements Consumer {
 
         final Specification spec = getSpecification();
 
-        UnitType parent = xr.getType(spec, EXTENDS_TAG, UnitType.class, this);
+        UnitType parent = xr.getAlreadyInitializedType(spec, EXTENDS_TAG, UnitType.class, this);
 
         this.baseOffence = xr.getAttribute(OFFENCE_TAG, parent.baseOffence);
 
@@ -969,6 +969,15 @@ public final class UnitType extends BuildableType implements Consumer {
             }
         }
     }
+    
+    /**
+    * {@inheritDoc}
+    */
+    @Override
+    protected void clearContainers(FreeColXMLReader xr) throws XMLStreamException {
+        super.clearContainers(xr);
+        consumption = null;
+    }
 
     /**
      * {@inheritDoc}
@@ -976,14 +985,9 @@ public final class UnitType extends BuildableType implements Consumer {
     @Override
     protected void readChildren(FreeColXMLReader xr) throws XMLStreamException {
         final Specification spec = getSpecification();
-
-        // Clear containers.
-        if (xr.shouldClearContainers()) {
-            consumption = null;
-        }
         defaultRole = spec.getDefaultRole();
 
-        UnitType parent = xr.getType(spec, EXTENDS_TAG, UnitType.class, this);
+        UnitType parent = xr.getAlreadyInitializedType(spec, EXTENDS_TAG, UnitType.class, this);
         if (parent != this) {
             defaultRole = parent.defaultRole;
 
