@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,11 +20,14 @@
 package net.sf.freecol.common.model.pathfinding;
 
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
+import java.util.function.Predicate;
+import java.util.function.ToDoubleFunction;
 
 import net.sf.freecol.common.model.Ability;
-import net.sf.freecol.common.model.FreeColObject;
+import static net.sf.freecol.common.model.Constants.*;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.PathNode;
@@ -32,6 +35,7 @@ import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 
 
 /**
@@ -45,15 +49,15 @@ public final class GoalDeciders {
      *
      * @param all If true, all deciders must succeed (and-semantics),
      *     if false, any decide may succeed (or-semantics).
-     * @param gds A series (two minimum) of <code>GoalDecider</code>s
+     * @param gds A series (two minimum) of {@code GoalDecider}s
      *     to compose.
-     * @return A new <code>GoalDecider</code> composed of the argument
+     * @return A new {@code GoalDecider} composed of the argument
      *     goal deciders.
      */
     public static GoalDecider getComposedGoalDecider(final boolean all,
         final GoalDecider... gds) {
         if (gds.length < 2) {
-            throw new IllegalArgumentException("Short GoalDecider list");
+            throw new RuntimeException("Short GoalDecider list: " + gds.length);
         }
 
         return new GoalDecider() {
@@ -169,7 +173,7 @@ public final class GoalDeciders {
      * Build a simple goal decider to find the first high seas tile
      * without using the unit parameter.
      *
-     * @return A <code>GoalDecider</code> that finds the nearest high seas tile.
+     * @return A {@code GoalDecider} that finds the nearest high seas tile.
      */
     public static GoalDecider getSimpleHighSeasGoalDecider() {
         return new GoalDecider() {
@@ -195,14 +199,14 @@ public final class GoalDeciders {
     /**
      * Builds a simple goal decider to find a single target location.
      *
-     * @param target The target <code>Location</code>.
-     * @return A <code>GoalDecider</code> that only succeeds for the
+     * @param target The target {@code Location}.
+     * @return A {@code GoalDecider} that only succeeds for the
      *     target location.
      */
     public static GoalDecider getLocationGoalDecider(final Location target) {
         return new GoalDecider() {
             private PathNode best = null;
-            private int bestCost = FreeColObject.INFINITY;
+            private int bestCost = INFINITY;
 
             @Override
             public PathNode getGoal() { return best; }
@@ -226,8 +230,8 @@ public final class GoalDeciders {
     /**
      * Builds a goal decider to find an adjacent tile to a target location.
      *
-     * @param target The target <code>Location</code>.
-     * @return A <code>GoalDecider</code> that only succeeds for tiles
+     * @param target The target {@code Location}.
+     * @return A {@code GoalDecider} that only succeeds for tiles
      *     adjacent to the target location.
      */
     public static GoalDecider getAdjacentLocationGoalDecider(Location target) {
@@ -257,8 +261,8 @@ public final class GoalDeciders {
      * Get a goal decider that succeeds for settlements owned by one
      * of a given list of enemies.
      *
-     * @param enemies The list of enemy <code>Player</code>s.
-     * @return A suitable <code>GoalDecider</code>.
+     * @param enemies The list of enemy {@code Player}s.
+     * @return A suitable {@code GoalDecider}.
      **/
     public static GoalDecider getEnemySettlementGoalDecider(final Collection<Player> enemies) {
         return new GoalDecider() {
@@ -294,13 +298,14 @@ public final class GoalDeciders {
      * - Favour a short journey to the target
      * - Prioritize not landing next to a hostile fort/fortress.
      *
-     * @param target The target <code>Tile</code>.
-     * @return A suitable <code>GoalDecider</code>.
+     * @param target The target {@code Tile}.
+     * @return A suitable {@code GoalDecider}.
      */
     public static GoalDecider getDisembarkGoalDecider(final Tile target) {
+        final double NO_DANGER_BONUS = 1000.0;
+        
         return new GoalDecider() {
             private double bestScore = -1.0;
-            private boolean goalDangerous = true;
             private PathNode goal = null;
 
             @Override
@@ -309,44 +314,34 @@ public final class GoalDeciders {
             public boolean hasSubGoals() { return true; }
             @Override
             public boolean check(Unit u, PathNode pathNode) {
-                Tile tile = pathNode.getTile();
+                final Tile tile = pathNode.getTile();
                 if (tile == null || !tile.isLand() || !tile.isEmpty()
                     || tile.hasSettlement()) return false;
+
                 final Player owner = u.getOwner();
-                boolean found = false, danger = false;
-                for (Tile t : pathNode.getTile().getSurroundingTiles(1)) {
-                    if (!t.isHighSeasConnected() || t.isLand()) continue;
-                    for (Tile t2 : t.getSurroundingTiles(1)) {
-                        Settlement settlement = t2.getSettlement();
-                        if (settlement != null
-                            && !owner.owns(settlement)
-                            && settlement.hasAbility(Ability.BOMBARD_SHIPS)
-                            && (owner.atWarWith(settlement.getOwner())
-                                || u.hasAbility(Ability.PIRACY))) {
-                            danger = true;
-                            break;
-                        }
-                    }
-                    if (goalDangerous) {
-                        found = true;
-                        if (!danger) {
-                            // Found first non-dangerous tile, reset scores
-                            goalDangerous = false;
-                            bestScore = -1.0f;
-                        }
-                    } else {
-                        found = !danger; // Only accept non-dangerous tiles
-                    }
-                }
-                if (found) {
-                    double distance = 1.0 + u.getGame().getMap()
-                        .getDistance(target, tile);
-                    double score = tile.getDefenceValue() / distance;
-                    if (bestScore < score) {
-                        bestScore = score;
-                        goal = pathNode;
-                        return true;
-                    }
+                final Map map = u.getGame().getMap();
+                final Predicate<Tile> dockPred = t ->
+                    t.isHighSeasConnected() && !t.isLand();
+                final Predicate<Tile> dangerPred = t -> {
+                    Settlement settlement = t.getSettlement();
+                    return (settlement != null
+                        && !owner.owns(settlement)
+                        && settlement.hasAbility(Ability.BOMBARD_SHIPS)
+                        && (owner.atWarWith(settlement.getOwner())
+                            || u.hasAbility(Ability.PIRACY)));
+                };
+                final ToDoubleFunction<Tile> tileScorer = cacheDouble(t ->
+                    (t.getDefenceValue() / (1.0 + map.getDistance(target, t))
+                        + ((none(t.getSurroundingTiles(1, 1), dangerPred))
+                            ? NO_DANGER_BONUS : 0.0)));
+                Tile best = maximize(tile.getSurroundingTiles(1, 1), dockPred,
+                                     Comparator.comparingDouble(tileScorer));
+                double score;
+                if (best != null
+                    && (score = tileScorer.applyAsDouble(best)) > bestScore) {
+                    bestScore = score;
+                    goal = pathNode;
+                    return true;
                 }
                 return false;
             }
@@ -356,8 +351,8 @@ public final class GoalDeciders {
     /**
      * Get a goal decider to find tiles that an enemy player can not see.
      *
-     * @param enemy The enemy <code>Player</code> to avoid.
-     * @return A suitable <code>GoalDecider</code>.
+     * @param enemy The enemy {@code Player} to avoid.
+     * @return A suitable {@code GoalDecider}.
      */
     public static GoalDecider getStealthyGoalDecider(final Player enemy) {
         return new GoalDecider() {
@@ -382,8 +377,8 @@ public final class GoalDeciders {
      * high seas count than a unit currently has.  Useful for tunnelling
      * out of intermittantly blocked rivers.
      *
-     * @param unit The <code>Unit</code> to get the goal decider for.
-     * @return A suitable <code>GoalDecider</code>.
+     * @param unit The {@code Unit} to get the goal decider for.
+     * @return A suitable {@code GoalDecider}.
      */
     public static GoalDecider getReduceHighSeasCountGoalDecider(final Unit unit) {
         return new GoalDecider() {
@@ -455,7 +450,7 @@ public final class GoalDeciders {
         /**
          * Create a multiple decider.
          *
-         * @param locs The list of <code>Location</code>s to search for
+         * @param locs The list of {@code Location}s to search for
          *     paths to an adjacent location for.
          */
         public MultipleAdjacentDecider(final List<Location> locs) {
@@ -469,13 +464,12 @@ public final class GoalDeciders {
                     public boolean check(Unit u, PathNode path) {
                         Tile tile = path.getTile();
                         if (tile == null) return false;
-                        for (Location loc : locs) {
-                            if (tile.isAdjacent(loc.getTile())) {
-                                PathNode p = results.get(loc);
-                                if (p == null
-                                    || p.getCost() > path.getCost()) {
-                                    results.put(loc, path);
-                                }
+                        for (Location loc : transform(locs,
+                                l -> tile.isAdjacent(l.getTile()))) {
+                            PathNode p = results.get(loc);
+                            if (p == null
+                                || p.getCost() > path.getCost()) {
+                                results.put(loc, path);
                             }
                         }
                         return false;
