@@ -37,13 +37,13 @@ public class VideoResource extends Resource {
     /**
      * Do not use directly.
      *
-     * @param primaryKey The primary key.
+     * @param cachingKey The caching key.
      * @param resourceLocator The {@code URI} used when loading this
      *     resource.
      * @exception IOException if the URI is malformed.
      */
-    public VideoResource(String primaryKey, URI resourceLocator) throws IOException {
-        super(primaryKey, resourceLocator);
+    public VideoResource(String cachingKey, URI resourceLocator) throws IOException {
+        super(cachingKey, resourceLocator);
         
         this.video = new Video(resourceLocator.toURL());
     }
