@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,14 +19,18 @@
 
 package net.sf.freecol.server.ai.mission;
 
+import java.util.Comparator;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
 
+import net.sf.freecol.common.debug.DebugUtils;
+import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
 import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.Constants;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Direction;
@@ -40,7 +44,7 @@ import net.sf.freecol.common.model.pathfinding.CostDecider;
 import net.sf.freecol.common.model.pathfinding.CostDeciders;
 import net.sf.freecol.common.model.pathfinding.GoalDecider;
 import net.sf.freecol.common.model.pathfinding.GoalDeciders;
-import net.sf.freecol.common.networking.NetworkConstants;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
 import net.sf.freecol.server.ai.AIColony;
 import net.sf.freecol.server.ai.AIMain;
@@ -53,14 +57,17 @@ import net.sf.freecol.server.ai.TileImprovementPlan;
 /**
  * Mission for controlling a pioneer.
  */
-public class PioneeringMission extends Mission {
+public final class PioneeringMission extends Mission {
 
     private static final Logger logger = Logger.getLogger(PioneeringMission.class.getName());
 
+    public static final String TAG = "pioneeringMission";
+
     /** The tag for this mission. */
     private static final String tag = "AI pioneer";
 
 
+    // FIXME: This is unused, delete?
     /**
      * Default distance in turns to a threatening unit.
      */
@@ -79,25 +86,26 @@ public class PioneeringMission extends Mission {
 
 
     /**
-     * Creates a pioneering mission for the given <code>AIUnit</code>.
+     * Creates a pioneering mission for the given {@code AIUnit}.
      * Note that PioneeringMission.isValid(aiUnit) should be called
      * before this, to guarantee that
      * findTileImprovementPlan/findColonyWithTools succeed.
      *
      * @param aiMain The main AI-object.
-     * @param aiUnit The <code>AIUnit</code> this mission is created for.
-     * @param target The target <code>Location</code>.
+     * @param aiUnit The {@code AIUnit} this mission is created for.
+     * @param target The target {@code Location}.
      */
     public PioneeringMission(AIMain aiMain, AIUnit aiUnit, Location target) {
-        super(aiMain, aiUnit, target);
+        super(aiMain, aiUnit);
+
+        setTarget(target);
     }
 
     /**
-     * Creates a new <code>PioneeringMission</code> and reads the
-     * given element.
+     * Creates a new {@code PioneeringMission} from a reader.
      *
      * @param aiMain The main AI-object.
-     * @param aiUnit The <code>AIUnit</code> this mission is created for.
+     * @param aiUnit The {@code AIUnit} this mission is created for.
      * @param xr The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered
      *      during parsing.
@@ -114,7 +122,8 @@ public class PioneeringMission extends Mission {
     /**
      * Get the best improvement associated with a tile.
      *
-     * @return The <code>TileImprovementPlan</code>, or null if not found.
+     * @param tile The {@code Tile} to get a plan for.
+     * @return The {@code TileImprovementPlan}, or null if not found.
      */
     private TileImprovementPlan getBestPlan(Tile tile) {
         return getEuropeanAIPlayer().getBestPlan(tile);
@@ -125,26 +134,28 @@ public class PioneeringMission extends Mission {
      * Take care to first check if the unit has a plan already, if so,
      * return that.
      *
-     * @return The <code>TileImprovementPlan</code>, or null if not found.
+     * @param aiUnit The {@code AIUnit} to get a plan for.
+     * @param tile The {@code Tile} to get a plan for.
+     * @return The {@code TileImprovementPlan}, or null if not found.
      */
-    private static TileImprovementPlan getBestPlan(AIUnit aiUnit, Tile tile) {
+    private static TileImprovementPlan getBestMissionPlan(AIUnit aiUnit, Tile tile) {
         return ((EuropeanAIPlayer)aiUnit.getAIOwner()).getBestPlan(tile);
     }
 
     /**
-     * Gets the <code>TileImprovementPlan</code> for this mission.
+     * Gets the {@code TileImprovementPlan} for this mission.
      *
-     * @return The <code>TileImprovementPlan</code>.
+     * @return The {@code TileImprovementPlan}.
      */
     public TileImprovementPlan getTileImprovementPlan() {
         return tileImprovementPlan;
     }
 
     /**
-     * Sets the <code>TileImprovementPlan</code> which should
+     * Sets the {@code TileImprovementPlan} which should
      * be the next target.
      *
-     * @param tip The <code>TileImprovementPlan</code>.
+     * @param tip The {@code TileImprovementPlan}.
      */
     public void setTileImprovementPlan(TileImprovementPlan tip) {
         TileImprovementPlan old = tileImprovementPlan;
@@ -185,10 +196,10 @@ public class PioneeringMission extends Mission {
     /**
      * Does a supplied unit have tools?
      *
-     * @param aiUnit The pioneer <code>AIUnit</code> to check.
+     * @param aiUnit The pioneer {@code AIUnit} to check.
      * @return True if the pioneer has tools.
      */
-    private static boolean hasTools(AIUnit aiUnit) {
+    private static boolean hasMissionTools(AIUnit aiUnit) {
         return aiUnit.getUnit().hasAbility(Ability.IMPROVE_TERRAIN);
     }
 
@@ -198,24 +209,24 @@ public class PioneeringMission extends Mission {
      * @return True if the pioneer has tools.
      */
     private boolean hasTools() {
-        return hasTools(getAIUnit());
+        return hasMissionTools(getAIUnit());
     }
 
     /**
      * Extract a valid target for this mission from a path.
      *
-     * @param aiUnit A <code>AIUnit</code> to perform the mission.
-     * @param path A <code>PathNode</code> to extract a target from.
+     * @param aiUnit A {@code AIUnit} to perform the mission.
+     * @param path A {@code PathNode} to extract a target from.
      * @return A target for this mission, or null if none found.
      */
-    public static Location extractTarget(AIUnit aiUnit, PathNode path) {
+    private static Location extractTarget(AIUnit aiUnit, PathNode path) {
         if (path == null) return null;
         final Location loc = path.getLastNode().getLocation();
         return (loc == null) ? null
-            : ((hasTools(aiUnit))
-                ? ((invalidReason(aiUnit, loc.getTile()) != null) ? null
+            : ((hasMissionTools(aiUnit))
+                ? ((invalidTileReason(aiUnit, loc.getTile()) != null) ? null
                     : loc.getTile())
-                : ((invalidReason(aiUnit, loc.getColony()) != null) ? null
+                : ((invalidColonyReason(aiUnit, loc.getColony()) != null) ? null
                     : loc.getColony()));
     }
 
@@ -223,16 +234,16 @@ public class PioneeringMission extends Mission {
      * Evaluate a potential pioneering mission for a given unit and
      * path.
      *
-     * @param aiUnit The <code>AIUnit</code> to do the mission.
-     * @param path A <code>PathNode</code> to take to the target.
+     * @param aiUnit The {@code AIUnit} to do the mission.
+     * @param path A {@code PathNode} to take to the target.
      * @return A score for the proposed mission.
      */
     public static int scorePath(AIUnit aiUnit, PathNode path) {
         final Location loc = extractTarget(aiUnit, path);
-        if (hasTools(aiUnit)) {
+        if (hasMissionTools(aiUnit)) {
             TileImprovementPlan tip;
             if (loc instanceof Tile
-                && (tip = getBestPlan(aiUnit, (Tile)loc)) != null) {
+                && (tip = getBestMissionPlan(aiUnit, (Tile)loc)) != null) {
                 return 1000 * tip.getValue() / (path.getTotalTurns() + 1);
             }
         } else {
@@ -246,10 +257,10 @@ public class PioneeringMission extends Mission {
     /**
      * Makes a goal decider that checks pioneering sites.
      *
-     * @param aiUnit The <code>AIUnit</code> to search with.
+     * @param aiUnit The {@code AIUnit} to search with.
      * @param deferOK Keep track of the nearest colonies to use as a
      *     fallback destination.
-     * @return A suitable <code>GoalDecider</code>.
+     * @return A suitable {@code GoalDecider}.
      */
     private static GoalDecider getGoalDecider(final AIUnit aiUnit,
                                               final boolean deferOK) {
@@ -280,13 +291,13 @@ public class PioneeringMission extends Mission {
     /**
      * Finds a suitable pioneering target for the supplied unit.
      *
-     * @param aiUnit The <code>AIUnit</code> to execute this mission.
+     * @param aiUnit The {@code AIUnit} to execute this mission.
      * @param range An upper bound on the number of moves.
      * @param deferOK Enables deferring to a fallback colony.
      * @return A path to the new target, or null if none found.
      */
-    public static PathNode findTargetPath(AIUnit aiUnit, int range,
-                                          boolean deferOK) {
+    private static PathNode findTargetPath(AIUnit aiUnit, int range,
+                                           boolean deferOK) {
         if (invalidAIUnitReason(aiUnit) != null) return null;
         final Unit unit = aiUnit.getUnit();
         final Location start = unit.getPathStartLocation();
@@ -302,25 +313,22 @@ public class PioneeringMission extends Mission {
     /**
      * Gets the Colony that most needs a pioneer.
      *
-     * @param aiUnit The pioneer <code>AIUnit</code>.
+     * @param aiUnit The pioneer {@code AIUnit}.
      * @return The colony with the most outstanding tile improvement plans.
      */
     private static Colony getBestPioneeringColony(AIUnit aiUnit) {
-        EuropeanAIPlayer aiPlayer = (EuropeanAIPlayer)aiUnit.getAIOwner();
-        AIColony bestColony = null;
-        int bestValue = -1;
-        for (AIColony aic : aiPlayer.getAIColonies()) {
-            int value = aic.getTileImprovementPlans().size();
-            if (value > bestValue) {
-                bestValue = value;
-                bestColony = aic;
-            }
-        }
-        if (bestColony == null) return null;
-        Colony colony = bestColony.getColony();
-        if (colony.isConnectedPort()) return colony;
-        PathNode path = aiUnit.getUnit().findOurNearestPort();
-        return (path == null) ? colony
+        final Comparator<AIColony> comp = Comparator.comparingInt(c ->
+            c.getTileImprovementPlans().size());
+        final EuropeanAIPlayer owner = (EuropeanAIPlayer)aiUnit.getAIOwner();
+        AIColony bestColony = maximize(owner.getAIColonies(), comp);
+            
+        PathNode path;
+        Colony colony;
+        return (bestColony == null)
+            ? null
+            : ((colony = bestColony.getColony()).isConnectedPort()
+                || (path = aiUnit.getUnit().findOurNearestPort()) == null)
+            ? colony
             : path.getLastNode().getLocation().getColony();
     }
 
@@ -328,37 +336,37 @@ public class PioneeringMission extends Mission {
      * Finds a suitable pioneering target for the supplied unit.
      * Falls back to the best settlement if the unit is not on the map.
      *
-     * @param aiUnit The <code>AIUnit</code> to test.
+     * @param aiUnit The {@code AIUnit} to test.
      * @param range An upper bound on the number of moves.
      * @param deferOK Enables deferring to a fallback colony.
      * @return A target for this mission.
      */
-    public static Location findTarget(AIUnit aiUnit, int range,
-                                      boolean deferOK) {
+    public static Location findMissionTarget(AIUnit aiUnit, int range,
+                                             boolean deferOK) {
         PathNode path = findTargetPath(aiUnit, range, false);
         if (path != null) return extractTarget(aiUnit, path);
         if (deferOK) return getBestPioneeringColony(aiUnit);
         Location loc = findCircleTarget(aiUnit, getGoalDecider(aiUnit, false),
                                         range*3, false);
-        return (hasTools(aiUnit)) ? loc : Location.upLoc(loc);
+        return (hasMissionTools(aiUnit)) ? loc : Location.upLoc(loc);
     }
 
     /**
      * Prepare a unit for this mission.
      *
-     * @param aiUnit The <code>AIUnit</code> to prepare.
+     * @param aiUnit The {@code AIUnit} to prepare.
      * @return A reason why the unit can not perform this mission, or null
      *     if none.
      */
     public static String prepare(AIUnit aiUnit) {
-        String reason = invalidReason(aiUnit);
+        String reason = invalidUnitReason(aiUnit);
         if (reason != null) return reason;
         final Unit unit = aiUnit.getUnit();
-        if (!hasTools(aiUnit)
+        if (!hasMissionTools(aiUnit)
             && !aiUnit.equipForRole(unit.getSpecification().getPioneerRole())) {
             return "unit-could-not-equip";
         }
-        return (hasTools(aiUnit) || unit.hasAbility(Ability.EXPERT_PIONEER))
+        return (hasMissionTools(aiUnit) || unit.hasAbility(Ability.EXPERT_PIONEER))
             ? null
             : "unit-missing-tools";
     }
@@ -366,11 +374,11 @@ public class PioneeringMission extends Mission {
     /**
      * Why would a PioneeringMission be invalid with the given unit.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
+     * @param aiUnit The {@code AIUnit} to check.
      * @return A reason why the mission would be invalid with the unit,
      *     or null if none found.
      */
-    private static String invalidMissionReason(AIUnit aiUnit) {
+    private static String invalidUnitReason(AIUnit aiUnit) {
         String reason = invalidAIUnitReason(aiUnit);
         return (reason != null) ? reason
             : (!aiUnit.getUnit().isPerson()) ? Mission.UNITNOTAPERSON
@@ -380,8 +388,8 @@ public class PioneeringMission extends Mission {
     /**
      * Why would a PioneeringMission be invalid with the given unit and colony.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param colony The <code>Colony</code> to check.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param colony The {@code Colony} to check.
      * @return A reason why the mission would be invalid, or null if
      *     none found.
      */
@@ -390,8 +398,8 @@ public class PioneeringMission extends Mission {
         Role role = aiUnit.getUnit().getSpecification().getRole("model.role.pioneer");
         return (reason != null)
             ? reason
-            : (!hasTools(aiUnit)
-               && !colony.canProvideGoods(role.getRequiredGoods()))
+            : (!hasMissionTools(aiUnit)
+               && !colony.canProvideGoods(role.getRequiredGoodsList()))
             ? "colony-can-not-provide-equipment"
             : null;
     }
@@ -399,9 +407,9 @@ public class PioneeringMission extends Mission {
     /**
      * Gets the existing tile improvement plan for a unit and tile.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param tile The <code>Tile</code> to check.
-     * @return The associated <code>TileImprovementPlan</code>.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param tile The {@code Tile} to check.
+     * @return The associated {@code TileImprovementPlan}.
      */
     private static TileImprovementPlan getPlan(AIUnit aiUnit, Tile tile) {
         PioneeringMission pm = aiUnit.getMission(PioneeringMission.class);
@@ -415,18 +423,18 @@ public class PioneeringMission extends Mission {
     /**
      * Why would a PioneeringMission be invalid with the given unit and tile.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param tile The <code>Tile</code> to check.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param tile The {@code Tile} to check.
      * @return A reason why the mission would be invalid, or null if
      *      none found.
      */
     private static String invalidTileReason(AIUnit aiUnit, Tile tile) {
         return (tile == null)
             ? Mission.TARGETINVALID
-            : (!hasTools(aiUnit))
+            : (!hasMissionTools(aiUnit))
             ? "unit-needs-tools"
             : (getPlan(aiUnit, tile) == null
-                && getBestPlan(aiUnit, tile) == null)
+                && getBestMissionPlan(aiUnit, tile) == null)
             ? "tile-has-no-plan"
             : (tile.getOwningSettlement() != null)
             ? invalidTargetReason(tile.getOwningSettlement(),
@@ -437,28 +445,28 @@ public class PioneeringMission extends Mission {
     /**
      * Why would this mission be invalid with the given AI unit?
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
+     * @param aiUnit The {@code AIUnit} to check.
      * @return A reason for mission invalidity, or null if none found.
      */
-    public static String invalidReason(AIUnit aiUnit) {
-        return invalidMissionReason(aiUnit);
+    public static String invalidMissionReason(AIUnit aiUnit) {
+        return invalidUnitReason(aiUnit);
     }
 
     /**
      * Why would this mission be invalid with the given AI unit and location?
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param loc The <code>Location</code> to check.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param loc The {@code Location} to check.
      * @return A reason for invalidity, or null if none found.
      */
-    public static String invalidReason(AIUnit aiUnit, Location loc) {
+    public static String invalidMissionReason(AIUnit aiUnit, Location loc) {
         String reason = invalidMissionReason(aiUnit);
         return (reason != null)
             ? reason
             : (loc instanceof Tile)
             ? invalidTileReason(aiUnit, (Tile)loc)
             : (loc instanceof Colony)
-            ? ((aiUnit.getUnit().getLocation() instanceof Tile)
+            ? ((aiUnit.getUnit().isOnTile())
                 ? invalidColonyReason(aiUnit, (Colony)loc)
                 : invalidTargetReason(loc, aiUnit.getUnit().getOwner()))
             : Mission.TARGETINVALID;
@@ -503,7 +511,7 @@ public class PioneeringMission extends Mission {
      */
     @Override
     public Location findTarget() {
-        return findTarget(getAIUnit(), 10, true);
+        return findMissionTarget(getAIUnit(), 10, true);
     }
     
     /**
@@ -518,7 +526,7 @@ public class PioneeringMission extends Mission {
             }
             if (tileImprovementPlan.isComplete()) return null;
         }
-        return invalidReason(getAIUnit(), getTarget());
+        return invalidMissionReason(getAIUnit(), getTarget());
     }
 
     /**
@@ -584,10 +592,12 @@ public class PioneeringMission extends Mission {
             case MOVE: // Arrived
                 break;
 
-            case MOVE_HIGH_SEAS: case MOVE_NO_MOVES:
-            case MOVE_NO_REPAIR: case MOVE_ILLEGAL:
+            case MOVE_HIGH_SEAS: case MOVE_NO_MOVES: case MOVE_ILLEGAL:
                 return lbWait(lb);
 
+            case MOVE_NO_REPAIR:
+                return lbFail(lb, false, AIUNITDIED);
+
             case MOVE_NO_ACCESS_EMBARK: case MOVE_NO_TILE:
                 return this;
 
@@ -600,13 +610,13 @@ public class PioneeringMission extends Mission {
             if (aiUnit.equipForRole(getSpecification().getPioneerRole())
                 && hasTools()) {
                 lb.add(", equips");
-                newTarget = findTarget(aiUnit, 10, false);
+                newTarget = findMissionTarget(aiUnit, 10, false);
                 if (newTarget == null) {
                     return lbFail(lb, false, "no pioneering target");
                 }
             } else {
                 lb.add(", fails to equip");
-                newTarget = findTarget(aiUnit, 10, false);
+                newTarget = findMissionTarget(aiUnit, 10, false);
                 if (newTarget == null
                     || Map.isSameLocation(newTarget, getTarget())) {
                     return lbFail(lb, false, "no tools target");
@@ -621,21 +631,24 @@ public class PioneeringMission extends Mission {
             && invalidTargetReason(getTarget(), player) == null) {
             Unit.MoveType mt = travelToTarget(getTarget(), costDecider, lb);
             switch (mt) {
-            case MOVE_HIGH_SEAS: case MOVE_NO_REPAIR:
+            case MOVE:
+                break;
+
+            case MOVE_HIGH_SEAS:
                 return lbWait(lb);
 
+            case MOVE_NO_REPAIR:
+                return lbFail(lb, false, AIUNITDIED);
+
             case MOVE_NO_MOVES: case MOVE_NO_TILE: case MOVE_ILLEGAL:
                 return this;
 
-            case MOVE:
-                break;
-
             default:
                 return lbMove(lb, mt);
             }
 
             lbAt(lb);
-            newTarget = findTarget(aiUnit, 10, false);
+            newTarget = findMissionTarget(aiUnit, 10, false);
             if (newTarget == null) {
                 return lbFail(lb, false, "found no pioneering target");
             }
@@ -672,15 +685,18 @@ public class PioneeringMission extends Mission {
         for (;;) {
             Unit.MoveType mt = travelToTarget(getTarget(), costDecider, lb);
             switch (mt) {
-            case MOVE_HIGH_SEAS: case MOVE_NO_REPAIR:
-                return lbWait(lb);
-                
-            case MOVE_NO_MOVES: case MOVE_NO_TILE: case MOVE_ILLEGAL:
-                return this;
-                
             case MOVE: // Arrived
                 break;
                 
+            case MOVE_HIGH_SEAS:
+                return lbWait(lb);
+                
+            case MOVE_NO_REPAIR:
+                return lbFail(lb, false, AIUNITDIED);
+
+            case MOVE_NO_MOVES: case MOVE_NO_TILE: case MOVE_ILLEGAL:
+                return this;
+                
             case MOVE_NO_ATTACK_CIVILIAN:
                 // Might be a temporary blockage due to an occupying
                 // unit at the target.  Move randomly and retry if
@@ -700,13 +716,13 @@ public class PioneeringMission extends Mission {
         if (!player.owns(tile)) {
             // FIXME: Better choice whether to pay or steal.
             // Currently always pay if we can, steal if we can not.
-            boolean fail = false;
+            boolean fail;
             int price = player.getLandPrice(tile);
             if (price < 0) {
                 fail = true;
             } else {
                 if (price > 0 && !player.checkGold(price)) {
-                    price = NetworkConstants.STEAL_LAND;
+                    price = Constants.STEAL_LAND;
                 }
                 fail = !AIMessage.askClaimLand(tile, aiUnit, price)
                     || !player.owns(tile); // Failed to claim ownership
@@ -715,7 +731,7 @@ public class PioneeringMission extends Mission {
                 aiPlayer.removeTileImprovementPlan(tileImprovementPlan);
                 tileImprovementPlan.dispose();
                 lb.add(", land claim failed at ", tile);
-                if ((newTarget = findTarget(aiUnit, 10, false)) == null) {
+                if ((newTarget = findMissionTarget(aiUnit, 10, false)) == null) {
                     return lbFail(lb, false, "no alternate target");
                 }
                 setTarget(newTarget);
@@ -757,7 +773,7 @@ public class PioneeringMission extends Mission {
             xw.writeAttribute(TARGET_TAG, target.getId());
 
             if (tileImprovementPlan != null) {
-                xw.writeAttribute(TileImprovementPlan.getXMLElementTagName(),
+                xw.writeAttribute(TileImprovementPlan.TAG,
                                   tileImprovementPlan);
             }
         }
@@ -775,8 +791,8 @@ public class PioneeringMission extends Mission {
         // Do not use setTarget in serialization
         target = xr.getLocationAttribute(getGame(), TARGET_TAG, false);
 
-        tileImprovementPlan = (xr.hasAttribute(TileImprovementPlan.getXMLElementTagName()))
-            ? xr.makeAIObject(aiMain, TileImprovementPlan.getXMLElementTagName(),
+        tileImprovementPlan = (xr.hasAttribute(TileImprovementPlan.TAG))
+            ? xr.makeAIObject(aiMain, TileImprovementPlan.TAG,
                 TileImprovementPlan.class, (TileImprovementPlan)null, true)
             : null;
     }
@@ -785,14 +801,11 @@ public class PioneeringMission extends Mission {
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "pioneeringMission".
-     */
-    public static String getXMLElementTagName() {
-        return "pioneeringMission";
+    public String getXMLTagName() { return TAG; }
+    
+    public String toStringForDebugExtraMissionInfo() {
+        final String targetString = DebugUtils.locationDisplayString(getTarget());
+        final String tileImprovement = (tileImprovementPlan != null) ? Messages.message(tileImprovementPlan.getType().getNameKey()) : "No plan";
+        return tileImprovement + " at " + targetString;
     }
 }
