@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,6 +19,7 @@
 
 package net.sf.freecol.server.generator;
 
+import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.util.LogBuilder;
 
@@ -31,16 +32,23 @@ public interface MapGenerator {
     /**
      * Create an empty map.
      *
+     * @param game The {@code Game} to generate for.
      * @param width The map width.
      * @param height The map height.
-     * @return A new empty <code>Map</code>.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A new empty {@code Map}.
      */
-    public abstract Map createEmptyMap(int width, int height, LogBuilder lb);
+    public abstract Map generateEmptyMap(Game game, int width, int height,
+                                         LogBuilder lb);
 
     /**
      * Creates the map with the current set options
      *
-     * @return The new <code>Map</code>.
+     * @param game The {@code Game} to generate for.
+     * @param importMap An optional {@code Map} to import.
+     * @param generateEuropeanPlayerUnits If {@code true}, then the european player units are generated.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return The new {@code Map}.
      */
-    public abstract Map createMap(LogBuilder lb);
+    public abstract Map generateMap(Game game, Map importMap, boolean generateEuropeanPlayerUnits, LogBuilder lb);
 }
