@@ -23,6 +23,7 @@ import java.awt.image.BufferedImage;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.InputStream;
 
 import javax.imageio.ImageIO;
 import javax.xml.stream.XMLStreamException;
@@ -59,8 +60,8 @@ public class MapConverter {
                     System.out.println("Renamed " + filename + " to " + newName + ".");
                     FreeColSavegameFile savegame = new FreeColSavegameFile(in);
                     BufferedImage thumbnail = null;
-                    try {
-                        thumbnail = ImageIO.read(savegame.getInputStream(FreeColSavegameFile.THUMBNAIL_FILE));
+                    try (InputStream thumbnailIn = savegame.getInputStream(FreeColSavegameFile.THUMBNAIL_FILE)) {
+                        thumbnail = ImageIO.read(thumbnailIn);
                         System.out.println("Loaded thumbnail.");
                     } catch (FileNotFoundException e) {
                         System.err.println("No thumbnail present.");
