@@ -21,6 +21,7 @@ package net.sf.freecol.tools;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -29,11 +30,12 @@ import javax.xml.validation.Schema;
 import javax.xml.validation.SchemaFactory;
 import javax.xml.validation.Validator;
 
-import net.sf.freecol.common.io.FreeColDirectories;
-import net.sf.freecol.common.io.FreeColSavegameFile;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 
+import net.sf.freecol.common.io.FreeColDirectories;
+import net.sf.freecol.common.io.FreeColSavegameFile;
+
 
 /**
  * Validate a saved game.
@@ -43,6 +45,7 @@ public class SaveGameValidator {
     public static void main(String[] args) throws Exception {
 
         SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
+        factory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
         File schemaLocation = new File("schema/data/data-savedGame.xsd");
         Schema schema = factory.newSchema(schemaLocation);
         Validator saveGameValidator = schema.newValidator();
@@ -64,7 +67,9 @@ public class SaveGameValidator {
             //System.out.println("Processing file " + file.getPath());
             try {
                 FreeColSavegameFile mapFile = new FreeColSavegameFile(file);
-                saveGameValidator.validate(new StreamSource(mapFile.getSavegameInputStream()));
+                try (InputStream in = mapFile.getSavegameInputStream()) {
+                    saveGameValidator.validate(new StreamSource(in));
+                }
                 System.out.println("Successfully validated " + file.getName());
             } catch (SAXParseException e) {
                 System.out.println(e.getMessage() 
