@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2022  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,19 +19,11 @@
 
 package net.sf.freecol.common.model;
 
-import java.io.ByteArrayInputStream;
-import java.lang.reflect.Field;
-import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.parsers.DocumentBuilder;
-
-import net.sf.freecol.common.model.Role;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.server.model.ServerColony;
 import net.sf.freecol.server.model.ServerUnit;
 import net.sf.freecol.util.test.FreeColTestCase;
 
-import org.w3c.dom.Document;
-import org.w3c.dom.Element;
-
 
 public class UnitTest extends FreeColTestCase {
 
@@ -104,11 +96,11 @@ public class UnitTest extends FreeColTestCase {
      */
     public void testIsColonist() {
         Game game = getStandardGame();
+        Map map = getTestMap(plains, true);
+        game.changeMap(map);
+
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player sioux = game.getPlayerByNationId("model.nation.sioux");
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
-
         Tile tile1 = map.getTile(6, 8);
         Tile tile2 = map.getTile(6, 9);
 
@@ -173,7 +165,7 @@ public class UnitTest extends FreeColTestCase {
     public void testFailedAddGoods() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         Colony colony = this.getStandardColony();
         int foodInColony = 300;
@@ -203,7 +195,8 @@ public class UnitTest extends FreeColTestCase {
     public void testMissionary() {
         Game game = getStandardGame();
         Map map = getTestMap(plains, true);
-        game.setMap(map);
+        game.changeMap(map);
+
         Player sioux = game.getPlayerByNationId("model.nation.sioux");
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Tile tile = map.getTile(6, 9);
@@ -240,7 +233,7 @@ public class UnitTest extends FreeColTestCase {
     public void testGetLineOfSight() {
         final Game game = getStandardGame();
         final Map map = getTestMap(plains, true);
-        game.setMap(map);
+        game.changeMap(map);
 
         Player player = game.getPlayerByNationId("model.nation.dutch");
         Tile tile = map.getTile(6, 9);
@@ -271,11 +264,11 @@ public class UnitTest extends FreeColTestCase {
 
     public void testUnitCanBuildColony() {
         Game game = getStandardGame();
+        Map map = getTestMap(plains, true);
+        game.changeMap(map);
+
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player sioux = game.getPlayerByNationId("model.nation.sioux");
-
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
         Tile tile1 = map.getTile(10, 4);
 
         Unit farmer = new ServerUnit(game, tile1, dutch, expertFarmerType);
@@ -304,7 +297,7 @@ public class UnitTest extends FreeColTestCase {
     public void testIndianDies() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         Player indianPlayer = game.getPlayerByNationId("model.nation.sioux");
         FreeColTestCase.IndianSettlementBuilder builder
@@ -332,7 +325,6 @@ public class UnitTest extends FreeColTestCase {
                     indianPlayer.hasUnit(brave));
     }
 
-
     public void testUnitAvailability() {
         Game game = getStandardGame();
         Player indian = game.getPlayerByNationId("model.nation.sioux");
@@ -362,11 +354,10 @@ public class UnitTest extends FreeColTestCase {
     public void testUnitLocationAfterBuildingColony() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
+
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
-
         Tile colonyTile = map.getTile(6, 8);
-
         Unit soldier = new ServerUnit(game, colonyTile, dutch,
                                       veteranSoldierType);
         assertEquals("Soldier location should be the colony tile", colonyTile,
@@ -393,12 +384,9 @@ public class UnitTest extends FreeColTestCase {
             if (u == soldier) fail("Unit building colony still in tile");
         }
 
-        found = false;
-        for (WorkLocation loc : colony.getCurrentWorkLocations()) {
-            found |= loc.getUnitList().contains(soldier);
-        }
         assertTrue("Soldier should be in a work location in the colony",
-                   found);
+                   any(colony.getCurrentWorkLocations(),
+                       wl -> wl.contains(soldier)));
         WorkLocation wl = soldier.getWorkLocation();
         assertNotNull("Soldier should be in a work location in the colony", wl);
         assertFalse("Soldier should not be working in central tile",
@@ -408,7 +396,7 @@ public class UnitTest extends FreeColTestCase {
     public void testUnitLosesExperienceWithRoleChange() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
 
@@ -432,7 +420,7 @@ public class UnitTest extends FreeColTestCase {
     public void testOwnerChange(){
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player french = game.getPlayerByNationId("model.nation.french");
@@ -442,24 +430,24 @@ public class UnitTest extends FreeColTestCase {
         assertEquals("Colonist should be dutch", dutch,
                      colonist.getOwner());
         assertEquals("Dutch player should have 1 unit", 1,
-                     dutch.getUnits().size());
+                     dutch.getUnitCount());
         assertEquals("French player should have no units", 0,
-                     french.getUnits().size());
+                     french.getUnitCount());
 
         // change owner
         colonist.changeOwner(french);
         assertEquals("Colonist should be french", french,
                      colonist.getOwner());
         assertEquals("Dutch player should have no units", 0,
-                     dutch.getUnits().size());
+                     dutch.getUnitCount());
         assertEquals("French player should have 1 unit", 1,
-                     french.getUnits().size());
+                     french.getUnitCount());
     }
 
     public void testCarrierOwnerChange(){
         Game game = getStandardGame();
         Map map = getTestMap(ocean);
-        game.setMap(map);
+        game.changeMap(map);
 
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player french = game.getPlayerByNationId("model.nation.french");
@@ -482,9 +470,9 @@ public class UnitTest extends FreeColTestCase {
         assertEquals("Colonist should be dutch", dutch,
                      colonist.getOwner());
         assertEquals("Dutch player should have 2 units", 2,
-                     dutch.getUnits().size());
+                     dutch.getUnitCount());
         assertEquals("French player should have no units", 0,
-                     french.getUnits().size());
+                     french.getUnitCount());
 
         // change carrier owner
         galleon.changeOwner(french);
@@ -493,16 +481,16 @@ public class UnitTest extends FreeColTestCase {
         assertEquals("Colonist should be french", french,
                      colonist.getOwner());
         assertEquals("Dutch player should have no units", 0,
-                     dutch.getUnits().size());
+                     dutch.getUnitCount());
         assertEquals("French player should have 2 units", 2,
-                     french.getUnits().size());
+                     french.getUnitCount());
     }
 
 
     public void testGetMovesAsString() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         Colony colony = getStandardColony(1);
         Unit unit = colony.getUnitList().get(0);
@@ -556,13 +544,12 @@ public class UnitTest extends FreeColTestCase {
         assertEquals(0, treasureTrain.getTransportFee());
     }
 
-
     public void testCopy() {
         Game game = getStandardGame();
-        Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Map map = getTestMap(plains, true);
-        game.setMap(map);
+        game.changeMap(map);
 
+        Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Tile tile1 = map.getTile(6, 8);
         Tile tile2 = map.getTile(6, 9);
 
@@ -571,7 +558,7 @@ public class UnitTest extends FreeColTestCase {
         Goods goods = new Goods(game, merchantman, cottonType, 44);
         merchantman.add(goods);
         
-        Unit other = merchantman.copy(game, merchantman.getClass());
+        Unit other = merchantman.copy(game);
 
         assertFalse(merchantman == other);
         assertEquals(merchantman.getId(), other.getId());
@@ -587,54 +574,20 @@ public class UnitTest extends FreeColTestCase {
         assertEquals(merchantman.getUnitList().get(0).getId(),
             other.getUnitList().get(0).getId());
     }
-
-    public void testElement() {
-        Game game = getStandardGame();
-        Player dutch = game.getPlayerByNationId("model.nation.dutch");
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
-
-        Tile tile1 = map.getTile(6, 8);
-        Tile tile2 = map.getTile(6, 9);
-
-        Unit merchantman = new ServerUnit(game, tile1, dutch, merchantmanType);
-        Unit soldier = new ServerUnit(game, merchantman, dutch, veteranSoldierType);
-        Goods goods = new Goods(game, merchantman, cottonType, 44);
-        merchantman.add(goods);
-
-        try {
-            String xml = merchantman.serialize();
-            Field nextId = Game.class.getDeclaredField("nextId");
-            nextId.setAccessible(true);
-            int id = nextId.getInt(game);
-            nextId.setInt(game, id + 1);
-            xml = xml.replace(merchantman.getId(), Unit.getXMLElementTagName() + ":" + id);
-
-            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
-            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
-            Document doc = dBuilder.parse(new ByteArrayInputStream(xml.getBytes()));
-            Element element = doc.getDocumentElement();
-            Unit clone = new Unit(game, element);
-
-            assertFalse(merchantman == clone);
-            assertFalse(merchantman.getId().equals(clone.getId()));
-            assertEquals(merchantman.getType(), clone.getType());
-            assertEquals(1, merchantman.getUnitCount());
-            assertEquals(1, clone.getUnitCount());
-            assertEquals(44, merchantman.getGoodsCount(cottonType));
-            assertEquals(44, clone.getGoodsCount(cottonType));
-            assertEquals(merchantman.getUnitList().get(0), clone.getUnitList().get(0));
-
-        } catch(Exception e) {
-            e.printStackTrace();
-            fail(e.getMessage());
-        }
-
-    }
-
+    
     public void testDefaultRole() {
         for (UnitType type : spec().getUnitTypeList()) {
             assertNotNull(type.getDefaultRole());
         }
     }
+
+    public void testAttrition() {
+        for (UnitType type : spec().getUnitTypeList()) {
+            if (type == indianConvertType) {
+                assertTrue(type.hasMaximumAttrition());
+            } else {
+                assertFalse(type.hasMaximumAttrition());
+            }
+        }
+    }        
 }
