@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2021  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -174,8 +174,8 @@ public class ColonyPlanTest extends FreeColTestCase {
     private Map buildMap(boolean withBuildRawMat) {
         MapBuilder builder = new MapBuilder(getGame());
         if(withBuildRawMat){
-            builder.setTile(4, 8, forestType);
-            builder.setTile(6, 8, mountainType);
+            builder.setTileType(4, 8, forestType);
+            builder.setTileType(6, 8, mountainType);
         }
         return builder.build();
     }
@@ -183,8 +183,8 @@ public class ColonyPlanTest extends FreeColTestCase {
     /**
      * Should a plan produce a type of goods?
      *
-     * @param plan The <code>ColonyPlan</code> to examine.
-     * @param type The <code>GoodsType</code> to check.
+     * @param plan The {@code ColonyPlan} to examine.
+     * @param type The {@code GoodsType} to check.
      * @return True is there is a work location plan for the required goods
      *     type in the colony plan.
      */
@@ -370,11 +370,11 @@ public class ColonyPlanTest extends FreeColTestCase {
     }
 
     public void testBestImprovements() throws Exception {
-        Map map = getTestMap(plains);
         Game game = getStandardGame();
-        game.setMap(map);
-        Tile tile1 = map.getTile(5, 8);
+        Map map = getTestMap(plains);
+        game.changeMap(map);
 
+        Tile tile1 = map.getTile(5, 8);
         tile1.setType(savannah);
         assertEquals(plow, TileImprovementPlan.getBestTileImprovementType(tile1, grainType));
         assertEquals(plow, TileImprovementPlan.getBestTileImprovementType(tile1, sugarType));
