@@ -78,7 +78,7 @@ public class PioneeringMissionTest extends FreeColTestCase {
                                              colonistType);
         final AIUnit aiUnit = aiMain.getAIUnit(colonist);
         assertNotNull(aiUnit);
-        aiUnit.setMission(null);
+        aiUnit.changeMission(null);
 
         // Check there are improvements to be made.
         aiColony.updateTileImprovementPlans(lb);
