@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2021  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -36,6 +36,7 @@ import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.networking.ChangeSet;
 import net.sf.freecol.server.ServerTestHelper;
 import net.sf.freecol.server.control.InGameController;
 import net.sf.freecol.util.test.FreeColTestCase;
@@ -80,11 +81,11 @@ public class ServerPlayerTest extends FreeColTestCase {
      * test that selling reduces the price for other players.
      */
     public void testMarketRecovery() {
-        Game game = ServerTestHelper.startServerGame(getTestMap());
+        ServerGame game = ServerTestHelper.startServerGame(getTestMap());
         InGameController igc = ServerTestHelper.getInGameController();
 
-        ServerPlayer french = (ServerPlayer) game.getPlayerByNationId("model.nation.french");
-        ServerPlayer english = (ServerPlayer) game.getPlayerByNationId("model.nation.english");
+        ServerPlayer french = getServerPlayer(game, "model.nation.french");
+        ServerPlayer english = getServerPlayer(game, "model.nation.english");
         Market frenchMarket = french.getMarket();
         Market englishMarket = english.getMarket();
         int frenchGold = french.getGold();
@@ -93,15 +94,13 @@ public class ServerPlayerTest extends FreeColTestCase {
         // Sell lightly in the English market to check that the good
         // is now considered "traded".
         Random random = new Random();
-        int m = english.sell(null, silverType, 1);
-        if (m > 0) english.propagateToEuropeanMarkets(silverType, m, random);
+        int m = english.sellInEurope(random, null, silverType, 1);
         assertTrue(englishMarket.hasBeenTraded(silverType));
         int englishAmount = englishMarket.getAmountInMarket(silverType);
 
         // Sell heavily in the French market, price should drop.
-        m = french.sell(null, silverType, 200);
-        if (m > 0) french.propagateToEuropeanMarkets(silverType, m, random);
-        assertEquals(frenchGold + silverPrice * 200, french.getGold());
+        m = french.sellInEurope(random, null, silverType, 200);
+        assertTrue(frenchGold < french.getGold());
         assertTrue(frenchMarket.hasBeenTraded(silverType));
         assertTrue(frenchMarket.getSalePrice(silverType, 1) < silverPrice);
 
@@ -124,12 +123,17 @@ public class ServerPlayerTest extends FreeColTestCase {
         ServerTestHelper.setRandom(mockRandom);
         boolean frenchRecovered = false;
         boolean englishRecovered = false;
+        ChangeSet cs;
         for (int i = 0; i < 100; i++) {
-            igc.yearlyGoodsAdjust((ServerPlayer) french);
+            cs = new ChangeSet();
+            french.csYearlyGoodsAdjust(random, cs);
+            game.sendTo(french, cs);
             if (frenchMarket.getSalePrice(silverType, 1) >= silverPrice) {
                 frenchRecovered = true;
             }
-            igc.yearlyGoodsAdjust((ServerPlayer) english);
+            cs = new ChangeSet();
+            english.csYearlyGoodsAdjust(random, cs);
+            game.sendTo(english, cs);
             if (englishMarket.getSalePrice(silverType, 1) >= silverPrice) {
                 englishRecovered = true;
             }
@@ -146,8 +150,8 @@ public class ServerPlayerTest extends FreeColTestCase {
         Game game = ServerTestHelper.startServerGame(getTestMap());
         
         Map map = game.getMap();
-        ServerPlayer dutch = (ServerPlayer) game.getPlayerByNationId("model.nation.dutch");
-        ServerPlayer french = (ServerPlayer) game.getPlayerByNationId("model.nation.french");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
+        ServerPlayer french = getServerPlayer(game, "model.nation.french");
         InGameController igc = ServerTestHelper.getInGameController();
         Tile tile0 = map.getTile(0, 0);
         Tile tile1 = map.getTile(6, 8);
@@ -177,7 +181,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         Colony colony = getStandardColony();
         colony.addGoods(cottonType, 200);
         ServerPlayer dutch
-            = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+            = getServerPlayer(game, "model.nation.dutch");
         Unit wagonInColony
             = new ServerUnit(game, colony.getTile(), dutch, wagonTrainType);
         Unit wagonNotInColony
@@ -222,7 +226,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         InGameController igc = ServerTestHelper.getInGameController();
 
         ServerPlayer dutch
-            = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+            = getServerPlayer(game, "model.nation.dutch");
         Europe europe = dutch.getEurope();
         Unit privateer1
             = new ServerUnit(game, europe, dutch, privateerType);
@@ -254,51 +258,57 @@ public class ServerPlayerTest extends FreeColTestCase {
 
     public void testCheckGameOverNoUnits() {
         Game game = ServerTestHelper.startServerGame(getTestMap());
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
 
         dutch.setGold(0);
-        assertEquals("Should not have units", 0, dutch.getUnits().size());
-        assertEquals("Should be game over due to no carrier", -1,
+        assertEquals("Should not have units", 0, dutch.getUnitCount());
+        assertEquals("Should not be alive, no carrier",
+                     ServerPlayer.DeadCheck.IS_DEAD,
                      dutch.checkForDeath());
     }
 
     public void testCheckNoGameOverEnoughMoney() {
         Game game = ServerTestHelper.startServerGame(getTestMap());
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
 
         dutch.setGold(10000);
-        assertEquals("Should not be game, enough money", 0,
+        assertEquals("Should be alive, has enough money",
+                     ServerPlayer.DeadCheck.IS_ALIVE,
                      dutch.checkForDeath());
     }
 
     public void testCheckNoGameOverHasColonistInNewWorld() {
         Game game = ServerTestHelper.startServerGame(getTestMap());
         Map map = game.getMap();
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
         dutch.setGold(0);
 
         new ServerUnit(game, map.getTile(4, 7), dutch, colonistType);
-        assertEquals("Should not be game over, has units", 0,
+        assertEquals("Should be alive, has units",
+                     ServerPlayer.DeadCheck.IS_ALIVE,
                      dutch.checkForDeath());
     }
 
     public void testCheckGameOver1600Threshold() {
         Game game = ServerTestHelper.startServerGame(getTestMap());
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
         dutch.setGold(0);
 
         new ServerUnit(game, dutch.getEurope(), dutch, galleonType);
-        assertEquals("Should have 1 unit", 1, dutch.getUnits().size());
-        assertEquals("Should not be game over, not 1600 yet, autorecruit", 1,
+        assertEquals("Should have 1 unit", 1, dutch.getUnitCount());
+        assertEquals("Should not be game over, not 1600 yet, autorecruit",
+                     ServerPlayer.DeadCheck.IS_AUTORECRUIT,
                      dutch.checkForDeath());
 
         new ServerUnit(game, dutch.getEurope(), dutch, colonistType);
-        assertEquals("Should have 2 units", 2, dutch.getUnits().size());
-        assertEquals("Should not be game over, not 1600 yet", 0,
+        assertEquals("Should have 2 units", 2, dutch.getUnitCount());
+        assertEquals("Should not be game over, not 1600 yet",
+                     ServerPlayer.DeadCheck.IS_ALIVE,
                      dutch.checkForDeath());
 
         game.setTurn(new Turn(1600));
-        assertEquals("Should be game over, no new world presence >= 1600", -1,
+        assertEquals("Should be game over, no new world presence >= 1600",
+                     ServerPlayer.DeadCheck.IS_DEAD,
                      dutch.checkForDeath());
     }
 
@@ -306,7 +316,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         Game game = ServerTestHelper.startServerGame(getTestMap(spec().getTileType("model.tile.highSeas")));
         Map map = game.getMap();
         InGameController igc = ServerTestHelper.getInGameController();
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
         dutch.setGold(0);
 
         Unit galleon = new ServerUnit(game, map.getTile(6, 8), dutch,
@@ -318,11 +328,13 @@ public class ServerPlayerTest extends FreeColTestCase {
                      1, galleon.getUnitCount());
         igc.moveTo(dutch, galleon, dutch.getEurope());
 
-        assertEquals("Should not be game over, units between new world and europe", 0,
+        assertEquals("Should be alive, units between new world and europe",
+                     ServerPlayer.DeadCheck.IS_ALIVE,
                      dutch.checkForDeath());
 
         game.setTurn(new Turn(1600));
-        assertEquals("Should be game over, no new world presence >= 1600", -1,
+        assertEquals("Should not be alive, no new world presence >= 1600",
+                     ServerPlayer.DeadCheck.IS_DEAD,
                      dutch.checkForDeath());
     }
 
@@ -330,7 +342,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         Game game = ServerTestHelper.startServerGame(getTestMap());
         Map map = game.getMap();
         InGameController igc = ServerTestHelper.getInGameController();
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
         dutch.setGold(0);
 
         Unit galleon = new ServerUnit(game, dutch.getEurope(), dutch,
@@ -342,37 +354,41 @@ public class ServerPlayerTest extends FreeColTestCase {
                      galleon.getUnitCount());
         igc.moveTo(dutch, galleon, map);
 
-        assertEquals("Should not be game over, units between new world and europe", 0,
+        assertEquals("Should be alive, units between new world and europe",
+                     ServerPlayer.DeadCheck.IS_ALIVE,
                      dutch.checkForDeath());
 
         game.setTurn(new Turn(1600));
-        assertEquals("Should be game over, no new world presence >= 1600", -1,
+        assertEquals("Should not be alive, no new world presence >= 1600",
+                     ServerPlayer.DeadCheck.IS_DEAD,
                      dutch.checkForDeath());
     }
 
     public void testSellingMakesPricesFall() {
-        Game g = ServerTestHelper.startServerGame(getTestMap());
-        ServerPlayer p = (ServerPlayer)g.getPlayerByNationId("model.nation.dutch");
+        Game game = ServerTestHelper.startServerGame(getTestMap());
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
 
-        Market dm = p.getMarket();
-        int previousGold = p.getGold();
+        Market dm = dutch.getMarket();
+        int previousGold = dutch.getGold();
         int price = spec().getInitialPrice(silverType);
-        p.sell(null, silverType, 1000);
+        dutch.sellInEurope(null, null, silverType, 1000);
 
-        assertEquals(previousGold + price * 1000, p.getGold());
+        assertTrue(previousGold < dutch.getGold());
         assertTrue(dm.getSalePrice(silverType, 1) < price);
     }
 
     public void testBuyingMakesPricesRaise() {
         Game game = ServerTestHelper.startServerGame(getTestMap());
-        ServerPlayer player = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
 
-        Market dm = player.getMarket();
-        player.modifyGold(1000000);
+        Market dm = dutch.getMarket();
+        dutch.modifyGold(1000000);
+        int previousGold = dutch.getGold();
         int price = dm.getCostToBuy(foodType);
-        int n = player.buy(new GoodsContainer(game, player.getEurope()),
-                                              foodType, 10000);
-        assertEquals(1000000 - 10000 * price, player.getGold());
+        dutch.buyInEurope(null,
+            new GoodsContainer(game, dutch.getEurope()),
+            foodType, 10000);
+        assertTrue(previousGold > dutch.getGold());
         assertTrue(dm.getBidPrice(foodType, 1) > price);
     }
 
@@ -390,7 +406,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         }
 
         while (price == market.getSalePrice(type, 1)) {
-            player.sell(null, type, 10);
+            player.sellInEurope(null, null, type, 10);
             result++;
         }
         return result;
@@ -415,7 +431,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         GoodsContainer container
             = new GoodsContainer(game, player.getEurope());
         while (price == market.getBidPrice(type, 1)) {
-            player.buy(container, type, 10);
+            player.buyInEurope(random, container, type, 10);
             result++;
         }
         return result;
@@ -428,8 +444,8 @@ public class ServerPlayerTest extends FreeColTestCase {
     public void testDutchMarket() {
 
         Game game = getStandardGame();
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
-        ServerPlayer french = (ServerPlayer)game.getPlayerByNationId("model.nation.french");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
+        ServerPlayer french = getServerPlayer(game, "model.nation.french");
         assertEquals("model.nationType.trade", dutch.getNationType().getId());
         assertTrue(dutch.getNationType().hasModifier(Modifier.TRADE_BONUS));
         assertTrue(dutch.hasModifier(Modifier.TRADE_BONUS));
@@ -438,7 +454,7 @@ public class ServerPlayerTest extends FreeColTestCase {
             int dutchSellAmount = sellUntilPriceDrop(game, dutch, silverType);
 
             Game g2 = getStandardGame();
-            ServerPlayer french2 = (ServerPlayer)g2.getPlayerByNationId("model.nation.french");
+            ServerPlayer french2 = getServerPlayer(game, "model.nation.french");
             int frenchSellAmount = sellUntilPriceDrop(g2, french2, silverType);
 
             assertTrue(dutchSellAmount > frenchSellAmount);
